local Creature = Entity:extend("Creature")

local vowels = {a=1,e=1,i=1,o=1,u=1}

lowerCase = function(word)
    return word
end

startsWithVowel = function(word)
    return vowels[word:sub(1,1)] or lowerCase(word) == "honest"
end

TOTAL_CREATURES = 0
CLONING = false

local getAttackedSound =  function()
    return string.format("hit%s",math.random(1,2))
end

Effect = Entity:extend("Effect")

function Effect:create(k)
    Entity.create(self, k)
    self:grow(.5,.7)
    
    self.parent = k.parent
    
    self.solid = false
    self.x = self.parent.x+(self.parent.w/2-self.w/2)
    self.y = self.parent.y+(self.parent.h/2-self.h/2)
    
    self.depth = DEPTHS.EFFECTS
    self.totalLife = self.life
    self.life = k.time or k.life or k.lifeTime or .3
    
    self.next = k.next
end

function Effect:step(dt)
    self.life = self.life - dt
    self.image_alpha = self.totalLife/self.life
    
    if self.life <= 0 then
        if self.next then
            self.room:nextTurn()
        end
        self:destroy()
    end
end

local  TxObj = Entity:extend('txObj')
local ttw = 360/nn 
function  TxObj:create(k)
    self.w = k.w or ttw
    self.h = self.w
    self.spawnDebris = null
    k.anim = k.anim or "bubble"
    Entity.create(self, k)
    self.text = k.text
    self.color = k.color
    self.scale = nil or k.scale
    self.font = k.font or love.graphics.getFont()
    --self.source = "bubble.png"
    
    self.life = k.life or 1.25*lume.max(.7,(#k.text/10))
    
    local s = self.scale
    self.text = lume.wordwrap(self.text, (self.w*1)/(self.font:getWidth("O")*s))
    self.solid = false
    self.static = true
    
    self.room:must_update(self)
    self.room:must_draw(self)
    
    self.textParts = split(self.text, "\n")
    self.datas = {}
    self.textParts = {self.text}
    for i, x in ipairs(self.textParts) do
        self.datas[i] = Sentence:new({x=self.x-self.offset_x,y=self.y-self.offset_y,w=self.w})
        self.datas[i]:newText(x) if i>1 then end
        self.datas[i].centered = true
    end
    self.centerText = not k.left
    self.ogww = self.w
    self.w = self.w*1.0
    self.ox = self.w-self.ogww
    
    
    self:grow(.5,.95)
end

function TxObj:step(dt)
    if (self.datas[1] and self.textDone) or (not self.datas[1]) then
        self.life = self.life-dt
    end
    
    if self.life <= .5 then
        self.image_alpha = self.life/.5
    end
    
    if self.life <= 0 then
        if self.parent then
            self.parent.text = nil
        end    
        self:destroy()
    end    
end

function TxObj:setText(t)
    self.text = t
    self.textParts = {t}--split(self.text, "\n")
    self.datas = {}
    for i, x in ipairs(self.textParts) do
        self.datas[i] = Sentence:new({x=self.x-self.offset_x,y=self.y-self.offset_y,w=self.w})
        self.datas[i]:newText(x)
    end
end

function TxObj:draw()
    local s = self.scale
    local fo = love.graphics.getFont()
    local f = self.font
    love.graphics.setFont(f)
    local fw = f:getWidth(self.text)*s
    local fh = f:getHeight( )*s
    local x = self.x+(self.w/2-fw/2)*0+self.offset_x*-1*0+self.shake_x
    local y = self.y--+(self.h/2-0/2)+self.offset_y*-1-fh*#self.textParts/2+self.shake_y
    
    for tx = 1, #self.textParts do
        local t = self.textParts[tx]
        local fw = f:getWidth(t)*s
        x = self.x+(self.centerText and (self.w/2-fw/2) or 1)-self.offset_x-self.ox/2
        --[[love.graphics.print(t,
        --self.text
         x, y, 0, s, s)]]
        
        local d = self.datas[tx]
        local pd = self.datas[tx-1]
        local s2 = 10*1*(self.w/tw)
        if d and (not pd or pd.done) then
            d.x = x-(self.w/2-fw/2)--4*s--self.offset_x+s2*s
            d.y = y-self.offset_y+s2*s*1+s*10
            d:update(love.timer.getDelta())
            d.alpha = self.image_alpha
            d:draw()
            
            if not self.datas[tx+1] then
                self.textDone = true
            end
        end
        
        y = y+fh
    end
    
    
    love.graphics.setFont(fo)
end

Creature.zoomIn = function(self, time, scale)
    warn("This causes turns not to move")
    
    time = lume.max(1,time or 1)
    local ox, oy, s = self.room.cameraMan.x, self.room.cameraMan.y, self.room.cameraMan.scale

    self.room.player.takeDamage = null
    self.takeDamage = null
    
    self.room.timer:tween(.7,self.room.cameraMan, {x=self.x, y=self.y, scale=s*(scale or 2)}, "in-quad")
    self.room:after(time,function()
        self.room.timer:tween(.7,self.room.cameraMan, {x=ox, y=oy, scale=s}, "out-quad")
         -- reset takeDamage
        self.room.player.takeDamage = nil
        self.takeDamage = nil
        
        self.room:playNextCreature() -- ?? turns won't move without it
  
    end)
end

Creature.speak = function(self, text, col, scale, b, life)
       
    local flipX = -(self.txFlipX or self.flipX)
    
    if self == self.room.player then
        --flipX = -1
    end
    local www = lume.max(self.w,tw or 100)--160/nn,lume.min(ttw,ttw/((600/nn)/self.w)))
    local t = TxObj:new({
        x = (self.textOffsetX or 0)+ (flipX==-1 and self.x-flipX*www*.5-flipX*
        self.w/2 or self.x-www/2),--0/nn,
        y = self.y+(th-self.h),--(self.textOffsetY or 0)+(self==self.room.player and -160/nn or 0) + self.y+-((self.isBoss and 169 or 0)*(www/ttw))/nn,
        w = www,
        text = text,
        color = col or self.color,
        scale = scale or self.textScale or self.isBoss
        and 2 or 1,
        life = 3,--self.textLife or 2,
        b = b or ""
    })
    t.flipX = flipX
    t.depth = ((self.depth-.21) or (DEPTHS.BULLETS))+love.timer.getTime()/1000000
    
    if self.text then
        self.text:destroy()
    end
    
    if self.talkSound then
        if self._talkies then
            for x, i in ipairs(self._talkies) do
                self.timer:cancel(i)
            end
            self._talkies = nil
        end
        
        self._talkies = {}
        for x = 1, lume.min(#text/5, math.random(2,3)) do
            self._talkies[x] = self.timer:after((x-1)*(math.random(17)/10),self.talkFunction)
        end
    end
    
     
    t:attach_parent(self)
    self.text = t
    t.depth=DEPTHS.EFFECTS--self.depth+5--10000
    t.parent = self
    
    return t
end

function Creature:create(k)
   -- love.graphics.setDefaultFilter("nearest", "nearest")
    TOTAL_CREATURES = TOTAL_CREATURES + 1
    
    local data = k.data or k
    self.data = data
    
    local pm = data.postMake
    data.postMake = nil
    
    Entity.create(self, k)
    
    k.postMake = pm
    
    self.isEntity = true
    self.isCreature = true
    
    self.isPlayer = data.isPlayer
    
    self.talkFunction = function()
        self:play_sound(getValue(self.talkSound), getValue(self.pitches))
    end
    
    local stats = data.stats or data
    self._stats = Stats:new({
        health = getValue(stats.health or stats.life or 10),
        strength = getValue(stats.strength or 2),
        accuracy = getValue(stats.accuracy or 5),
        speed = getValue(stats.speed or 1),
        agility = getValue(stats.agility or 6),
        defense = getValue(stats.defense or 0),
        stealth = getValue(stats.stealth or 0),
        
        flammable = getValue(stats.flammable or 0),
        exp = getValue(stats.exp or 1)
    }, self)
    
    self.clumsiness = data.clumsiness or 0
    self.drain = data.drain or {}
    
    self.base = lume.copy(self.stats)
    self.hasMove = {}
    
    
    self.summoned = {}
    self.poweredByMagic = k.poweredByMagic
    self.immobile = k.immobile
    
    self.isHumanoid = data.isHumanoid
    self.noBones = data.noBones == nil and self.poweredByMagic or data.noBones
    self.boneImage = data.boneImage
    self.skullImage = data.skullImage
    
    self.pitches = data.pitches or {1,.8,.9,1.2,.75,1}--{1,.7,.8,2,.4}
    
    self.buffs = {}
    
    self.seenTiles = {}
    
    self.name = data.name or "???"
    
    self.isClone = data.isClone or CLONING
    
    self.lastSwitched = 0
    
    self.article = data.article or "the"
    self.unspecificArticle = data.unspecificArticle or startsWithVowel(self.name) and "an" or "a"
    self.pronoun = data.pronoun or "it"
    self.pronoun2 = data.pronoun2 or "its"
    self.title = data.title or string.format("%s %s", self.article, self.name)
    self.unspecific = data.unspecific or string.format("%s %s",self.unspecificArticle, self.name)
    self.titlePossessive = data.possessive or data.titlePossessive or string.format("%s's", self.title)
    
    self.flammability = data.flammability or 1
    
    self.turnSpeed = 0
    
    self.health = getValue(data.life or data.health or self.stats.health or 10)
    self.strength = data.strength or self.stats.strength or 1
    self.attack = data.attack or self.strength
    self.defense = data.defense or 0
    
    self.skinAlpha = data.alpha or 1
    
    self.life = nil
    self.totalHealth = self.health
    self.maxHealth = self.totalHealth
    
    self.stats.baseStrength = self.stats.strength
    
    self.sm = self.room.stateMachine
    
    self.sleeping = getValue(data.sleeping or false)
    self.nutrition = getValue(data.nutrition or 10)
    self.metabolism = getValue(data.metabolism or 1/5)
    self.energy = getValue(data.energy or 10)
    self.morale = data.morale or 10
    
    self.thoughtless = data.thoughtless
    
    self.eatRate = data.eatRate or .5
    self.lovesFood = data.lovesFood or false
    
    self.attackRange = data.attackRange or 1
    self.patrolsRoom = data.patrolsRoom or false
    
    self.nimble = data.nimble
    
    self.currentState = getValue(data.state or data.currentState or self.sleeping and "Sleeping" or "Bored")
    self.totalEnergy = getValue(data.maxEnergy or data.totalEnergy or self.energy)
    
    self.inventory = {} -- list of items organized by type {itemData={list of items of that data type}}
    self.inventoryItems = {} -- list of types of items
    self.allInventoryItemsByType = {} -- includes even differentiated items in one list for each item type (as in, mace or wand of of healing)
    
    self.toUpdateItems = {}
    self.inventorySpace = data.inventorySpace or data.storageSpace or 30
    
    self.canPickUp = data.canPickUp or {}
    
    -- keeps a record of weapons used for auto identification
    self.usedItems = {}
    
    self.allItems = {}
    
    self.movementType = k.movementType or "walk"
    
    self.kwargs = k
    
    self.w = k.w or ((tw)/scale)
    self.h = k.h or ((th*2)/scale)
    self:set_box()
    
    self:center()
    --self.debug=1
    if self.w>tw then
        self.offset_x = self.offset_x - (self.w-tw)/2
    end
    
    self._og_offset_y = self.offset_y
    self.offset_y = self.offset_y - (self.h-th)
    
    self.clothesOffset = k.clothesOffset or nil
    
    self.sight = data.sight or 7
    self.vision = data.vision or self.sight
    self.closest = data.closest or 0
    self.furthest = data.furthest or 0
    self.persistence = data.persistence or 5
    
    
    if (data.inventory or data.helmet or data.hat or data.armour or data.glasses or data.weapon or data.equipped) and not CLONING then
        data.inventory = data.inventory or {}
        
        self.canPickItems = true
        self.room.addLog = tostring
        local c = self.cry
        self.cry = tostring
        for x = 1, #data.inventory+4 do
            local d = data.inventory[x] or x==(#data.inventory+2) and getValue(data.helmet or data.hat)
            if x==(#data.inventory+1) then
                d = getValue(data.armour)
            elseif x==(#data.inventory+3) then
                d = getValue(data.equipped or data.weapon)
            elseif x==(#data.inventory+4) then
                d = getValue(data.glasses)
            end
            
            if d then
                local i = self:addEnchantedItem(type(d)=="table" and d[1] or d, type(d)=="table" and d[2], type(d) == "table" and d[3] or d.enchants, true)
                if x>#data.inventory then
                    i:onUse(self)
                    self:equipItem(i)
                end
            end
                    
        end
        self.room.addLog = nil
        self.cry = c
    end
    
    self.canPickItems = data.canPickItems
    self.checkInventory = true
    
    self.intelligent = k.intelligent or false
    
    self.bindedToMaster = k.bindedToMaster
    
    self.static = 1
    
    self.lifebar = toybox.LifeBar(self, {
        thickness = 1,
        x = self.x,
        y = self.y,
        w = 154,
        h = 20,
        alwaysDraw = 4,
        healthColor= "red",
        damagedColor = "black",
        glows = "white",
        font = font8,
        drawText = true
    })
    
    self.hearing = data.hearing or 5
    self.maxHearing = data.maxHearing or 10
    
    self.hostility = data.hostility or 2
    self.dislikeTrigger = data.dislikeTrigger or 3
    self.allyLiking = data.allyLiking or 2
    
    self.selfWorth = data.selfWorth or 3
    
    self.playerDislike = data.playerDislike or self.dislikeTrigger
    self.baseDislike = data.baseDislike or -1
    self.pettiness = data.pettiness or self.dislikeTrigger*.7 -- how much hate to carry
    self.gratitude = data.gratitude or 1
    
    
    self._updateTurn = data.updateTurn
    
    self.stealth = self.stats.stealth--data.stealth or 0
    self.cowardice = data.cowardice or 0
    
    self.alerter = getValue(data.alerter or 0)
    self.called = {}
    
    self.furthestFromLeader = data.furthestFromLeader or 10
    self.bondingLevel = data.bondingLevel or 10
    self.leaderLevel = (data.leaderLevel or data.leaderValue or 0) + TOTAL_CREATURES/10000
    
    self.foodLiking = data.foodLiking or self.lovesFood and (self.bondingLevel-1) or 3
    
    self.food = data.food or {}
    
    self.light = data.light
    
    self.gas = k.gas and lume.copy(k.gas)
    
    if self.gas then
        self.maxGas = k.maxGas or getValue(self.gas.every or 4)
        self.toGas = self.maxGas
    end
    
    
    self.liking = data.liking or {}
    self.liking[self.name] = self.liking[self.name] or self.selfWorth
    
    self.fears  = data.fears or {}
    
    self.gold = data.gold or 0
    
    self.scariness = data.scariness or 0
    
    self.leader = nil
    self.allies = {}
    
    self.team = getValue(data.team)
    
    self.wield_x = data.wield_x or -(data.w or tw)/15
    self.wield_y = data.wield_y or ((data.h or th*2)/2)/10
    
    self.isLiving = data.isLiving == nil or data.isLiving
    self.breathes = data.breathes == nil and self.isLiving or data.breathes
    self.immuneTo = data.immuneTo or {}
    
    self.deathText = data.deathText or (not self.isLiving and "was destroyed") or "was killed"
    self.killedText = data.killedText or (not self.isLiving and "destroyed") or "killed"
    
    local texts = data.texts or self
    
    self.respectKillText = data.respectKillText or texts.respectKill or texts.respectKillText or nil
    self.respectText = data.respectText or texts.respect or texts.respectText or nil
    self.followText = data.followText or texts.follow or texts.followTexts or nil
    
    self.info = k.info or k.description or "A creature"
    self.onDie = k.onDeath or k.onDie
    
    self.blood = k.blood
    self.bloodColor = k.bloodColor
    self.noBlood = k.noBlood == nil and self.poweredByMagic or k.noBlood
     
    local cantRead = data.cantRead
    if type(cantRead) == "number" then
        cantRead = cantRead >= math.random(100)
    end
    
    self.cantRead = getValue(cantRead)
    
    if type(self.cantRead) == "number" then
        self.cantRead = self.cantRead >= math.random(100)
    end
    
    self.drop  = k.drop
    self.drop2 = k.drop2
    self.drop3 = k.drop3
    
    self.dropAll = k.dropAll
    
    
    self.pierce = k.pierce
    
    
    self.offBalance = nil
    
    self.skills = {}
    self.abilities = {}
    
    self.tags = data.tags or {}
    local t = self.tags
    
    t[self.name]     = true
    
    t.magical        = self.poweredByMagic
    t.alive          = self.isLiving
    
    self.height = k.height or nil
    
    self.unsteady = k.unsteady or false
    
    self.confused  = k.confused or 0
    self.paralyzed = k.paralyzed or 0
    self.petrified = k.petrified or 0
    self.poisoned  = k.poisoned or 0
    self.protected = k.protected or 0
    self.blinded   = k.blinded or 0
    self.enslaved  = 0
    
    self.flying    = k.flying or 0
    self.might = k.might or 0
    self.speedy = k.speedy or 0
    
    self.kayoteTime = k.kayoteTime or 0
    self._kayoteTime = self.kayoteTime
    
    self.invisible = k.invisible or k.isInvisible and -1 or 0
    
    self.discordant = k.discordant or 0
    self.effectTime = {.2,.5,.8,.7}
    self.confusedTimer = 0
    self.paralyzedTimer = 0
    self.poisonedTimer = 2
    
    self.canAttack = data.canAttack

    self.getAttack = data.getAttack
    self.ogAttack = self.getAttack
    
    self.equipped = self.equipped or nil
    self.helmet = self.helmet or nil
    self.armour = self.armour or nil
    self.glasses = self.glasses or nil
    
    self.onDrawCard_list = {}
    self.onPlayCard_list = {}
    self.onGetBuff_list = {}
    self.onPlayBuff_list = {}
    self.onStartTurn_list = {}
    self.onEndTurn_list = {}
    self.onSummon_list = {}
    self.onDeath_list = {}
    
    self.eventLists = {
        "MerchandiseDestroyed",
        "MerchandisePickedUp",
        "Steal",
        "GetBuff",
        "PlayBuff",
        "StartTurn",
        "EndTurn",
        "Summon",
        "Death",
        "Damaged",
        "Attack",
        "Kill",
        "Teleport",
        "Move",
        "Dodge",
        "Eat",
        "BeforeEat",
        "Burn"
    }
    
    self.events = {}
    
    for x = 1, #self.eventLists do
        local ev = self.eventLists[x]
        self.events[ev] = {}
        
        
        -- add event: on<Event Name>
        
        self[string.format("on%s", ev)] = function(self, func, val, times, data)
            local list = self.events[ev]
            local data = data or {}
            data.times = data.times or times or -1
            list[#list+1] = {func, val or 1, times or -1, data}
            data.data = {func, val, times or - 1}
            data.times = times or -1
            
            return ev, list[#list]
        end
        
        
        --play the event: play<Event Name>
        
        self[string.format("play%s", ev)] = function(self, ...)
            local done
            local list = self.events[ev]
            for x = 1, #list do
                local data = list[x]
                if data then
                    data[1](data[4], self, data[2], ...)
                    done = data[1]~=null or done
                    data[4].times = data[4].times-1--data[3]-1
                    --data[4].times = data[3]
                    if data[4].times == 0 then
                        table.remove(list, x)
                        if data[4].kill then
                            data[4].kill(data[4], self, data[2], ...)
                        end
                        
                        if data[4].addedBuff then
                            lume.remove(self.buffs, data[4])
                        end
                    end
                end
            end
            
            if self.hasGottenAbility then
                for name, ab in pairs(self.abilities) do
                    local abl = abilities[name]
                    local f = abl.events and abl.events[ev]
                    if f then
                        f(ab, self, ...)
                    end
                end
            end
            
            return done
        end
        
    end
    
    
    self.depth = data.depth or DEPTHS.CREATURES
    self.currentFrame = 0
    
    if data.items then -- obsolete with data.inventory a few lines above
        for x, i in ipairs(items) do
            self:addOtem(i) -- errorz :P
        end
    end
    
    if data.abilities then
        for x, i in pairs(data.abilities) do
            local lvl = getValue(i.level or 1)
            if lvl > 0 then
                self:addAbility(x, i, lvl)
            end
        end
    end
    
    
    if data.skills then
        for x, i in pairs(data.skills) do
            local lvl = getValue(i)
            if lvl > 0 then
                self:addSkill(x, lvl)
            end
        end
    end
    
    if data.postMake then
        data.postMake(self)
    end

end

function Creature:getSight()
    return self.blinded <= 0 and self.sight or lume.max(self.sight - (1*self.blinded), 1)
end

function Creature:getSpeed()
    local p = self.parasite
    return lume.max(.1, self.stats.speed - (p and p.drain.speed or 0))
end

-- not to carry over when cloning
local important = {
    id = true,
    data = true,
    timer = true,
    inventory = true,
    inventoryItems = true,
    allInventoryItemsByType = true,
    toUpdateItems = true,
    inventorySpace = true,
    allItems = true,
    
    world = true,
    room = true,
    sprite = true,
    tile = true,
    
    wielding = true,
    
    isPlayer = true,
    
    x = true,
    y = true
}

-- what to not copy when cloning
basic_clone_limits = {
    getAttack = true,
    equipped = true,
    armour = true,
    helmet = true,
    glasses = true
}

-- to transfer when polymorphing
local polymorphed = {
    inventory = true,
    inventoryItems = true,
    toUpdateItems = true,
    inventorySpace = true,
    allInventoryItemsByType = true,
    allItems = true,
    
    isPlayer = true,
    parasite = true,
    liking = true,
    
    
}

-- to transfer wehn switching minds
local mindSwitch = {
    isPlayer = true,
    liking = true
}


equipFunctions = {
    equipped = "setWeapon",
    armour = "setArmour",
    helmet = "setHelmet",
    glasses = "setGlasses",
}

local pcolor = "hotpink"
local pTexts = {"polymorphed", "transformed"}

function Creature:playStrangeSound(polymorph, ending)
    return self:play_sound("entrance")
end

function Creature:endPolymorph() --if 1 then return self:polymorph(self.polymorphedParent,-1) end
    local cr = self.polymorphedParent
    
    self:playStrangeSound(true, true)
    
    local r = self
    local t = self.tile
    self.isPlayer = nil
    self.room:removeCreature(self)
    self.room:destroy_instance(self)
    self.name = "blahblah"

    self.room:destroy_instance(cr)
    
    self.room.world:add(cr,self.x,self.y,cr.w,cr.h)
    
    cr.polymorphed = false
    self.room:spawn(cr, t)
    cr:set_box()--cr.x,cr.y,cr.w,cr.h)
    self.room:store_instance(cr)
    -- self.room:must_update(cr)
    
    cr:addLog(string.format("%s %s back into %s!", self.title, getValue(pTexts), cr.unspecific))
    
    if self == self.room.player then
        self.room.player = cr
        pl = cr
        self.room:reloadFov()
        self.room:set_target(cr)
    end
    
    if self.controller then
        self.controller:attach(cr)
        cr.controller = self.controller
    end
    
    self.room:addEnchantLight(self.tile, 8, pcolor)
    
    return cr
end

function Creature:polymorph(cr, time, keepTeam)
    if self.polymorphedParent then
        return self:endPolymorph():polymorph(cr, time)
    end
    
    local t = self.tile
    self:playStrangeSound(true)
    
    self.room:destroy_instance(self)
    self.tile.unit = nil
    
    local cr = self.room:spawn(cr, t)
    cr.stats.health = cr.maxHealth*.01*self:healthPercentage()
    cr.polymorphedParent = self
    cr.polymorphTime = time or 2
    --cr.diued = true
    
    for name, val in pairs(polymorphed) do
        cr[name] = self[name]
    end
    
    -- tpdo: make it so allies that see you polymorph recognize you
    if keepTeam then
        cr.team = self.team
    end
    
    if self.controller then
        self.controller:attach(cr)
        cr.controller = self.controller
    end
    
    if self == self.room.player then
        self.room.player = cr
        self.room:reloadFov()
        self.room:set_target(cr)
        pl = cr
        --self.room.set_target = error
    end
    
    self.polymorphed = cr
    
    local all = self.room.creatures
    local i
    --for x = 1, #all do
        --if all[x] == self then
            --[[if (all[#all]==cr) then
                table.remove(all, #all)--cr)
            else
                warn("Polymorph creature")
                lume.remove(all, cr)
            end]]
            
            -- all[x] = cr
            -- cr.polyX = x
        --end
    --end
    
    cr:addLog(string.format("%s %s into %s", self.title, getValue(pTexts), self.unspecific))
    
    lume.remove(self.room.creatures)
    
    t.room:addEnchantLight(t, 8, pcolor)
    
    if self == self.room.currentPlaying then
        self.room:playNextCreature()
    end
    
    return cr
end

function Creature:endMindSwitch()
    local cr = self.mindSwitchedTo
    
    if cr.died then
        self:play_sound("cursed")
        
        self:addLog(string.format("%s feels stuck in it's current body", self.title))
        self.mindSwitchedTo = nil
        return
    end
    
    cr:playStrangeSound(nil, true)
    
    local t = self.tile
    
    if type(cr) == "table" and cr.tile then
        log("sw "..cr.name)
        t = cr.tile
    end
    
    local cr = self.room:spawn(cr, t)
    
    cr.mindSwitchedTo = nil
    cr.mindSwitchTime = false
    
    --cr.diued = true
    local ml = cr:getLiking(self)
    local cl = self:getLiking(cr)
    
    for name, val in pairs(mindSwitch) do
        -- freaky friday
        local old = cr[name]
        cr[name] = self[name]
        self[name] = old
    end
    
    -- so cr will still dislike/know about enemy if switched
    self.liking[cr] = ml
    cr.liking[self] = cl
    
    if self.controller then
        self.controller:attach(cr)
        cr.controller = self.controller
    end
    
    if self == self.room.player then
        pl = cr
        self.room.player = cr
        self.room:reloadFov()
        self.room:set_target(cr)
        self.room.inventoryUser = cr
        
        self.room.inventoryUser = cr
        self.room:closeInventory()
        self.room:reloadInventoryUI(cr)
        
        --self.room.set_target = error
    end
    
    self.mindSwitchedTo = nil
    
    if self == self.room.currentPlaying then
        self.room:playNextCreature()
    end
    
    self:addLog(string.format("%s switched back minds with %s!",self.title, cr.title))
    
end

-- mind switch
function Creature:mindSwitch(cr, time)

    local t = self.tile
    
    
    local cr = cr or self
    while cr == self do
        cr = getValue(self.room.creatures)
        if cr.tile.roomData~=self.tile.roomData then cr=self end
    end
    
    
    
    if type(cr) == "table" and cr.tile then
        log("sw "..cr.name)
        t = cr.tile
    end
    
    if self.mindSwitchedTo or cr.mindSwitchedTo then
        self:addLog(string.format("%s is already in a different body!", self.mindSwitchedTo and self.title or cr.title))
        return
    end
    
    local cr = self.room:spawn(cr, t)
    
    cr.mindSwitchedTo = self
    cr.mindSwitchTime = time or false
    
    --cr.diued = true
    local ml = cr:getLiking(self)
    local cl = self:getLiking(cr)
    
    for name, val in pairs(mindSwitch) do
        -- freaky friday
        local old = cr[name]
        cr[name] = self[name]
        self[name] = old
    end
    
    -- so cr will still dislike/know about enemy if switched
    self.liking[cr] = ml
    cr.liking[self] = cl
    
    if self.controller then
        self.controller:attach(cr)
        cr.controller = self.controller
    end
    
    if self == self.room.player then
        pl = cr
        self.room.player = cr
        self.room:reloadFov()
        self.room:set_target(cr)
        self.room.inventoryUser = cr
        
        self.room.inventoryUser = cr
        self.room:closeInventory()
        self.room:reloadInventoryUI(cr)
        
        cr.playing = true
        --self.room.set_target = error
    end
    
    self.mindSwitchedTo = cr
    
    if self == self.room.currentPlaying then
        self.room:playNextCreature()
    end
    
    self:addLog(string.format("%s switched minds with %s!",self.title, cr.title))
end


function Creature:clone(tile, fullHp, advanced)
    CLONING = true
    local cr = self.room:spawn(self.data, tile or self.room:getFreeTileAround(self.tile, "unit"))
    CLONING = false
    
    self.stats.health = self.stats.health/(fullHp and 1 or 2)
    
    log("cloning "..self.name)
    assert(cr.tile~=self.tile)
    
    for x, i in pairs(self) do
        if not important[x] and (advanced or not basic_clone_limits[x]) then
            cr[x] = type(i)=="table" and i.class and i or lume.copy(i)
            if x == "equipped" or x == "armour" or x == "helmet" or x == "glasses" then
                local item = self[x]
                local i = cr[equipFunctions[x]](cr, i)
            end
        end
    end
  --  assert(not cr.armour)
    assert(cr.tile~=self.tile)
    cr.isClone = true
    cr.isPlayer = false
    cr.done = true
    
    if self.isPlayer then
        cr.title = "Your clone"
        cr.name = "clone"
    end
    
    warn("[CLONING] cloned "..cr.name..","..cr.id)
    
    return cr
end

function Creature:knockback(xdir, ydir, amount, speed)
    if self.isSturdy then
        self:addLog(string.format("%s can't be knocked back!", self.title))
    end
    
    local t = self.tile
    local lastTile
    amount = amount or 1
    local att = amount+(self.stats.strength/2)
    local wall
    
    for x = 1, amount do
        local new = self.room:getTile(t._x+xdir*x, t._y+ydir*x)
        
        
        if new and (not new.unit and (not new.item or not new.item.isObstacle) and not new.solid) then
            lastTile = new
        else
            local att = x+(self.stats.strength/2)
            local continue
            log("knockback_attack")
            if new.unit then
                new.unit:getAttacked(self, att+100, true, true, nil, nil, nil, true)
                if not new.unit.dying then
                    wall = new.unit
                else
                   continue = true
                end
            end
            
            --if new.solid then
                
            if new.item and new.item.isObstacle and new.item.getAttacked then
                new.item:getAttacked(self, att)
                wall = new.item
            end
            
            wall = wall or new
            
            if not continue then
                amount = x
                break
            end
        end
    end
    
    if lastTile then-- SmokeTrail(self)
        speed = (speed or .3)+.05*(amount)
        local att2 = lume.max(.5,amount-(self.stats.strength/2))
        if wall then
            wall.deathText = wall.deathText or "%s %s from impact."
        end
        local amount = self.h > th and 0 or amount
        self.room.timer:tween(speed or .5, self, {angle=self.angle+180*amount}, "in-quad")
        self.room:moveObject(self, 0, 0, speed, lastTile, wall and att2, wall, true, true)
    end
    
end

function Creature:isReallyDead()
    return self.died-- and self.stats.health <= 0
end


function Creature:setWeapon(weapon)
    local i = self:forceAddItem(weapon)
    if i.data.useText == "equip" then
        self:equipItem(i)
    end
end

function Creature:setArmour(armour)
    local i = self:forceAddItem(armour)
    if i.data.useText == "equip" then
        self:equipItem(i)
    end
end

function Creature:setHelmet(helmet)
    local i = self:forceAddItem(helmet)
    if i.data.useText == "equip" then
        self:equipItem(i)
    end
end


function Creature:setGlasses(glasses)
    local i = self:forceAddItem(glasses)
    if i.data.useText == "equip" then
        self:equipItem(i)
    end
end


function Creature:addAbility(ability, kwargs, lvl)
    self.hasGottenAbility = true
    self.firstAbility = self.firstAbility or ability
    
    lvl = lvl or 1
    
    local kwargs = lume.copy(kwargs or {})
    kwargs.level = lvl or kwargs.level
    
    assert(abilities[ability], string.format("%s ability doesn't exist, dingdong!", ability))
    
    local once = abilities[ability].once
    
    if once then
        once(kwargs, self)
    end
    
    self.abilities[ability] = kwargs
end

function Creature:addSkill(skill ,lvl)
    local v = data.skills[skill]
    
    if not v then
        error("No such skill called "..tostring(skill))
    end
    
    v.activate(self, lvl)
    
    -- skill name, level, exp?
    self.skills[skill] = {skill, lvl, 0}
end

function Creature:addClass(class, lvl)
    lvl = lvl or 1
    
    local v = data.classes[class] or class
    
    self.powerClass = v
    v.apply(self, lvl)
    
    if v.weapon then
        local weapon = type(v.weapon) == "function" and v.weapon(lvl) or getValue(v.weapon)
        if weapon then
            local i = self:forceAddItem(weapon)
            if i.data.useText == "equip" then
                self:equipItem(i)
            end
            self.weapon = i
        end
    end
    
    if v.armour then
        local armour = type(v.armour) == "function" and v.armour(lvl) or getValue(v.armour)
        if armour then
            local i = self:forceAddItem(armour)
            if i.data.useText == "equip" then
                self:equipItem(i)
            end
            self.armour = i
        end
    end
    
       
    if v.helmet then
        local helmet = type(v.helmet) == "function" and v.helmet(lvl) or getValue(v.helmet)
        if helmet then
            local i = self:forceAddItem(helmet)
            if i.data.useText == "equip" then
                self:equipItem(i)
            end
            self.helmet = i
        end
    end
    
    if v.glasses then
        local glasses = type(v.glasses) == "function" and v.glasses(lvl) or getValue(v.glasses)
        if glasses then
            local i = self:forceAddItem(glasses)
            if i.data.useText == "equip" then
                self:equipItem(i)
            end
            self.glasses = i
        end
    end
    
    for x, i in ipairs(v.skills or {}) do
        local lvl = type(i) == "table" and (type(i[2]) == "number" or type(i[2]) == "table") and i[2] or lvl
        self:addSkill(type(i) == "table" and (type(i[2]) == "string" and getValue(i) or getValue(i[1])) or getValue(i), getValue(lvl))
    end
    
    for x, i in ipairs(v.abilities or {}) do
        local lvl = type(i) == "table" and (type(i[2]) == "number" or type(i[2]) == "table") and i[2] or lvl
        self:addAbility(type(i) == "table" and (type(i[2]) == "string" and getValue(i) or getValue(i[1])) or getValue(i), {}, getValue(lvl))
    end
    
    return self
end

function Creature:makeLeader(cr, noCheck)
    self.leader = cr
    
    cr.team = cr.team or lume.uuid()
    self.team = cr.team
    
    if cr.isPlayer then
        self.hostility = 2
    end
    
    if not noCheck and not self.liking[obj] then
        self:like(cr, self.bondingLevel)
    end
    
    cr:addAlly(self)
end

function Creature:getEnslavedTo(cr, time)
    if self.leader == cr then
        self:addLog(string.format("%s already follows %s...", self.title, cr.title))
        return
    end
    
    if self.enslaved > 0 then
        self:addLog(string.format("%s is already enslaved!", self.title))
        return
    end
    
    self.oldTeam = self.team
    self.oldLeader = self.leader
    
    self.enslaved = time
    self:makeLeader(cr, true)
    
    self:addLog(string.format("%s is enslaved to %s!", self.title, cr.title))
end

function Creature:removeLeader()
    assert(self.leader)
    self:cry("no leader no more!", "pink", true)--:shake(30,5,30)
    
    self.leader = nil
end

function Creature:tooFarFromLeader()
    if self.leader then
        return lume.distance(self.x,self.y,self.leader.x,self.leader.y)/tw>self.furthestFromLeader
    end
end

local empty = {}

function Creature:fear(obj, level)
    level = level or 0
    local val = 0
    
    if not self.fears[obj] and type(obj) == "table" then
    
        local tags = obj.getTags and obj:getTags() or obj.tags or empty
        
        for x, tag in pairs(tags) do
            local v = self:getFear(tags)
            val = val + v
        end
        
    end
    
    self.fears[obj] = (self.fears[obj] or val)+level
    
    return self.fears[obj]
end

function Creature:getFear(obj)
    return (self.fears[obj] or self:fear(obj))
end

function Creature:isScaredOf(obj) obj.name_tag = obj.scariness..",?????"
    return (lume.max(obj.scariness,0) > (10-self.cowardice)) or (lume.max(self:getFear(obj),0)>(10-self.cowardice))
end

function Creature:like(obj, level)
    level = level or 0
    local val = 0
    -- e:isenemy
    -- if level and level<0 and obj.name == "goblin" and self.data.name~="orc" then error(
    -- (self.liking[obj] or "0 ")..(obj.equipped and obj.equipped.name or "")..","..(obj.liking[self] or "?")..","..(obj.target and obj.target.data.name or "") ..","..(obj.titleo or "")..self.data.name..","..obj.title..","..level) end
    
    if not self.liking[obj] and type(obj) == "table" then
    
        local tags = obj.getTags and obj:getTags() or obj.tags or empty
        n = ""
        for x, tag in pairs(tags) do  
            local v = self:getLiking(x) n = n.." \n"..v..":"..x
            val = val + v
        end
        
    end
    
    -- ogPlayer for when other monsts still think it's a player
    -- add tag to switched monster when they figure it out
    
    self.liking[obj] = (self.liking[obj] or type(obj) == "string" and 0 or
    ((obj.ogPlayer and -self.playerDislike or -self.baseDislike)+
     (self.hostility>0 and obj.team~=self.team and val>0 and 0 or val)
    ))+
        level
    -- if obj.name=="orc" and self.name == "goblin" and self.liking[obj]>-2 then error(tostring(obj.team)..","..tostring(self.team).."("..val..") "..n..","..self.liking[obj]) end
    if level>0 and obj ~= self.leader then
        local l = self.liking[obj]+(self.leader and (self:getLiking(self.leader)/2) or 0)
        if l >= self.bondingLevel then
            self:addLog(string.format("%s started following %s.",self.title,obj.title))
            self:speakText(self.followText or self.respectText, obj)
            self:makeLeader(obj, true)
        end
    end
        
    return self.liking[obj]
end

function Creature:speakText(text, subject)
    subject = subject or self
    
    if subject.isPlayer and math.random() < .3 then
        return
    end
    
    if not subject.isPlayer and math.random() < .8 then
        return
    end
    
    
    local t = getValue(text)
    if t then
        log(t)
        self:speak(string.format(t, subject.name, subject.name, subject.name))
    end
end

function Creature:getLiking(obj)
    return (self.liking[obj] or self:like(obj,0))
end

function Creature:getDisliking(obj)
    return self:getLiking(obj)*-1
end

function Creature:getDislike(obj)
    return self:getDisliking(obj)
end

function Creature:getThrowDistance(item)
    return
        (item and item.throwDistance and item.throwDistance+self.stats.strength*3)
        or self.stats.strength*5
end

function Creature:addAlly(ally)
    ally.leader = self
    self.name = "leadder"
    log("LEADER "..self.name)
    self.liking[ally] = lume.max(self.liking[ally] or 3, -self.baseDislike)
end

function Creature:hasItem(item)
    if type(item) == "string" or not item.isItem then
        return self.allInventoryItemsByType[items[item] or item]
    end
    
    for x = 1, (#(self.inventory[item.data] or {})) do
        if self.inventory[item.data][x] == item then
            return true
        end
    end
end

function kkey() self:teleport(pl,kk) end
function Creature:getItem(item)
    local item = items[item] or item
    return self.allInventoryItemsByType[item] and self.allInventoryItemsByType[item][1]
end

function Creature:getItemOfType(ttype)
    for x, i in pairs(self.inventory) do
        if i[1].itemType == ttype or i[1][ttype] then
            return i[1]
        end
    end
end

function Creature:getAllItemsOfType(ttype)
    local items = {}
    
    for x, i in pairs(self.inventory) do
        if i[1].itemType == ttype or i[1][ttype] then
            for xx = 1, #i do
                items[#items+1] = i[xx]
            end
        end
    end
    
    return items
end

function Creature:getRandomItem()
    return getValue(self.allItems)
end

function Creature:getFoodItem()
    for x = 1, #self.allItems do
        local i = self.allItems[x]
        if self:canEat(i) then
            return i
        end
    end
end

function Creature:forceAddItem(i, a, i2)
    return self:addItem(i,a,i2,true,nil,true)
end

function Creature:addEnchantedItem(itemData, amount, enchants, force)
    return self:addItem(itemData, amount, nil, force, enchants)
end

plural = function(n)
    return string.format("%ss",n)
end

function Creature:canDoPickUp(data)
    local c = self.canPickUp
    return c[data._id] or c[data.itemType or data.type] or c[data.type]
end

function Creature:addItem(i, a, item, f, ...)
    local r = self:_addItem(i, a, item, f, ...)
    if f and not r then
        error("Item not added even on force! ".. inspect(i,1))
    end
    
    return r
end

function Creature:_addItem(itemData, amount, item, force, enchants, fake)
    if type(itemData) == "string" then
        log("adding "..itemData)
    end
    
    local data = items[itemData] or itemData
    local amount = amount or data.isItem and data.amount
    
    if amount then
        
        for x = 1,(amount-1) do
            if self:addItem(item or data.item or data.data or data,nil,nil,force,enchants,fake) then
                if data.isItem then
                    if data.amount and data.amount > 0 then
                        data.amount = (data.amount or amount)-1
                    end
                end
            end
        end
    end
    
    
    local stack = data.stack and data.stack > 1 and data.stack
    
    if data.stack then
        amount = (data.stack)-1
    
        for x = 1,(amount) do
            log("stacking")
            if self:addItem(item or data.item or data.data or data,nil,nil,force,enchants,true) then
            
                if data.isItem then
                    if data.stack then
                        data.stack = data.stack - 1
                    end
                end
            end
        end
    end
    
    if ((not self.canPickItems and not self.isPlayer) and not force) and not self:canDoPickUp(data) then
        return
    end
    log("can pixkup")
    
    if data.noPickup and not force then
        return
    end
    
    local onFloor = data.isItem
    local item = data.isItem and data or Item:new(data, true)
    item.space = lume.min(item.space or 1,1)
    
    local sp = self.inventorySpace
    local isSpace = sp >= item.space or item.space == 0
    
    if item and item.isWeapon and not item.isAmmo then
        item:makeDifferentFromStack()
    end
    
    if item.isCarried then error(item.id) end
    
    if onFloor then
        data = item.spaceData or data.data
        
        -- remove item from world if space
        -- if not space then return
        if not isSpace and self.checkInventory and not force then
            log("No space! "..sp..", but item takes "..item.space)
            if self.isPlayer then
                self:addLog(string.format("&colors.red %s has no space for %s!", self.title, item.title))
                self.room.camera:shake(25,.3,65)
            else
                self:addLog(string.format("%s failed to pickup %s!", self.title, item.title))
            end
            
            return
        end
        
        if item.tile then
            local pLock = item.tile.pedestalLocked
            
            if pLock and pLock[item.tile] == item then
                self.room:addLog("You hear a locking sound")
                pLock.unlocked = false
                pLock.taken = item
                item.tile.takenPedestalItem = item
            end
                
            item.tile.item = nil
            item.tile = nil
        end
        self.room.items[item] = nil
        
        local burn = item:removeBuff("burn")
        if burn then
            self:applyBuff("burn", burn.times, burn.data[2])
        end
        log("item was on floor")
    
    elseif not space then
        -- force out an object
        self:removeItem(self.inventory[self.inventoryItems[#self.inventoryItems]][1])
    end
    
    
    item.isCarried = true
    
    -- reset attacked variable used in AI throwing attack
    item.attacked = nil
    
    -- remove item from world
    item.room:destroy_instance(item)
    
    item.stack = nil
    if item.spaceData then log("[ITEM] item has spacedata") end
    data = item.spaceData or data
    
    if not self.inventory[data] then
        log("new item kind")
        table.insert(self.inventoryItems, data)
    end
    
    local ogData = items[item._id] or error("No origional data for item "..tostring(item._id)..", "..inspect(item,2))
    
    item.ogData = ogData
    
    self.inventory[data] = (self.inventory[data] or {})
    self.allInventoryItemsByType[ogData] = (self.allInventoryItemsByType[ogData] or {})
    
    data.count = (data.count or 0)+(love.timer.getTime()/10000000)
    
    table.insert(self.inventory[data], item)
    table.insert(self.allInventoryItemsByType[ogData], item)
    
    --assert(item.data == data, data._id)
    if item.updateTurn then
        self.toUpdateItems[#self.toUpdateItems+1] = item
    end
    
    self.allItems[#self.allItems + 1] = item
    
    item.added = self.time_alive
    
    self.inventorySpace = self.inventorySpace - item.space
    item.user = self
    
    if item.shopkeeper and not item.shopkeeper.died then
        item.shopkeeper:playMerchandisePickedUp(item)
    end
    
    if item.chik then error() end
    item.color = item.itemColor
    item:onCarry(self)
    
    if (self.isPlayer or self.seen) and not fake then
        self:addLog(string.format("%s picked up %s%s", self.title, stack and string.format("%s ",stack) or "",stack and plural(item.name) or item.name), self.isPlayer and "gold")
        self:cry(string.format("%s%s",stack and string.format("%s ",stack) or "",stack and plural(item.name) or item.name), item.pcolor or item.itemColor or item.color or "gold")
        if self.isPlayer then
            self:play_sound((item.name == "gold" or item.isGold) and "coins" or "collect")
        end
    end
    
    if self.isPlayer and not self.isClone then
        if item.isShooter or item.isWand or (onFloor and item.utility < 0 and not item.isScroll and not item.isPotion) then
            self.room.putItemInQuickSlot(item)
        end
        
        self.room:reloadInventoryUI(self)

    end
    self:wieldItem(item,"center")
    
    
    
    if not onFloor then
        if enchants then
            item:enhance(getValue(enchants))
        end
        
    end
    
    return item
end

function Creature:equipItem(item)
    if type(item) ~= "table" then
        error(string.format("No such item as %s", item))
    end
    
    self:unwield(item)
    
    if item.isHelmet then
        return self:equipHelmet(item)
    elseif item.isArmour then
        return self:equipArmour(item)
    elseif item.isGlasses then
        return self:equipGlasses(item)
    end
    
    if self.equipped then
        local old = self.equipped
        old.equipped = false
    end
    
    self.equipped = item
    item.equipped = self
    
    self:reloadWieldPos(item)
    
    self:wield(item, "center")
end

function Creature:reloadWieldPos(item)
    item.w = item._oggw or item.w
    item.h = item._oggh or item.h
    
    item.wield_x = tw/3--(tw or item.w)/2--tw/8
    
    local pl = pl or self
    local cl = (self.clothesOffset or 0)*2
    
    -- item.debug = 1
    
    item.wield_y = (self.h/(th*2))+cl + (self.h/3-item.h/2)--self.h/2 - (self.offset_y-pl.offset_y) - self.h/4
    if njj  then 
    item.wield_y = self.h/2-(th/2-item.h/2) end--drawe
    
    
    local sc = lume.min(200,self.w-(tw))
    
    item._oggw = item._oggw or item.w
    item._oggh = item._oggh or item.h
    
    
    item.w = item._oggw+sc
    item.h = item._oggh+sc
end

function Creature:unequipItem(item)
    if item.isHelmet then
        return self:unequipHelmet(item)
    elseif item.isArmour then
        return self:unequipArmour(item)
    elseif item.isGlasses then
        return self:unequipGlasses(item)
    end
    
    if self.equipped then
        self:unwield(self.equipped)
        local old = self.equipped
        old.equipped = false
        self.equipped = false
    end
end


function Creature:unequipHelmet(item)
    if self.helmet then
        local old = self.helmet
        old.equipped = false
        self.helmet = nil
        self:removeEquippedStats(old)
        return
    end
    
    self.helmet = item
    item.equipped = self
    if item.sprite then
        item.sprite:syncTo(self.sprite)
    end
end

local feelTexts = {
    speed = {
        more = {"slighty faster", "a little quicker"},
        less = {"slower", "heavier"}
    },
    strength = {
        more = {"stronger"},
        less = {"weaker"}
    },
    sight = {
        more = {"a bit more aware"},
        less = {"a little less aware"},
    },
    agility = {
        more = {"more agile", "more flexible"},
        less = {"more encumbered", "less agile"},
    },
    accuracy = {
        more = {"more accurate"},
        less = {"less accurate"}
    }
}

function Creature:unequipArmour(item)
    if self.armour then
        local old = self.armour
        old.equipped = false
        self.armour = nil
        self:removeEquippedStats(old)
        return
    end
    
    self.armour = item
    item.equipped = self
    if item.sprite then
        item.sprite:syncTo(self.sprite)
    end
end


function Creature:unequipGlasses(item)
    if self.glasses then
        local old = self.glasses
        old.equipped = false
        self.glasses = nil
        self:removeEquippedStats(old)
        return
    end
    
    self.glasses = item
    item.equipped = self
    if item.sprite then
        item.sprite:syncTo(self.sprite)
    end
end


function Creature:addEquippedStats(item, neg)
    if not item.stats then
        return
    end
    
    local feels = {}
    
    for x, i in pairs(item.stats) do
        local s = x == "sight" and self or self.stats
        i = neg and (-i) or i
        
        if not s[x] then
            error("No stat "..x.." for "..(s == self and "self" or "stats table"))
        end
        
        s[x] = s[x] - i
        if feelTexts[x] and i ~= 0 then
            local t = getValue(feelTexts[x][i>0 and "more" or "less"])
            if t then
                feels[#feels+1] = {t, i>0 and "lime" or "red"}
            end
        end
    end
    
    local txts = ""
    for x = 1, #feels do
        local f = feels[x]
        local n = string.format("&colors.%s %s %s", f[2], f[1], x==#feels and "" or x==(#feels-1) and "and" or ", ")
        txts = string.format("%s %s", txts, n)
    end
    
    local txt = string.format("%s feel %s",self.title, txts)
    if txts ~= "" and self.isPlayer then
        self:addLog(txt)
    end
    
end

function Creature:removeEquippedStats(item)
    if not item.stats then
        return
    end
    
    return self:addEquippedStats(item, true)
end


function Creature:equipHelmet(item)
    if self.helmet then
        local old = self.helmet
        old.equipped = false
        
        self:removeEquippedStats(old)
    end
    
    self.helmet = item
    item.equipped = self
    if item.sprite then
        item.sprite:syncTo(self.sprite)
    end
    
    self:addEquippedStats(item)
end


function Creature:equipArmour(item)
    if self.armour then
        local old = self.armour
        old.equipped = false
        
        self:removeEquippedStats(old)
    end
    
    self.armour = item
    item.equipped = self
    if item.sprite then
        item.sprite:syncTo(self.sprite)
    end
    
    self:addEquippedStats(item)
end


function Creature:equipGlasses(item)
    if self.glasses then
        local old = self.glasses
        old.equipped = false
        
        self:removeEquippedStats(old)
    end
    
    self.glasses = item
    item.equipped = self
    if item.sprite then
        item.sprite:syncTo(self.sprite)
    end
    
    self:addEquippedStats(item)
end

function Creature:wieldItem(item)
    self:reloadWieldPos(item)
    self.wieldedItem = item
    return self:wield(item, "center")
end

function Creature:reloadItem(item)
    self:removeItem(item, true)
    self:addItem(item, nil, nil, true)
end
function met() self:spawnItem("meat",pl.tile) end--tele--e:ise
function Creature:removeItem(item, destroy, dropped)
    item = items[item] or item
    log("[INVENTORY] removing "..item._id)
    
   
    
    if self.inventory[item] then
        item = getValue(self.inventory[item])
    end
    
    self:unwield(item)
    item.isCarried = false
    log("[DROPPED] "..item.name)
    
    local amount = self.inventory[item.spaceData or item.data]
    
    if not amount then local i = {} for ii,xz in pairs(self.inventory) do i[ii.name..","..tostring(ii)]=#xz end
        error(inspect(i)..","..item.data.name.." ( "..(item.spaceData and "space data true" or "no space data")..")")
    end
    
    --amount = amojunt or nil
    
    lume.remove(amount or {}, item)
    lume.remove(self.allInventoryItemsByType[item.ogData] or error("Cant remove "..item.name), item)
    
    self.inventorySpace = self.inventorySpace + item.space
    
    -- **not done to allow the item to wrap up events
    -- item.user = nil
    
    item.oldUser = self
    
    if item.updateTurn then log(item.name..",".." gooone")
        lume.remove(self.toUpdateItems, item)
    end
    
    lume.remove(self.allItems, item)
    
    if item.quickslot then
        item.quickslot.item = self.inventory[item.data][1] or nil
        if item.quickslot.item then
            item.quickslot.item.quickslot = item.quickslot
        else
            local fs = self.room.freeQuickSlots
            fs[#fs+1] = item.quickslot
        end
        item.quickslot = nil
    end
    
    if #amount == 0 then
        lume.remove(self.inventoryItems, item.data)
        self.inventory[item.data] = nil
    end
    
    if item.equipped then
        item.equipped:unequipItem(item)
    end
    
    if not destroy then
        -- drop item
        self.room:spawnItem(item, self.tile, dropped)
    
    end
    
    
    if self == self.room.player then
        self.room:reloadInventoryUI(self)
    end
end

-- add an event (played with "play<Event>" at appropriate time)
function Creature:onEvent(eventName, event, ...)
    return self[string.format("on%s",eventName)](self, event, ...)
end
monke = function()
    self:spawn("monkey",pl.tile)
end

local old_play_sound = Creature.play_sound

function Creature:_play_sound(...)
    local sound = old_play_sound(self, ...)
end

function Creature:get_volume()
    if self.isPlayer then
        return .5
    end
    
    if self.isSeen then return .2 end
    
    return .1
end

function Creature:applyBuff(buff, time, value, enchants)
    if type(buff) == "table" and not data.buffs[buff._id or "none"] then
        return self:applyBuff(buff[1] or buff.name or buff.id,
        buff[2] or buff.time, buff[3] or buff.value or buff.val, buff[4] or buff.enchants)
    
    end
    
    buff = data.buffs[buff] or buff
    
    if self:isImmuneTo(buff) then
        return
    end
    
    local eb = buff.enchants
    buff.enchants = enchants or buff.enchants or 1
    
    value = value or buff.getValue(1) or 1
    time = time or buff.getTime(1) or 1
    
    if buff.once then
        buff.once(buff, self, value, time)
    else
        for x = 1, #self.buffs do
            local b = self.buffs[x]
            if b.id == buff then
                b.times = b.times + (time or 1)
                b.data[2] = b.data[2]+value
                if time < 0 then
                    b.times = time
                end
                
                return true
            end
        end
            
    end
    
    -- todo: ignore if already buffed
    if buff.text then
        self:addLog(string.format(buff.text, self.title))
    elseif buff.continue ~= "" then
        self:addLog(
            string.format("%s is %s.", self.title, buff.continue or buff.verb or buff._id),
                        self.room.player:isAlly(self) and "red" or self.isPlayer and "red" or "lime"
                    )
    end
    
    if buff then
        local data = lume.copy(buff)
        data.addedBuff = true
        data.id = buff
        self.buffs[#self.buffs+1] = data
        buff.isBuff = true
        data.times = time
        local event, eventData = self:onEvent(
            buff.eventType or self == self.room.player and "EndTurn" or "StartTurn", 
            buff.action or null,
            value,
            time,
            data
        )
        data.event, data.eventData = event, eventData
        
        buff.enchants = eb
    end
    
    return true
    
    
end

function Creature:removeBuff(data, n)
    if #self.buffs == 0 then return end
    
    if type(data) == "string" then
        local r
        for x = 1, #self.buffs do
            local buff = self.buffs[x]
            if buff and (data == true or buff._id == data) then
                r = self:removeBuff(buff, x)
            end
        end
        
        return r
    end
    
    if self.events[data.event] then
        lume.remove(self.events[data.event], data.eventData)
    else
        warn("[REMOVE BUFF] No event for "..data._id.." at "..inspect(data,2))
    end
    
    -- n should save time, lume.remove is slower but needed for unknown index
    local f = (n and table or lume)
    f.remove(self.buffs, n or data)
    
    return data
end

function Creature:drawBurn(alpha)
    
    local source = "setpieces/flame"
    local maxFrames = 4
    if true then
        maxFrames = 5
        source = "setpieces/fire"
    end
    
    local currentFrame = math.floor(self.currentFrame%maxFrames)
    local cr = currentFrame
    
    while cr > (maxFrames-1) do
        cr = 0+(cr-(maxFrames))
    end
    while cr < 0 do
        cr = 0
    end


    local source = string.format("%s/%s.png",source, cr)
    local img = game:getAsset(source)
    local _w, _h = resizeImage(img, tw, th)
    local ww, hh = img:getDimensions()
    local xx = tw/2--gas.xx*0
    local yy = 5
    local tile = self
    local r,g,b,a = love.graphics.getColor()
    set_color(1,1,1,alpha or 1)
   -- draw_rect("line",tile.x+xx,tile.y+th/2,tw,th)
    lg.draw(img, tile.x+xx, tile.y+math.random(-yy,yy)+th/2, math.rad(0 or self.angle), _w, _h, ww/2, hh/2)
    
    
    love.graphics.setColor(r,g,b,a)
end


function Creature:drawDiscord(alpha)
    
    local source = "setpieces/flame"
    local maxFrames = 4
    if true then
        maxFrames = 5
        source = "setpieces/grumble"
    end
    
    local currentFrame = math.floor(self.currentFrame%maxFrames)
    local cr = currentFrame
    
    while cr > (maxFrames-1) do
        cr = 0+(cr-(maxFrames))
    end
    while cr < 0 do
        cr = 0
    end


    local source = string.format("%s/%s.png",source, cr)
    local img = game:getAsset(source)
    ttw = tw*.7
    local _w, _h = resizeImage(img, ttw, ttw)
    local ww, hh = img:getDimensions()
    local xx = tw/2-(tw-ttw)/2--gas.xx*0
    local yy = 5
    local tile = self.tile
    local r,g,b,a = love.graphics.getColor()
    
    local aa = (self.light_alpha+(alpha or 1))/2
    local col = .8
    set_color(col,col,col,aa)

    local c = (self.clothesOffset or 0)
    local height = self.height or self.h==250 and th/4 or 0
   -- draw_rect("line",tile.x+xx,tile.y+th/2,tw,th)
    lg.draw(img, self.x+xx, self.tile.y+th+math.random(-yy,yy)*1-(self.h-height)+c, math.rad(0 or self.angle), _w, _h, ww/2, hh/2)
    -- draw_rect("line",self.x,self.y, self.w,self.h)
    
    
    love.graphics.setColor(r,g,b,a)
end

spa = function() pl.stats.health=100 self:spawn("angel",pl.tile) self:spawn("kobold",pl.tile) end

function Creature:getTarget(type, myTeam)
    error("Not to be used, ding dong.")
    if not type then
        local crs = lume.keys(self.room.creatures)
        table.sort(crs,depth)
        for _, cr in ipairs(crs) do
            if ((myTeam and cr.team == self.team) or (not myTeam and cr ~= self.team)) then
                return cr
            end
        end
        
    elseif type == "far" or type == "furthest" then
        local crs = lume.keys(self.room.creatures)
        local dis, targ
        for _, cr in ipairs(crs) do
            if ((myTeam and cr.team == self.team) or (not myTeam and cr ~= self.team)) then
                local newDis = lume.distance(self.x+self.w/2, self.y+self.h/2, cr:getCenter())
                if not dis or newDis > dis then
                    targ = cr
                    dis = newDis
                end
            end
        end
        
        return targ
    
    elseif type == "close" or type == "closest" then
        local crs = lume.keys(self.room.creatures)
        local dis, targ
        for _, cr in ipairs(crs) do
            if ((myTeam and cr.team == self.team) or (not myTeam and cr ~= self.team)) then
                local newDis = lume.distance(self.x+self.w/2, self.y+self.h/2, cr:getCenter())
                if not dis or newDis < dis then
                    targ = cr
                    dis = newDis
                end
            end
        end
        
        return targ
    
    elseif type == "random" then
        local crs = lume.keys(self.room.creatures)
        local cr = self
        local count = 0
        while cr == self do
            cr = getValue(crs)
            if cr~=self and ((myTeam and cr.team == self.team) or (not myTeam and cr ~= self.team)) then
                return cr
            else
                cr = self
            end
            
            count = count + 1
            if count > 50 then
                return
            end
            
        end
    end

end

function spop() self:after(.1,function() local k = toybox.room:spawn("kobold",pl.tile) k.suss=1 k:addClass("scribe") ko = k end) end

function chkk () local l for x, c in ipairs(self.creatures) do if ko==c then l=x break end end if not l then pl.w=pl.w*2 else warn("suss is true at "..l) end end

function Creature:recordItemDrop(item)
    self.dropped = item
    self.droppedItem = (self.droppedItem or 0) + 1
end

function Creature:doDrop(dropTable, looter)
    local spawned
    
    if dropTable then
        local itemm
        local name = getItemByWeight(dropTable)
        local item = lume.copy(dropTable[name])
        
        if type(item[2]) == "table" and type(item[2][1]) == "number" or type(item[2])=="number" then
            for x = 1, getValue(item[2]) do
                itemm = self.room:spawnItem(getValue(item[3] or name), self.tile)
                self:recordItemDrop(itemm)
                spawned = true
            end
        else
            if type(item[2]) ~= "string" then
                item[2] = {item[2] or items[name] and name}
            end
            
            for x = 1, #item[2] do
                itemm = self.room:spawnItem(getValue(item[2][x]), self.tile)
                self:recordItemDrop(itemm)
                spawned = true
            end
        end
        
        for x = 1, looter.master and 2 or 1 do
            local looter = x == 1 and looter or looter.master
            local done
        
            if not spawned and looter and looter.isCreature and looter.huntingFood then
                local food 
                for x, i in pairs(looter.food) do
                    if dropTable[x] then
                        food = x
                    end
                end
            
                if food then
                    itemm = self.room:spawnItem(food, self.tile)
                    self:recordItemDrop(itemm)
                end
            end
        
        
            if itemm and looter and looter.huntingFood and looter:canEat(itemm) then
                looter.foodTarget = itemm
                done = true
            end
            
            if done then
                break
            end
        end
    end
end

function Creature:isSubmerged()
    local w = self.tile and self.tile.water
    
    if w then
        return w.isDeep and not self:isFlying()
    end
    
    return false
end


local durabilities = {.1, .3,.3,.3,.2,.5,.2,.9}

function Creature:die(attacker, attack, immediate, enemy)
    if self.isAlreadyDead then
        error("removement problem with "..self.name)
    end
    
    self.isAlreadyDead = true
    
    local x, y, w, h = self:get_center()
    
    self.__beforeDeathUpdateTurn = self.__beforeDeathUpdateTurn or self.updateTurn
    self.__beforeDeathPlayFunction = self.__beforeDeathPlayFunction or self.play
    
    self.updateTurn = null
    self.play = null
    
    -- <self.died = true> moved to bottom to allow addLog to be called propally
    -- self.died = true
    
    for x = 1,#self.summoned+3 do
        local mon = self.summoned[x]
        if mon and mon.poweredByMagic and not mon.died then
            mon:die()
        end
    end
    
    if self.master then
        lume.remove(self.master.summoned, self)
    end
    
    local tu = self.tile.unit
    self.tile.unit = nil
    
    self.droppedItem = 0
    
    if not self.isClone then
    
    self:doDrop(self.drop, enemy)
    self:doDrop(self.drop2, enemy)
    self:doDrop(self.drop3, enemy)
    
    if self.weapon then
        local d = self.weapon:getDurabilityPercentage()
        if d > 50 or math.random(49)<d or self.dropAll then
            self.weapon.durability = self.weapon.durability*(d<30 and 1 or lume.min(d/100,getValue(durabilities)))
            
            self.dropped = self.weapon
            self.droppedItem = (self.droppedItem or 0) + 1
            
            self:removeItem(self.weapon)
        end
    end
    
    local max = getValue({1,3,4,0,3,3,2})
    
    for data, list in pairs(self.inventory) do
        if max > 0 or self.dropAll then
            local i = getValue(list)
            
            if not i.dead and not i.noDrop then
                self.dropped = i
                self.droppedItem = (self.droppedItem or 0) + 1
            
                self:removeItem(i)
                max = max - 1
            end
        else
            break
        end
    end-- grund feel less gidy when yiur geting something all the time
    
    for data, list in pairs(self.inventory) do
        local i = getValue(list)
        if i.isImportant or i.important or i.stolen or i.onceOnGround then
            for xx = 1, #list do
                local i = list[xx]
                self.dropped = i
                self.droppedItem = (self.droppedItem or 0) + 1
            
                self:removeItem(i)
            end
        end
    end
    
    if self.droppedItem == 1 then
        self:addLog(string.format("%s dropped %s!",self.title,self.dropped.unspecific))
    elseif self.droppedItem > 0 then
        self:addLog(string.format("%s dropped some items!",self.title))
    end
    
    self.droppedItem = 0
    
    end
    
    self.tile.unit = tu
    
    r = game.room
    
    if self.blood ~= false and not self.noBlood then
        DROP_DEBRIS = true
        self:spawnDebris(self.bloodColor or "red")
        DROP_DEBRIS = false
    end
    
    --Explode(x,y,3)
    if self.isSeen then
       --self:shake(30,.3,40)
    end
    log("died lol")
    self.died = true
    -- dying
    
    if enemy and enemy.isCreature and not self.isPlayer then
        local function respect(unit)
            log("respect")
            if unit ~= enemy then
                unit:like(enemy, unit.gratitude)
                unit:speakText(unit.respectKillText or unit.respectText, enemy)
                unit:addLog(string.format("%s respects %s kill.", unit.title, enemy.titlePossessive))
            end
        end
        
        log("start t")
        NO_COROUTINE = true
        self:getEnemies(respect)
        NO_COROUTINE = false
    end
    
    log("next")
    

    
    local function doDie()
        if self.onDie then
            self:onDie()
        end
        
        self.room:removeCreature(self)
        
        if self.parasite then
            self:removeParasite()
        end
        
        attacker = attacker or {}
        self:playDeath(attacker, attack)
        if attacker.playKill then
            attacker:playKill(self)
        end
        
        if (not attacker.playKill or attacker ~= enemy) and enemy and enemy.playKill then
            enemy:playKill(self, attacker)
        end
        
        self:spawnBones()
            
        
        log("removed dead bod")
    end
    
    if immediate then
        doDie()
    else
        self.room.timer:tween(.2, self, {image_alpha=0}, "out-quad", doDie)
    end
    
    
end

local function bone_update(self)
    self.color = not self.iusPlayer and self.tile.colorUsed
    if self.tile:canBeSeen() then
        self.image_alpha = .4
    else
        self.image_alpha = self.tile.seen and .1 or 0
    end
    self.image_alpha = self.isPulayer and 1 or not self.tile.seen and 0 or (self.tile.image_alpha+.8)/2
end

local function boneCheckDrawDepth(self, other)
    if other.isTile and other.solid then
        return self
    elseif other.isTile then
        return other
    end
    
    if not other.isBone then
        return other
    end
    
    return nil
end

function Creature:spawnBones(noSkull)
        
        self:play_sound("squash")--string.format("splash_%s",math.random(1,2)))
        local source = string.format("effects/scorch_%d.png",math.random(1,2))

        local t = self.tile or self.room:getTileP(self.x,self.y)
        if t then
        
            local i = t.sicorch_image or t:add_image(source)
            t.scorch_image = i
            t.scorch_source = i.source
            
            i.color = {1,1,1,.8}
            t.scorch_timer = self.room.timer:tween(45,i.color,{[4]=.01})
        end
        
        if not self.noBlood and math.random()>.7 and nil then

            self.tile:addPuddle({
                color = self.bloodColor or "red",
                uuid = "blood",
                value = math.random(40,100)/100,
                string.format("effects/stain/stain_%s.png", math.random(1,4)),
                data = {}
            })
            
        end
        
        if self.noBones then
            return
        end
        
        local tiles = {self.tile, self.tile}
        for x = -1,1 do
            for y = -1,1 do
                local t = self.room:getTile(self.tile._x+x, self.tile._y+y)
                if t and not t:isSolid() then
                    tiles[#tiles+1] = t
                end
            end
        end
        
        local d = DEPTHS.WALLS-1
        
        self.room.playerBones = self.room.playerBones or {}
        
        for b = 1, math.random(3,5) do
            local m = math.random(90,120)/100
            local skull = not noSkull and ((self.isPlayer or self.isHumanoid or self.skullImage) and b == 1 and getValue(self.skullImage or "items/skull.png"))
            
            local nn = .75
            
            if skull then
                nn = 1
            end
            
            local bone = toybox.NewBaseObject({
                source = skull or getValue(self.boneImage) or "items/bone.png",
                w = 125*nn*m, h = 125*nn*m,
                x = self.tile.x, y = self.tile.y,
                solid = false,
                static = true,
                image_alpha = .6
            })
            
            if self.isPlayer then
                self.room.playerBones[b] = bone
                bone.isPlayer = true
            end
            
            -- self.room:must_draw(bone)
            self.room:must_update(bone)
            bone.depth = d -- .01
            bone.draw = bone_update
            bone.isBone = true
            bone.checkDrawDepth = boneCheckDrawDepth
            --bone.debug=1
            
            local t  = getValue(tiles)
            local tx = math.random(t.x, t.x+t.w)
            local ty = math.random(t.y, t.y+t.h)
            
            bone.tile = t
            if t:isChasm() then
                bone.__draw = null
            end
            
            self.room:tween(.2, bone, {x=tx, y=ty, angle=math.random(-360,360)*2, image_alpha=.4}, "in-quad")
        end
end


function Creature:boostSpeed(val)

end

function Creature:addLog(...)
    if self.isSeen and not self.died then 
            local x, y, w, h = self.room.camera:getVisible()
            
            if self.x > x+w or self.x < x or self.y > y+h or self.y < y then
                self.isSeen = false
                warn(self.name..","..x..","..y..","..w..","..h..": "..self.x..","..self.y.."is outside but said seen")
                return
            end
        return self.room:addLog(...)
    end
end

    

function Creature:makeImmuneTo(obj, setFalse)
    local i = self.immuneTo
    local numVal = type(setFalse) == "number"
    local val = numVal and setFalse or not setFalse
    
    if type(obj) == "table" then
        i[obj.name] = type(i[obj.name]) == "number" and numVal and (i[obj.name]+val) or val
        --i[obj.class] = val
    else
        i[obj] = type(i[obj]) == "number" and numVal and (i[obj]+val) or val
    end
end

function Creature:isImmuneTo(obj)
    local i = self.immuneTo
    return i[obj] or i[obj.name] or (obj._id and i[obj._id]) or (obj.class and i[obj.class.name])
end

function Creature:heal(n)
    self.stats.health = lume.min(self.stats.health+n, self.maxHealth)
end

function Creature:takeDamage(...)
    return self:getAttacked(...)
end

function Creature:tweenAttacker(attacker, enemy, fast)
    -- tween attacker to still move to attack
    if attacker.isCreature and seen and not attacker.simpleDamage and not fast then
        local fdir = getDir(self.x-attacker.x)
        if fdir ~= 0 then
            attacker.flipX = fdir
            if fdir == 1 then
                attacker.angle = 1
            else
                attacker.angle = 1
            end
        end
        
        --[[ ATTACKER SET TO "self", NOT ACTUALLY SELF!! ]] --
        local o, self = self, attacker
        local dir = tw/5
        
        local backback
        if self.isPlayer then
        backback = function()
                self.room:set_target(self)
                     
            end
            self.room:set_target(self.tile)
        end
        
        
        local function back()
            self.timer:tween(.1,
                self,
                {x=self.x-getDir(o.x-self.x)*dir, y=self.y-getDir(o.y-self.y)*dir},
                "in-quad",
                backback
            )
        end

        self.timer:tween(.15,
           self,
            {x=self.x+getDir(o.x-self.x)*dir, y=self.y+getDir(o.y-self.y)*dir},
            "out-quad",back
        )
        
        return .15+.1
    end
end


function Creature:getAttacked(attacker, att, fast, noDodge, fake, noDefend, attackVerb, noWeapon)

    

    if type(attacker) == "number" then
        att = att or {}
        att.attack = attacker
        attacker = att
    end
    
    
    
    local val = type(att) == "number" and att or attacker.attack
    local enemy = attacker.user or attacker.parent or attacker
    local seen = attacker.isSeen or enemy.isSeen or self.isSeen
    
    
    log(string.format("%s [Get attacked Seen:%s] enemy:%s attacker:%s att:%s fast:%s noDodge:%s fake:%s noDefend:%s attackVerb:%s noWeapon:%s",
        self.title or self.name or "?",
        tostring(seen),
        enemy and (enemy.name or enemy.title or "enemy") or "?",
        attacker and (attacker.title or attacker.name or "attacker") or "?",
        att or "0?",
        tostring(fast),
        tostring(noDodge),
        tostring(fake),
        tostring(noDefend),
        tostring(attackVerb),
        tostring(noWeapon)
    ))
    
    if self.isAlreadyDead then
        warn(string.format("[ATTACK] Creature %s already knocked out %s!", self.name, attacker and attacker.name or ""))
        
        if attacker == self.room.player and not fake then
            self.room:playNextCreature()
        else
            attacker.done = true
            enemy.done = true
        end
        
        return
    end
    
     
    if not attacker.simpleDamage and not fast then
        if attacker.attackAnim then
            attacker.sprite:switch(attacker.attackAnim)
        end
        
        if attacker.attackDelay then
            self.room:after(attacker.attackDelay, function()
                self:getAttacked(attacker, att, "skip", noDodge, fake)
            end)
            return
        end
    end
    
    if fast == "skip" then
        fast = nil
    end
    if enemy.filterAttack then
        val = enemy:filterAttack(val, self)
    end
    
    local i = self.immuneTo
    if self:isImmuneTo(attacker) then--i[attacker.class] or i[attacker.name] then
        return
    end
    
    if self.isPlayer or (ememy and enemy.isPlayer) then
        self.room.moveToTile = nil
    end
    
    if not noDodge and (attacker.isItem or attacker.isCreature) and enemy.stats then
        local eq = enemy.equipped and enemy.equipped.accuracy
        
        local accuracy = enemy.stats.accuracy+(eq or 0)/(eq and 2 or 1)
        if enemy.equipped then
            accuracy = accuracy/enemy.equipped:getPenaltyValue()
        end
        
        local agility = self.stats.agility/(self.offBalance and 1.4 or 1)
        
        local hitChance = (accuracy/agility)*.7 -- defaults to 70% hit chance
        
        if math.random() > hitChance then
        
            local rand = math.random(10,40)*(math.random()>.5 and .1 or -.1)+1
            
            if (agility/rand) > accuracy then
                self:addLog(
                    string.format("%s dodged an attack!", self.title),
                    self.room.player:isAlly(self) and "lime" or self.isPlayer and "lime" or "red"
                )
                
                self:play_sound(string.format("miss_%s", math.random(1,2)))
                enemy.missedAttack = true
                
                if self.seen then
                    local fdir = getDir(attacker.x-self.x)
                    if fdir ~= 0 then
                        self.flipX = fdir
                    else
                        fdir = 1
                    end
                    
                    self.room:tween(.2, self, {x=self.x-tw/1.5*fdir}, "in-quad", function()
                        self.room:tween(.1, self, {x=self.x+tw/1.5*fdir}, "out-quad")
                    end)
                end
                
                
    
    
                
                
            else
                self:addLog(
                    string.format("%s missed an attack!", enemy.title),
                    self.room.player:isAlly(self) and "lime" or self.isPlayer and "lime" or "red"
                )
                enemy.missedAttack = true
                self:play_sound(string.format("miss_%s", math.random(1,2)))
            end
            
            if true then
                
                -- tween attacker to still move to attack
                self:tweenAttacker(attacker, enemy, fast)
            end
            
            self.attacker = attacker
            self.attackerE = enemy
            enemy.attacking = self
            attacker.attacking = self
            
            self:playDodge(attacker, enemy)
            
            
            local function next()
                if attacker == self.room.player and not fake then
                    self.room:playNextCreature()
                else
                    attacker.done = true
                    enemy.done = true
                end
            end
            --"done"
            self.room.timer:after(.01, next)
            
            return
        end
    end
    
    self:play_sound("hit" or string.format("hit_%s", math.random(1,3)))
    
    local df = math.random(self.stats.defense)/2
    local ogVal = val
    val = val - (attacker.noDefend and 0 or noDefend and 0 or df)
    
    if val <= 0 then
        if val < -1 or math.random()<.3 or ogVal == val then
            self:addLog(string.format("%s failed to damage %s!", attacker.title or attacker.name or enemy.title or enemy.name, self.title))
        else
            self:addLog(string.format("%s blocked an attack!", self.title))
        end
        
        self:tweenAttacker(attacker, enemy, fast)
        
        if true then
        
            self.attacker = attacker
            self.attackerE = enemy
            enemy.attacking = self
            attacker.attacking = self
            
            self:playDodge(attacker, enemy)
            
            
            local function next()
                if attacker == self.room.player and not fake then
                    self.room:playNextCreature()
                else
                    attacker.done = true
                    enemy.done = true
                end
            end
            
            --"done"
            self.room.timer:after(.01, next)
            
        end
        
        return
    end
    
    if self.protected > 0 then
        local p = self.protected
        if p >= val then
            self.protected = p - val
            self:addLog(string.format("%s was protected from a hit!",self.title))
            local function next()
                if attacker == self.room.player and not fake then
                    self.room:playNextCreature()
                else
                    attacker.done = true
                    enemy.done = true
                end
            end
            
            --"done"
            self.room.timer:after(.01, next)
            self:tweenAttacker(attacker, enemy, fast)
            
            return
        elseif p < val then
            self.protected = 0
            val = val - p
        end
    end
    
    if attacker.isCreature then
        attacker:addLog(
            string.format("%s %s %s", attacker.title or attacker.name, attackVerb or attacker.attackVerb or "bashed", self.title),
                self.room.player:isAlly(self) and "red" or self.isPlayer and "red" or "lime"
        )
    elseif attacker.isItem then
        self:addLog(string.format("%s got hit by %s", self.title, attacker.unspecific or attacker.title or attacker.name))
    end
   
  
    
    
    if self.healtht then
        self.room.timer:cancel(self.healtht)
    end
    
    self:shake(30, fast and .1 or .3, 35, attacker.x and
        (self:getDir(attacker.x-self.x)~=0 and (self:getDir(attacker.y-self.y)~=0 and "XY" or "X") or
         self:getDir(attacker.y-self.y)~=0 and "Y"
        )
    )
    
    
    self.attacker = attacker
    self.attackerE = enemy
    attacker.attacking = self
    enemy.attacking = self
    
    do
        -- here so effects happen faster
        if attacker.equipped and attacker.isCreature and not noWeapon then--and attacker.equipped.getAttack then
            attacker.equipped:checkEnchants(self, attacker)
        elseif attacker.checkEnchants and not noWeapon then
            attacker:checkEnchants(self, enemy or self)
        end
    end
    
    local function func()
        self:flash(.2)
        
    local xx = self.x + self.w/2
    local yy = self.y
    local s = 3
    if false then
    local t = Text:new({
        x = xx or x,
        y = yy or y,
        text = val,
        color = "lightred",
        scale = s,
        b = "",
        angle = math.random(-15,15)
    })
    t.vy=-60
    
    
    self.room:must_update(t)
    end
    
    
        --self.room.timer:cancel(self.ang2)
        self.ang2 = nil
        
        self.angle_2 = 0
        --self.room.camera:shake(15,.2,14)
    
        --self:shake(30,.3,35)
        
        self:playDamaged(attacker, val, enemy)
        
        if self.armour then
            self.armour:onDamaged(attacker, val, enemy)
        end
        
        
        if false then--self.blood ~= false and not self.noBlood and self.isPlayer then
            local x,y = self.room.camera:toWorldCoords(self.room.hx, self.room.hy)
            DROP_DEBRIS = true
            Debris.spawn(x, y, color or "white", nil, 30*(inc or 1), self.debrisW or nil)
            
            DROP_DEBRIS = false
            
            
            local x,y = self.room.camera:toCameraCoords(self.room.hx, self.room.hy)
            DROP_DEBRIS = true
            Debris.spawn(x, y, self.bloodColor or "green", nil, 30*(inc or 1))
            
            DROP_DEBRIS = false
        end
    
        if attacker.playAttacked then
            attacker:playAttacked(self)
        end
        
        
        if attacker.playAttack then
            attacker:playAttack(self)
        end
        
        
        if self.shealth <= .9 then log("?")
            --self:playDeath(attacker, att)
        end
        
        local o = self
        local pow = o.isLiving and not o.invincible
        local vv = 1.1*.5*(math.random(8,13))/10
        local tx = math.random(self.x, self.x+self.w)
        local ty = math.random(self.tile.y, self.tile.y+self.tile.h)
        
        local obj = toybox.NewBaseObject({x=tx, y=ty, w = self.w*vv, h=self.w*vv})
        obj.solid = false
        obj.sprite = toybox.new_sprite(obj, {
            animations = {
                idle = {
                     source = pow and "setpieces/splash_pow" or "effects/pow" or "effects/hit2",
                     delay = (pow and .03 or 0.05)/1.5,
                     mode = "once",
                     useImages = true,
                     onAnimOver = function()
                         obj:destroy()
                     end
                }
            }
        })
        obj.offset_x=0 obj.offset_y=0
        
        if self.isPlayer and self.shealth <= .9 and not self.firstWind then
            self.firstWind = true
            self.dying = false
            self:addLog("@You almost died!", "red")
            self.shealth = 3
            
    
        elseif self.shealth <= .9 then
            if attacker.deathText and not attacker.isCreature then
                self:addLog(string.format(attacker.deathText, self.title, self.deathText))
            
            elseif attacker.isCreature or attacker.name then
                self:addLog(string.format("%s was %s by %s", self.title, self.killedText,
                attacker.title or string.format("a %s", attacker.name)))
            else
                self:addLog(string.format("%s %s...", self.title, self.deathText))
            end
            
            log("Losing hp!! "..self.name..", "..self.id)
            
            if self.isAlreadyDead then
                warn(string.format("[ATTACK] Creature %s already knocker out %s", self.name, attacker and attacker.name or ""))
            else
                self:die(attacker, val, fast or attacker.simpleDamage or not seen, enemy)
            end
        else
            if self.isPlayer then
                if self:healthPercentage() <= 20 and not self.logged20 then
                    self.logged20 = true
                    self.room:alert("Health below 20%!")
                elseif self:healthPercentage() <= 40 and not self.logged40 then
                    self.logged40 = true
                    self.logged20 = nil
                    self.room:alert("Health below 40%!")
                else
                    self.logged40 = nil
                    self.logged20 = nil
                end
            end
        end
        
        self.healtht = self.room.timer:tween(.7,self.stats,{health=self.shealth<=.9 and 0 or self.shealth})
        --self.room.camera:shake(17,.2,20)
        self.attacker = attacker
        self.attackerE = enemy
        attacker.attacking = self
        enemy.attacking = self
        
        if self.isPlayer then
            self.room:set_target(self)
        end
        
        
        if attacker == self.room.player and not fake then
            self.room:playNextCreature()
        else
            attacker.done = true
            enemy.done = true
        end
            
    end
    
    
    if self == self.room.player then
        self.room:set_target(self.tile)
    end
    
    if self.petrified > 0 and self.tile:isCold() then
        val = val * 2
        self:addLog(string.format("%s took extra damage in the &colors.skyblue cold ~ as a stone statue!", self.title))
    end
    
    self.shealth = lume.max(0,self.stats.health - val)
    
    local tt = self.shealth<=0 and .1 or seen and (attacker.isCreature and not attacker.simpleDamage and not fast and .3 or .1) or 0.01

    self.healtht = self.room.timer:after(tt,func)
    
    self.hasBeenSeen = true
    
    self.anglee_2 = 0
    self.ang2 = self.room:tween(tt, self, {anglee_2=-15},"in-quad")
    -- self.angle_2 = -math.random(5,10)*(self.flipX)
    -- self.room.freeze = .2
    if self.isSeen then
        if (attacker.isCreature or enemy.isCreature) and attacker.isPlayer then
            self.room.camera:shake(60,.25,20)
        else
            self.room.freeze=.15
        end
    end
    
    if self.shealth <= 0 then
        self.dying = true
        --self.updateTurn = null
        --self.room:removeCreature(self)
    end
    
    if attacker.isCreature then
        self:like(attacker, -self.pettiness)
        attacker.attackedThisTurn = true
        if self.leader then
            -- ??todo: maybe make it so leader likes attacker more on certain conditions
            -- like hate or discipline?
            self.leader:like(attacker, -self.leader.pettiness)
        end
    end
    
    if self == attacker.leader then
        attacker:removeLeader()
    end
    --if attacker. then warn("fass") end
    
    if attacker.isCreature and seen and not attacker.simpleDamage and not fast then
        local fdir = getDir(self.x-attacker.x)
        if fdir ~= 0 then
            attacker.flipX = fdir
            if fdir == 1 then
                attacker.angle = 1
            else
                attacker.angle = 1
            end
        end
        
        local o, self = self, attacker
        local dir = tw/5
        
        local backback
        if self.isPlayer then
            backback = function()
                self.room:set_target(self)
            end
            self.room:set_target(self.tile)
        end
        
        
        local function back()
            self.timer:tween(.1,
                self,
                {x=self.x-getDir(o.x-self.x)*dir, y=self.y-getDir(o.y-self.y)*dir},
                "in-quad",
                backback
            )
        end

        self.timer:tween(.15,
            self,
            {x=self.x+getDir(o.x-self.x)*dir, y=self.y+getDir(o.y-self.y)*dir},
            "out-quad",back
       )
    
    end
    
    return true
end

function Creature:isFlying()
    return self.flying ~= 0 or self._kayoteTime > 0
end

local thh = th/2

function Creature:drawEquipment(eq)
    -- if type(eq) ~= "table" then return error(eq) end
    
    local thh = self.h/4
    eq.w = self.w
    eq.h = self.h

    eq.angle = self.angle
        
    eq.image_alpha = self.image_alpha
    eq.light_alpha = self.light_alpha
    
    eq.flipX = self.flipX
    eq.flipY = self.flipY
    eq.axis_x = self.axis_x
    eq.axis_y = self.axis_y
    
    local clothesOffset = eq.small and (self.clothesOffset and 0 or -25) or self.clothesOffset or 0
    
    local sx = self.sprite and self.sprite.x or self.x
    local sy = (self.sprite and self.sprite.y or self.y) + clothesOffset*(self.w/tw) +self.offset_y*.5
    
    eq:redraw(sx, sy-thh)
    
end

function Creature:draw_before()
    self.a, self.b = lg.getBlendMode()
    lg.setBlendMode(self.a)
    
    if self.helmet then
        self:drawEquipment(self.helmet)
    end
    
    
    if self.armour then
        self:drawEquipment(self.armour)
    end
    
    if self.glasses then
        self:drawEquipment(self.glasses)
    end
end

function Creature:draw()

    if self.helmet then
        self:drawEquipment(self.helmet)
    end
    
    
    if self.armour then
        self:drawEquipment(self.armour)
    end
    
    
    if self.glasses then
        self:drawEquipment(self.glasses)
    end
    
    Entity.draw(self)
    
    self.lifebar.x = self.x
    self.lifebar.y = self.y-self.h/2
    
    local rxx,yy = self.lifebar.x,self.lifebar.y
    lg.push()
    lg.translate(rxx, yy)
    self.c = self.c or math.rad(math.random(-14,15))
    lg.rotate(self.c)
    lg.translate(-rxx, -yy)
    
    local of = love.graphics.getFont()
    local font = font8
    love.graphics.setFont(font)
    local text = self.name or "Creature"
    
    if self:isDiscordant() then
        text = string.format("%s (discordant)", text)
    end
    
    
    local tww = font:getWidth(text)
    local thh = font:getHeight()
    -- love.graphics.print(text,self.lifebar.x+(self.lifebar.w/2-tww/2),self.lifebar.y-thh-2)
    love.graphics.setFont(of)
    
    local r,g,b,a = set_color(0,0,0,.4)
    local xx = 10
    local yy = 2
    -- draw_rect("fill",self.lifebar.x-xx,self.lifebar.y-yy,self.lifebar.w,self.lifebar.h)
    set_color(r,g,b,a)
    self.lifebar.drawText = false
    self.lifebar.healthText = math.floor(self.stats.health)
    if self.isPlayer and nil then self.lifebar:draw() end
    
    lg.pop()
    
    if self:isDiscordant() then
        self:drawDiscord()
    end
    
    if self.leader then
        -- lg.line(self.x,self.y,self.leader.x,self.leader.y)
    end
    
    self:drawBuffs()
    
    if self.protected > 0 then
    local x, y, w, h = self.x, self.y, tw, th*2
    y = y-h/2+th/10
    local n = w/10
    
    local a = lume.max(1, self.protected/10)
    local r,g,b,a = set_color(getColor("cyan", .1))
    
    lg.polygon("fill",x+w/2, y, x+w, y+h/2,
    x+w, y+h/2, x+w/2, y+h,
    x+w/2, y+h, x, y+h/2,
    x, y+h/2, x+w/2, y)
    
    set_color(getColor("skyblue", a))
    
    lg.line(x+w/2, y, x+w, y+h/2)
    lg.line(x+w, y+h/2, x+w/2, y+h)
    lg.line(x+w/2, y+h, x, y+h/2)
    lg.line(x, y+h/2, x+w/2, y)
    
    
    lg.line(x+w/2, y, x+w-n, y+h/2)
    lg.line(x+w-n, y+h/2, x+w/2, y+h)
    lg.line(x+w/2, y+h, x+n, y+h/2)
    lg.line(x+n, y+h/2, x+w/2, y)
    
    local n = n*2
    lg.line(x+w/2, y, x+w-n, y+h/2)
    lg.line(x+w-n, y+h/2, x+w/2, y+h)
    lg.line(x+w/2, y+h, x+n, y+h/2)
    lg.line(x+n, y+h/2, x+w/2, y)
    
    local n = n*2
    lg.line(x+w/2, y, x+w-n, y+h/2)
    lg.line(x+w-n, y+h/2, x+w/2, y+h)
    lg.line(x+w/2, y+h, x+n, y+h/2)
    lg.line(x+n, y+h/2, x+w/2, y)
    
    lg.line(x+w/2, y, x+w/2, y+h)
    lg.line(x, y+h/2, x+w, y+h/2)
    
    set_color(r,g,b,a)
    
    end
    
    if self.a then
        lg.setBlendMode(self.a, self.b)
        self.a = nil
        self.b = nil
    end
    
    if self.parasite then
        local parasite = self.parasite
        local pd = parasite.__draw
        parasite.__draw = parasite.__olddraw
        parasite:redraw(self.x, self.y)
        parasite.__draw = pd
    end
end

--[[Obsolete
function Creature:draw_after()
    for x = 1, 3 do
        local wield = Entity._wield[x]
        self:drawWield(self.lding[wield], wield)
    end
    
end
]]--

function Creature:updateTurn()
    if self.polymorphed then
        return
    end
    
    for x = 1, #self.toUpdateItems do
        if self.toUpdateItems[x] then -- in case item gets removed while updating
            local item = self.toUpdateItems[x]
            item.room = self.room
            item:playStartTurn()
            item:updateTurn(self, self.room)
        end
    end
    
    if self.onFire and type(self.onFire) == "number" then
        self.onFire = self.onFire - 1
        if self.onFire <= 0 then
            self.onFire = nil
        end
    end
    
    if self.gas then
        self.toGas  = self.toGas - 1
        
        if self.toGas <= 0 then
            local g = self.gas
            Gas:new(g, self.tile)
            
            self.toGas = self.maxGas
        end
    end
    
    
    for x, i in pairs(self.immuneTo) do
        if type(i) == "number" then
            self.immuneTo[x] = i-1
            if (i-1) <= 0 then
                self.immuneTo[x] = nil
            end
        end
    end
    
    if self._updateTurn then
        self:_updateTurn()
    end
    
    if self.tile.noises then
        local noisy = {}
        for x, noise in pairs(self.tile.noises) do
            if noise[1] > self.maxHearing then
                local dmg = noise[1]/self.maxHearing
                self.hearing = lume.max(0, self.hearing - dmg)
                noisy.name = string.format("%s noise", noise[4])
                noisy.attack = dmg
                self:getAttacked(noisy)
                
                if self.isPlayer or self.isSeen then
                    self:addLog(
                        string.format("The intense noise damages %s and %s hearing", self.title, self.pronoun),
                        self.room.player:isAlly(self) and "red" or self.isPlayer and "red" or "lime"
                    )
                end
            end
        end
    end
    
    self:updateParasite()
end

function Creature:drawBuffs()
    local dd = 6
    local p = 3
    for xx = 1,#self.buffs do
        local buff = self.buffs[xx]
        
        if buff.draw then
            buff:draw(self)
        end
    end
    
    if nil then
        local w = (self.lifebar.w-p*dd)/dd
        local h = w or self.lifebar.h/1.5
        
        local x = self.lifebar.x + (xx-1)*w+p*(xx-1)
        local y = self.lifebar.y + self.lifebar.h + p
        
        local i = game:getAsset(string.format("buffs/%s",buff.source))
        local _w, _h = resizeImage(i, w, h)
        love.graphics.draw(i, x, y, 0, _w, _h)--draw_rect("line",x,y,w,h)
        
        local f = lg.getFont()
        local nf = font8
        local sc = .8
        lg.setFont(nf)
        local txt = buff.time or buff.times or -1
        
        if txt <= -1 then
            txt = ""
        else
            txt = tostring(txt)
        end
        
        local tw = nf:getWidth(txt)*sc
        local th = nf:getHeight()*sc
        
        lg.print(txt, x+w-tw, y+h-th, 0, sc, sc)
    end
    
end

local func,map,tiles,val,vState,objj,simplem,smdone, subbed
local allies, allyCallback

local func2 = function(x,y,r,v)
    
    local tile = map.grid[x] and map.grid[x][y]
    --assert(tile)
    
    if tile and subbed and not tile.water then
        tile = nil
    end

    if tile then
        log("doing tile "..x..","..y)
        table.insert(tiles,tile)
        local d = func(tile)
        log("done tile")
        
        -- still chance for getting food target if already state (smdone)
        if simplem or smdone then
            return
        end
        
        
        if d then
            local dState = (STATES[d] or 0)+(
                (d=="H" and objj.foodTarget and (15-objj.nutrition)) or
                (d=="E" and 0) or
                (0)
            )
            
            if d == "H" and objj.lovesFood and objj.foodTarget then
                smdone = true
                val = d
                return
            end
            
            --val = not val and d or ((STATES[d] or 0) > (ov or 0) and d) or val
            if not val or (dState>vState) then
                log(d.." better than "..tostring(val))
                val = d
                vState = dState
            end
            
            if d == E then
                if WARCRAWL then
                    return d
                end
            end
        end
    end -- text =
    
end

function Creature:getTilesInRange(radius, funct)
    local tiles = {}
    
    local t = self.tile
    
    for x = -radius, radius do
        for y = -radius, radius do
            local tile = self.room:getTile(t._x+x, t._y+y)
            if tile then
                tiles[#tiles+1] = tile
                if funct then
                    funct(tile)
                end
            end
        end
    end
    
    return tiles
end

function Creature:getObservable(callback, simple)
    log(self.title.." observing")
    
    func = callback or NULL
    tiles = {}
    val = nil
    map = self.room
    objj = self
    subbed = self:isSubmerged()
    simplem = simple
    
    local xx, yy = self:get_X(), self:get_Y()
    local cval
    
    if true then
        cval = self.room.fov:compute(self:get_X(),self:get_Y(),self:getSight(),func2,true)
    else
      for x = -self.sight, self.sight do
        for y = -self.sight, self.sight do
            if x~=0 or y~=0 then
                local t = self.room:getTile(xx+x, yy+y)
                if t then
                    local c = func2(t._x, t._y)
                    cval = c
                    if c then
                        break
                    end
                end
            end
        end
      end
    end
    
    self.seenTiles = tiles
    
    vState, map,tiles, objj, simplem, smdone, subbed = nil
    
    return val or nil--tiles
end

function Creature:get_X()
    return self.tile._x
end

function Creature:get_Y()
    return self.tile._y
end

function Creature:getObservables(...)
    return self:getObservable(...)
end

local function checkForTarget(item)
    if item.isTile and (item.solid or not item:isFree()) then
        return true
    end
end

function Creature:inRange(targe,ranget,pass)

    local target = target or self.target
    if target == self or not target then
        return
    end
    
    local cx,cy = self.tile.x+tw/2, self.tile.y+th/2
    local tx,ty = (target.tile or target).x+tw/2,  (target.tile or target).y+th/2

    local distance2 = lume.distance(cx, cy, tx, ty)

    if distance2 <= (range or self:getSight()) then
        self.isNearTarget = true
        local itemInfo, len = self.world:querySegmentWithCoords(cx,cy,tx,ty,checkForTarget)
        -- ignore itemsInfo[1] because that's always self
        local info = itemInfo[2]
        if info then
            self.laserX = info.x1
            self.laserY = info.y1
            if info.item.unit == self.target then
                return true
            end
        end
    end

end

function Creature:canAttack(target)
    local target = target or self.target
    if target == self then
        return
    end
    
    if self.equipped and self.equipped.isShooter then
        local r = self:inRange(target,self.equipped.range)
        return r
    end
    
    local attRange = type(self.attackRange) == "function" and self:attackRange(target) or self.attackRange
    + (self.equipped and self.equipped.range or 0) + .5
    
    
    --return r
    return lume.distance(self:get_X(), self:get_Y(), target:get_X(), target:get_Y())<=attRange--self:collidesWith(target)--r
end

function Creature:getMeleeRange()
    return self.equipped and self.equipped.meleeRange or self.meleeRange or 1
end

function Creature:getAttackRange()
    return type(self.attackRange) == "function" and self:attackRange(target) or self.attackRange
    + (self.equipped and self.equipped.range or 0) + .5
end

function Creature:continueAttack()
    if not self.toDoAttack or #self.toDoAttack <= 0 then
        log("none attack")
        return
    end
    warn()
    local uss = self.simpleDamage
    self.simpleDamage = true
    
    local c = table.remove(self.toDoAttack, 1)
    
    self:doAttack(c, true, true)
    
    self.simpleDamage = uss
end

function Creature:doAttack(other, fake, once)
    local checkDurability
    
    if not once and not fake and ((self.equipped and self.equipped.pierce) or self.pierce) then
        self.toDoAttack = {}
        local tile = other.tile
        local dx = self:getDir(other.x-self.x)
        local dy = self:getDir(other.y-self.y)
        for i = 1, getValue(self.equipped and self.equipped.pierce or 0)+getValue(self.pierce or 0) do
            local t = self.room:getTile(tile._x+dx*i, tile._y+dy*i)
            local u = t and (t.unit or (t.item and t.item.isObstacle and t.item))
            if u then
                self.toDoAttack[#self.toDoAttack+1] = u
            end
        end
        
    end
    
    if not once and not fake then
        local val = ((self.equipped and self.equipped.attackRecovery) or self.attackRecovery)
        self.offBalance = val

        checkDurability = true
    end
    
    
    if self.equipped and self.equipped.getAttack then
        local eq = self.equipped
        self.equipped:getAttack(self, other)
        if checkDurability and not self.missedAttack then
            self.equipped.durability = self.equipped.durability - (self.equipped.selfDamage or 1)
            
                
            if self.equipped.durability <= 0 and math.random()>.5 then
                if eq.lastLegDurability then
                    local mes = string.format("%s's %s broke!", self.title, eq.name)
                    self:addLog(mes)
                    
                    if self.isPlayer then
                        self.room:alert(mes)
                    end
                    
                    local eq = self.equipped
                    eq.life = 1
                    eq:getAttacked(self, 1)
                    self.toDoAttack = nil
                else
                    eq.lastLegDurability = true
                end
                
                self:recordItemUse(eq)
            end
        end
        
        return self:continueAttack()
    
    elseif self.equipped then
        if checkDurability and not self.missedAttack then
            local eq = self.equipped
            
            self.equipped.durability = self.equipped.durability - (self.equipped.selfDamage or 1)
            if self.equipped.durability <= 0 and math.random()>.5 then
                if eq.lastLegDurability then
                    local mes = string.format("%s's %s broke!", self.title, eq.name)
                    self:addLog(mes)
                    
                    if self.isPlayer then
                        self.room:alert(mes)
                    end
                    
                    local eq = self.equipped
                    eq.life = 1
                    eq:getAttacked(self, 1)
                    self.toDoAttack = nil
                else
                    eq.lastLegDurability = true
                end
            end
            
            self:recordItemUse(eq)
        end
    end
    
    if self.getAttack then
        if self:getAttack(other) ~= false then
            log("myown att (getAttack)")
            return self:continueAttack()
        end
    end
    
    log("applying dmg")
    self:applyDamage(other, fake)
    
    return self:continueAttack()
end

function Creature:recordItemUse(item)
    local t = tostring(item)
    self.usedItems[t] = (self.usedItems[t] or 0)+1
    item.timesUsed = self.usedItems[t]
    
    if self.isPlayer and item.timesUsed >= 7 or item.lastLegDurability then
        if not item.revealedName then
            item:revealName()
        end
    end
end

function Creature:applyDamage(other, fake, noDodge)
    local os = self.attack
    self.attack = self:getDamage()
    other:takeDamage(self, nil, nil, noDodge, fake)
    self.attack = os
end

function Creature:tooCloseTo(target)
    --target:cry((lume.distance(target.x, target.y, self.x, self.y)/tw)..","..self.closest,"lime")
   
    local range = self.closest
    
    if self:isScaredOf(target) then
        range = lume.max(self.closest, self:getFear(target)+self.cowardice+2)
        self.name = range..", fear" --cwarn("yoh","orange")
    end
    
    return (lume.distance(target.x, target.y, self.x, self.y)/tw) < range--false--self:inRange(1)
end

function Creature:tooFarFrom(target)
    if true then
        return (lume.distance(target.x, target.y, self.x, self.y)/tw) > self.furthest
    end
    
    local t = target.tile.room~=self.tile.room--not self:inRange(target)
    if t then
        warn("Too far from "..target.name)
    end
    return true
end

function Creature:moveTo(tile)

    if self.immobile then
        self.done = 1
        log("immobile")
        return
    end
    
    local t1 = love.timer.getTime()
    
    self.tile.unit = nil
    self.room.except = tile
    local path = self.room:findPathTo(self, tile:get_X(), tile:get_Y())
    self.room.except = nil
    self.tile.unit = self
    
    -- patrollers seem to "by coincidence" go to invisible player
    -- -- fixed, tileTarget was invisible player
    
    local t2 = love.timer.getTime()
    if path or nil then
        --error(inspect(path,3))
        local p = path[2]
        if p then
            --self:cry(tile.class.name..",".."Patrolling (target = "..tostring(self.target)..") ,"..p.x..","..p.y..tostring(self.moving).." tile ,"..tostring(self.playing)..getDir(p.x-self:get_X())..",".. getDir(p.y-self:get_Y()))

            local m = self.room:moveObject(self, getDir(p.x-self:get_X()), getDir(p.y-self:get_Y()))
            self.tileTarget = tile
            return m
        else
            log("no path"..#path)
            
            if self.tile == self.tileTarget then
                log("tiled gotten")
            end
            
            self.tileTarget = nil
        end
    end
    
    if false then 
        local t4 = love.timer.getTime()
        local m = allMaps[tile.tile or tile]
        if m then self:addLog("reused tile!")
        else m = self.room:getDMap(tile.tile or tile, nil, self.tile)
        end
    
        local t = self.tile--m[self.tile]
        local lowest, move
     
        for x = 1,4 do
            local d = dirs4[x]
            local tt = self.room:getTile(t._x+d[1], t._y+d[2])
            if tt and  m[tt] and (not lowest or m[tt]<lowest) then
                lowest = m[tt]
                move = tt
            end
        end

        local moved
    
        if move then
            local p = move
            moved = self.room:moveObject(self, getDir(p._x-self:get_X()), getDir(p._y-self:get_Y()))
            self.targetTile = tile
        end
    
        local t5 = love.timer.getTime()
       -- error((t2-t1).." : "..(t5-t4))
       
    end
    
    if true then
        if 1 then
            self.done = 1
            self:cry("!!!!???")
        end
    end
    
    return moved
end

function Creature:makeNamed(gender, name)
    
    self.name = name or self.name
    self.title = self.name
    self.pronoun = gender == "male" and "he" or gender == "female" and "she" or "it"
    self.pronoun2 = gender == "male" and "his" or gender == "female" and "her" or "its"
    self.article = ""
    self.unspecificArticle = ""

    self.unspecific = self.title
    
    self.titlePossessive = string.format("%s's", self.title)
end

function Creature:callOthers(value)
    self:addLog(string.format("%s cried out!", self.title))
    
    self.room.nm:addNoise("alert", value or 7, self.tile)
    
    if self.isSeen then
        self:shake(40,1,50)
    end
    
    self.done = true
end



function Creature:runAwayFrom(tile, skip, num)
    -- this could be fire and the like
    local tile = tile.tile or (tile and tile.isTile and tile) or self.tile
    local runTile = tile
    
    num = num or 0
    
    if not runTile then
        warn("[RUNNING] NO Tile for running!!")
    end
    
    local maxx = 100
    local og = maxx
    
    if not tile._x then error("invalid chaser "..inspect(tile,2)) end
    
    self:cry("running", "cyan") -- set_target
    log("running")
    while (runTile == skip or runTile.roomData == tile.roomData or (lume.distance(tile._x,tile._y,runTile._x,runTile._y))<10) and maxx>0 do
        runTile = self.room:getRandomSpaceTile()
        maxx = maxx - 1
    end
    
    if not self:moveTo(runTile) then
        log("woah runner")
        if not self:moveTo(self.tile.roomData:getRandomSpaceTile()) then
            if skip or num>10 then
            
                if num > 10 then
                    warn("[RUNNING] repeated run a loooot!")
                    self.w = self.w*4
                end
                
                self.done = true
            else
                num = num + 1
                return self:runAwayFrom(tile, runTile, num)
            end
        end
    end
end

local function getAllies(tile)
    --local tile = map:getTile(x,y)
    
    if tile then
        local unit = tile.unit
        if unit and unit ~= objj and objj:isAlly(unit) then
            allies[#allies+1] = unit
            -- to avoid repeat iterations
            if allyCallback then
                allyCallback(unit)
            end
        end
    end
end

function Creature:getAllies(callback)
    allies, allyCallback = {}, callback
    self:getObservable(getAllies)
    
    return allies
end

local enemies, enemyCallback
local function getEnemies(tile)--x,y)
    --local tile = map:getTile(x,y)
    --log("try "..x..","..y.." at ??" ..objj.tile._x..","..objj.tile._y)
    if tile then
        local unit = tile.unit
        log("is Enemy?")
        if unit and objj:isEnemy(unit) then
            enemies[#enemies+1] = unit
            log("yas")
            -- to avoid repeat iterations
            if enemyCallback then
                enemyCallback(unit)
            end
        end
    end
end

function Creature:getEnemies(callback)
    enemies, enemyCallback = {}, callback
    assert(self.liking, self.data.name)
    self:getObservable(getEnemies)
    
    local e = enemies
    enemies = {}
    
    return e
end


function Creature:isEnemy(other)
    if other == self then
        return
    end
    
    if self.discordant > 0 then log("discord!")
        return true
    end
    
    if self.isPlayer and not other.isPlayer then
        log("player enme")
        if not self.equipped then
            return other:isEnemy(self)
        else
            return other.team ~= self.team
        end
    end
    
    log("deeper")
    
   -- confusion sheep
    --discordant light asset (tile size?) or maybe grumble sign?
    --texty on alerts
    -- Be a good person
    
    if self == other then return end
    
    if self:isConfused() then
        log("confused")
        return true
    end
    
    
    if other.discordant > 0 then
        return true
    end

    -- hostility 0 or less: turn the other cheek
    
    if self:isScaredOf(other) then log("scared")
        return true
    end
    
    if self.hostility <= 0 then log("coward?: "..tostring(self:isCowardly()))
        return self:isCowardly() -- and error(other.data.name..","..self.data.name) -- returns true only if creature wants to run
    end
    
    log("deeeeeper")
    
    local victim = other.attacking and other.attacking ~= self.team and other.attacking
    local attacking = other.attacker and other.attacker ~= self.team and other.attacker
    
    
       log( (other == self.leader and self:getDislike(other) >= (self.dislikeTrigger+5)))
        
        log((self:getDislike(other) >= self.dislikeTrigger+(other.team==self.team and 2 or 0)))
        
        log(self.attacker == other and self.attacker ~= self.leader)-- provoked or...
        
        -- join in on fight
        log(attacking and attacking.isCreature and ((self.team and self:isAlly(attacking) and (self.team~=other.team or self:getLiking(other)<self:getLiking(attacking))) or attacking == self.leader))
        
        -- help out teammate
        log(victim and  ((self.team and self:isAlly(victim) and (self.team~=other.team or self:getLiking(other)<self:getLiking(victim))) or victim == self.leader))
        log(victim and victim.data.name)
    ----
    ----
    if ignorePlayer and other.isPlayer then
        return
    end
    
        return -- ...
    
    -- hostility 3: all not in my team...
    self.hostility >= 3 and other.team ~= self.team or 
    
    -- hostility 2: all that I dislike, or provoked...
    self.hostility >=2 and (
    
        (other == self.leader and self:getDislike(other) >= (self.dislikeTrigger+5)) or
        
        (self:getDislike(other) >= self.dislikeTrigger+(other.team==self.team and 2 or 0)) or
        
        self.attacker == other and self.attacker ~= self.leader or-- provoked or...
        
        -- join in on fight
        attacking and attacking.isCreature and ((self.team and self:isAlly(attacking) and (self.team~=other.team or self:getLiking(other)<self:getLiking(attacking))) or attacking == self.leader) or
        
        -- help out teammate
        victim and  ((self.team and self:isAlly(victim) and (self.team~=other.team or self:getLiking(other)<self:getLiking(victim))) or victim == self.leader)
    ) or
    
    -- hostility 1: only non team members if disliked
    self.hostility >= 1 and (
        other.team ~= self.team and self:getDislike(other) >= self.dislikeTrigger
    )
    
end

function Creature:isAlly(other, noChexk)
    if not self.isPlayer and self:isDiscordant() then log("disc ??"..self.discordant)
        return false
    end
    
    if not self.isPlayer and self:isConfused() then
        return math.random()>.5
    end
    
    
    
    return (other.data == self.data or
    --other.team == self.team or
    self:getLiking(other)>=self.allyLiking) and not other:isDiscordant() and
    (not noCheck or other:isAlly(self, true))
end

function Creature:inGroup()
    return self.leader and self.leader.tile.roomData == self.tile.roomData or #self:getAllies()>0
end
    

function Creature:isDiscordant()
    return self.discordant > 0
end

function Creature:canClimb(new)
    local r = (self.wantsToClimb and (self.goalTile == self.room.nextFloorTile or self.targetTile == self.room.nextFloorTile))
    or (new and self.target and self.target.room == new)
    or (new and self.leader and self.leader.room == new)
    
    or (not new and
        (self.target and self.target.room ~= self.room) or
        (self.leader and self.leader.room ~= self.room)
    )
    
    log(" wants to climb, "..self.title..","..inspect(r)..","..inspect(self.wantsToClimb)..","..tostring(self.goalTile))
    return r
end
                

function Creature:patrol(room)
     
    log("....patrolling","skyblue")
    if self.tile == self.tileTarget then
        self.tileTarget = nil
    end
    
    if self.goalTile and self.goalTile.room ~= self.room then
        log("[PATROL] GOAL TILE CANCELED")
        self.goalTile = nil
    end
    
    if self.goalTile then
        log("goal?")
        self:moveTo(self.goalTile)
        return
    end
    
    if self.leader then
        self:moveTo(self.leader)
        return
    end
    
    if self.stayInRoom and self.tileTarget and self.tileTarget.roomData ~= self.tile.roomData then
        self.tileTarget = nil
    end
    
    local tile = self.tileTarget or (self.stayInRoom and self.tile.roomData or room or (self.patrolsRoom or math.random()>.2) and self.tile.roomData or self.room):getRandomSpaceTile()
    if not tile then
        tile = self.room:getRandomSpaceTile()
        if not tile then
            self.done = 1
            warn("NO SPACE TILE TO PATROL TO")
            log("No space to patrol")
            return
        end
    end
    
    return self:moveTo(tile)
end

function Creature:patrolRoom(room)
    return self:patrol(room or self.tile.roomData)
end

function Creature:getDamage()
    return (self.attack+getValue(self.equipped and self.equipped.attack or 0))/(self.equipped and self.equipped:getPenaltyValue() or 1)
end

function Creature:isFlammable()
    return self.flammability>0
end

function Creature:healthPercentage()
    return ((self.health/self.maxHealth)*100)
end

function Creature:isCowardly()
    return ((self.health/self.maxHealth)*10) <= self.cowardice
end

function Creature:isHungry()
    return self.nutrition <= 2 or self.isHunting
end

function Creature:isSatieted()
    return self.nutrition >= 7
end

function Creature:isTired()
    return self.energy <= 2
end

function Creature:isSleeping()
    return self.sleeping
end

function Creature:isDead()
    return self.health <= 0
end

function Creature:canEat(obj)
    local isFood = self.food[obj.name] or self.food[obj.class.name] or self.food[obj.type] or (obj.parentItem and self.food[obj.parentItem.name])
    
    return self.isLiving and self.nutrition < 11 and isFood
end

function Creature:eat(food)
    local f = food.nutrition
    food.life = food.life or math.abs(f)
    
    self:playBeforeEat(food)
    
    food:onUse(self)
    
    if food.getAttacked then
        food:getAttacked(self, self.eatRate)
    else
        food.life = food.life - self.eatRate
        error("What is this **food**? "..inspect(food,2))
    end
    
    if food.oldUser then
        self:like(food.oldUser, self.gratitude*self.eatRate*.5)
    end
    
    food:shake(10,.2,20)
    food:flash(.2)
    self.done = true
    
    self.nutrition = self.nutrition + (self:canEat(food) and math.abs(f) or f)
    self:addLog(string.format("%s took a bite of %s", self.title, food.title))
    
    self:heal(math.floor(f))--math.abs(f))
    
    self:playEat(food)
end

function Creature:updateStatusEffects(time)
    time = time or 1
    self.confused = lume.max(0, self.confused - 1)
    self.paralyzed = lume.max(0, self.paralyzed - 1)
    self.petrified = lume.max(0, self.petrified - 1)
    self.protected = lume.max(0, self.protected - 1)
    self.poisoned = lume.max(0, self.poisoned - 1)
    self.blinded = lume.max(0, self.blinded - 1)
    
    if self.enslaved > 0 and (self.enslaved <= time or not self.leader or self.leader.died) then
        self:addLog(string.format("%s is no longer enslaved!", self.title))
        
        self.team = self.oldTeam
        if self.oldLeader and not self.oldLeader.died then
            self:makeLeader(self.oldLeader)
        end
        
        self.enslaved = 0
        self.oldLeader = nil
        self.oldTeam = nil
    end
    
    self.enslaved = lume.max(0, self.enslaved - 1)
    
    
    self.discordant = lume.max(0, self.discordant - 1)
end

function Creature:isConfused()
    return self.confused > 0
end

function Creature:isParalyzed()
    return self.paralyzed > 0
end

function Creature:isPoisoned()
    return self.poisoned > 0
end

function Creature:checkPoisonDamage()
    if true then
        if self.poisoned > 0 then
            local dmg = {buff="true",name="poison",isBuff=true}
            self:addLog(string.format("%s was attacked by poison.", self.title))
            self:getAttacked(dmg, self.poisoned)
        end
    end
end

function Creature:step(dt)
    self.health = self.stats.health
    self.strength = self.stats.strength --??
    
    if not self.tile then
        log("NO TILE FOR "..self.title)
        return
    end
    
    if self.tile:isChasm() and not self:isFlying() and self.fell ~= tostring(self.tile) then
        -- assert(not self.fell)
        self.fell = tostring(self.tile)
        self.room:moveToNextFloor(self)
    end
    
    self.dt = not self.isPlayer and (self.stats.speed==1 and self.room.creatureDt or self.stats.speed>self.room.player.stats.speed and dt or self.room.creatureDt) or dt
    
    if self.done then log("self.done and "..(self.playing and "pl" or "no pl")..","..self.title) end
    
    if self.light and not self.lightData and self.tile and self.tile.unit == self and not self.died then
        local l = self.light
        self.lightData = self.room:addLight(self.tile, l[1], l[2])
    end

    self.confusedTimer = self.confusedTimer - dt
    self.paralyzedTimer = self.paralyzedTimer - dt
    self.poisonedTimer = self.poisonedTimer - dt
    
    if self.confusedTimer <= 0 and self:isConfused() then
        self:spawnEffect("confuse","setpieces/question.png")
        self.confusedTimer = getValue(self.effectTime)
    end
    
    if self.paralyzedTimer <= 0 and self:isParalyzed() and self.petrified <= 0 then
        self:spawnEffect("paralyze")
        self.paralyzedTimer = getValue(self.effectTime)
    end
    
    if self.might > 0 then
        spawnEffect(self, "setpieces/arrow.png", nil, "skyblue", -90, -th/2, false, true)
    end
    
    if self.speedy > 0 then
        spawnEffect(self, "setpieces/arrow.png", nil, "lightorange", -90, -th/2, false, true)
    end
    
    if self.speedy < 0 then
        spawnEffect(self, "setpieces/arrow.png", nil, "lightorange", 0, -th/1.7, false, true)
    end
    
    if self.poisonedTimer <= 0 and self:isPoisoned() then
        self:spawnEffect("poison")
        self.poisonedTimer = getValue(self.effectTime)--*(self.poisoned>3 and 2 or 7)
    end
    
    -- if ever update two times after one another then playnextcreature loop might break
    -- due to player waiting for gap in update to finish wraping up its actions if it played
    -- the next creature immediatley (e.g. when player attacks by movement)

    if (self~=self.room.player) and self.done == "done" then
        log("DONE 1")
        self.done = false
        self.room:playNextCreature()
    elseif (self.room.player.died or self.playing) and self.done and (self~=self.room.player) then
        log("Doning")
        self.done = "done"
        self.done = false
        self.room:playNextCreature()
    elseif self.done and not self.playing and (self~=self.room.player) and self.room.currentPlaying == self then
        self.doneCount = (self.doneCount or .6)-dt
        log("PLAYING FALSE?")
        if  self.isPlayer then error(inspect(self.room.player,1)) end
        if self.doneCount <= 0 then
            log("MOVED ON FROM COUNT DONE")
            log("Moved on from count, done, not playing, badddd?? "..self.title)
            self.doneCount = nil
            self.room:playNextCreature()
        end
    elseif self.isPlayer and self.died then
        log("so next")
        if not self.ppaused then
            self.room:playNextCreature()
        end
        self.room:removeCreature(self)
    elseif self.isPlayer and self.room.currentPlaying == self and self.done and not self.playing then
        self.room:playNextCreature()
    elseif self.done then
        -- self.room:playNextCreature()
    end
    
    self.name_tag = ""
    self.currentFrame = self.currentFrame + dt/(.15)
    
end

function Creature:lostTarget(target)
    if target.stats and not target.getStealth then
        warn("[WIERD TARGET] "..target.name..","..(target.class.name or "?"))
    end
    
    local stealth = 11-(target.stats and target.getStealth and target:getStealth() or 0)
    local dis = lume.distance(self.x/tw, self.y/th, target.x/tw, target.y/th)
    if (dis > stealth and math.random(20)>(target:isInvisible() and 5 or self.vision))
    or (target.tile and target.tile.unit ~= target) then
        --error(dis..","..stealth..","..self.vision)
        if target.isPlayer then
            self:addLog(string.format("%s lost track of %s!", self.title, target.title))
        end
        
        --self.target = nil
        return true
    end
    
    -- for ex-discrodant creatures
    if self:isAlly(target) then
        return true
    end
    
    return false
end

function Creature:isInvisible()
    return self.invisible ~= 0
end

function Creature:getStealth()
    return self:isInvisible() and 20 or self.stats.stealth
end

function Creature:spawnEffect(buff, sourcee)
    local source = buffs[buff] and buffs[buff].source or buff
    
    if buff == "poison" then
        return spawnPoison(self)
    end
    
    spawnEffect(self, sourcee)--, source)
end

function Creature:removeParasite()
    if self.parasite then
        local p = self.parasite
        p:set_box()--self.room:store_instancep:change_room(self.room, self.x, self.y)
        self.room:spawn(p, self.tile or error())
        p.__draw = p.__olddraw
        p.sprite:setDelay(p._oldSpriteDelay)
        
        self.clumsiness = self.clumsiness - (p.clumsinessAdd)
        
        self.parasite = nil
    end
end

function Creature:updateParasite()
    if self.parasite then
        if self.parasite.drain.health then
            -- takeDamage(attacker, att, fast, noDodge, fake, noDefend, attackVerb)
            self:getAttacked(self.parasite, self.parasite.drain.health, nil, true, nil, true, "suffocates")
        end
        
        if self.onFire then
            self:addLog(string.format("%s recoils from the heat!", self.parasite.title))
            self:removeParasite()
        end
    end
end


local function playNext()
    return toybox.room:playNextCreature()
end


function Creature:play()
    self.mixedAttack = nil

    self.didTurn = true
    self.doneCount = nil
    self.submerged = self:isSubmerged()
    
    self.alwaysPlay = self.alwaysPlay or math.random() > .7 and true or ""
    self.hasBeenSeen = self.hasBeenSeen or self.isSeen
    
    if not self.hasBeenSeen and self.alwaysPlay ~= true then
        local pl = self.room.player
        local dis = pl and lume.distance(self.x,self.y,pl.x,pl.y)
        if pl and dis > (15*tw) then
            return self.room:playNextCreature()
        end
    end
    
    if self.polymorphed or self.died then
        return self.room:playNextCreature()
    end
    
    -- todo: turn -1s to use turn dt
    
    if self.tile and self.tile:isChasm() then
        self._kayoteTime = self._kayoteTime - 1
    end
    
    assert(self.__step)
    self.health = self.stats.health
    log("huh")
    self:playStartTurn()
    
    if self.polymorphTime then
        self.polymorphTime = self.polymorphTime - 1
        if self.polymorphTime == 0 then
            self.polymorphTime = nil
            self:endPolymorph()
        end
    end
    
    if self.mindSwitchTime then
        self.mindSwitchTime = self.mindSwitchTime - 1
        if self.mindSwitchTime <= 0 then
            self.mindSwitchTime = nil
            self:endMindSwitch()
        end
    end
    
    if not self.isPlayer then
        self:checkPoisonDamage()
    end
    
    log("start turn!!! of "..self.name.." "..self.id.." ("..(self.isPlayer and "player)" or "creature)"))
    
    if self.lastSwitched > 0 then
        self.lastSwitched = self.lastSwitched - 1
    end
    
    if not self.isPlayer then
        self.isSeen = self.isSeen and self.image_alpha>.3 and self.light_alpha >.3
        if self.isSeen then
            local x, y, w, h = self.room.camera:getVisible()
            --e:addlo
            if self.x > x+w or self.x < x or self.y > y+h or self.y < y then
                self.isSeen = false
                --error(x..","..y..","..w..","..h..": "..self.x..","..self.y)
            end
        end
       -- self.cry=function(n) if tostring(n)=="2" then error() end end
    end
    
    log("hmmm")
    
    if self.died or self.dying then
        if self.isPlayer then
             self.room:after(.1, playNext)
             log("Player died...")
             return
        else
            return self.room:playNextCreature()
        end
    end
    
    if self.equipped then
        self:wieldItem(self.equipped)--, "center")
    elseif self.isPlayer and self.wieldedItem then
        self:unwield(self.wieldedItem)
    end
    
    if self.gun then
        self:wieldItem(self.gun)
    end
    
    if not self.wieldedItem or self.wieldedItem ~= self.equipped then
        local t = self:getItem("torch")
        
        if t and not t.off then
            self:wieldItem(t)
        end
    end
    
    -- todo!!?? : add a toWield variable, or something, wields item every turn for AI
    
    -- leave nutrition for now so creatures are only hungry if scripted
    -- self.nutrition = lume.max(0, self.nutrition - self.metabolism)
    
    if self.leader and self.leader.died then
        self:removeLeader()
    end
    
    log("speed?")
    
    if self.turnSpeed < 1 and self.room.currentTurn>2 then
        -- so speed smaller than 1 (e.g. .5) won't be decreased when not used
        -- (self.currentTurn>2) so it won't apply in first turn
        
        log("Played but no so fast "..self.turnSpeed)
        
        self.didTurn = false
        return self.room:playNextCreature()
    end
    
    
    self.didTurn = true
    
    log("para?")
    
    if self.toTeleport then
        self.room:teleport(self, type(self.toTeleport) == "table" and (self.toTeleport.isTile or self.toTeleport.isRoom) and self.toTeleport or self.tile.roomData, true)
        self.toTeleport = nil
    end
    
    if self:isParalyzed() or self.thoughtless or (not self.isPlayer and self.tile.ice) then
        if self.tile.ice then
            self:addLog(string.format("%s is frozen in place!", self.title))
        end
        
        return self.room:playNextCreature()
    end
    log("balance")
    
    if self.offBalance then
        if type(self.offBalance) ~= "number" then
            self.offBalance = 1
        end
        
        self.offBalance = self.offBalance - 1
        
        if self.offBalance < 0 then
            self.offBalance = nil
        elseif self.offBalance == 0 then
            self:addLog(string.format("%s is off balance!", self.title))
            self.offBalance = nil
            log("off balance "..(self.isPlayer and "player off" or ""))
            return self.room:playNextCreature()
        end
    end
        
    if self.isPlayer then
        allMaps = {}
        return
    end
    --isdead
    local pl = self.room.player
    if not self.isSeen and (lume.distance(self.x,self.y,pl.x,pl.y)/tw)<math.random(10,50) then
    --    return self.room:playNextCreature()
    end
    self.cry = null
    self.room.playing = self.room.playing+1
    local fg = self.foodTarget
    if fg and fg.died then
        self.foodTarget = nil
    elseif fg and not self.world:hasItem(fg) and fg.isItem then
        if fg.user and (fg.user~=self.leader or self ~= fg.user.leader or self.leader ~= fg.user.leader) then
            self:like(fg.user, -self.pettiness) pl:cry("nooo "..fg.user.name.." bad to "..self.name)
            self.foodTarget = nil
        else
            self.foodTarget = nil
        end
    elseif fg and lume.distance(fg.x,fg.y,self.x,self.y)/tw>self:getSight() then
        self.foodTarget = nil-- warn("out")
    end
    
    if self.target and self.target.died then
        self.target = nil
        self.currentState = "Waking"
        
    end
    
    if self.target and self:lostTarget(self.target) then
    
        self.target = nil
        self.tileTarget = nil
        self.currentState = "Waking" --self.doAttack =error
    end
    log("pl")
    
    self:playTurn()
    
    log("done?: "..(self.done and "yes" or "no, turn not done yet...; ")..(self.isSeen and "is seen" or "cant be seen"))
    if not self.isS8een and self.done and not self.room.player.died then
        self:step(.1)
    end
end

function Creature:playTurn()
    self.sm:peform(self)
end

return Creature