local i = {}
items = i
data.items = i

PRICES = {
    cheap = 10,
    expensive = 50,
    veryExpensive = 150,
    normal = 25,
    reallyCheap = 5,
}

local explodingTitle = "&colors.lightred exploding ~"

local mRange = 5 -- light intensity for mathsticks
local bombRange = 2

i.matchstick = {
    source = "items/matchstick.png",
    name = "mathstick",
    price  = 2,
    isUtility = true,
    
    space = 1,
    flammability = 2,
    w = tw/2, h = th/2,
    
    onUse = not function(self, user)
        user:shootGun("shotgun","shell", {x = user.x-1, y = user.y+0},nil)
    end or function(self, user)
        user:removeItem(self, true)
        local item = user:addItem("burning_matchstick")
        item.light = user.room:addLight(user.tile, mRange, self.color or "orange")
        item.tile = user.tile
        item.color = self.color or "orange"
        item.life = item.life + 1
        item:updateTurn(user)
        
    end
}

function addc(t, color, tt, ...)
    --tt = .5
    if not color then return t end
    for i = 1, #color do
        t[i] = ((t[i] or 0)*(1-tt) + color[i]*tt)
       -- if t[i]>=1 then return t end
    end
    return addc(t, ...)
end

function addc2(t, color, tt, ...)
    --tt = .5
    if not color then return t end
    for i = 1, #color do
        t[i] = (t[i] or 0)*1 + color[i]*1
       -- if t[i]>=1 then return t end
    end
    return addc(t, ...)
end

local function multiply(t, color, ...)
    local nn, nn2 = 0,2,0,1
    if not color then return t end
    for i = 1, #color do
        t[i] = (math.floor(((t[i] or 1)*255) * (color[i]*255) / 255 + 0.5))/255
    end
    return multiply(t, ...)
end

multiplyc = multiply
_G.multiply = multiply

local function spawnNewItems(new, item1, item2, anvil)
    local r = item1.room or toybox.room
    local item1Spawned, item2Spawned
    
    for x = 1, #new do
        local n = new[x]
        if n then
            if n == item1._id and not item1Spawned then
                item1Spawned = true
                r:spawnItem(item1, anvil.tile)
            elseif n == item2._id and not item2Spawned then
                item2Spawned = true
                r:spawnItem(item2, anvil.tile)
            else
                r:spawnItem(n, anvil.tile)
            end
            
            --table.remove(new, x)
        end
    end
end

i.bullets = {
    name = "bullets",
    source = "items/bullets.png",
    space = 0,
    isAmmo = true
}

i.shells = {
    name = "shells",
    source = "items/shells.png",
    space = 0,
    isAmmo = true
}

i.explosives = {
    name = "explosives",
    source = "items/explosives.png",
    space = 0,
    isAmmo = true
}

i.gold = {
    name = "gold",
    source = "items/gold.png",
    light = {3,"gold"},
    space = 0,
    
    getName = function(self)
        return string.format("%s pieces of gold", lume.max(self.value, 2))
    end,
    
    update = function(self)
        self.name = self:getName()
    end,
    
    onCarry = function(self, cr)
        if cr.isPlayer then
        end
        
        if true then
            cr.gold = cr.gold + (self.value or 0)
            cr:removeItem(self, true)
            self.room:destroy_instance(self)
        end
    end
}

i.gravestone = {
    source =( {"items/gravestone1.png","items/gravestone2.png"})[1],
    name = "gravestone",
    
    noPickup = true,
    isObstacle = true,
    unitBlocks = true,
    
    life = 5,
    
    onDestroyed = function(self)
        self:spawnDebris()
        if math.random() > .5 then
            self.room:spawn("skeleton", self.tile)
        end
    end
}

i.woodenWall = {
    name = "wood wall",
    source = "tiles/walls/wood.png",
    
    noPickup = true,
    isObstacle = true,
    isWall = true,
    unitBlocks = true,
    
    burnLife = 3,
    
    flammability = 3,
}

i.woodenFloor = {
    name = "wood floor",
    source = "tiles/woodFloor.png",
    
    noPickup = true,
    isFloor = true,
    
    burnLife = 2,
    
    flammability = 2,
}

i.carpetFloor = {
    name = "carpet floor",
    source = "tiles/carpet.png",
    
    noPickup = true,
    isFloor = true,
    
    burnLife = 2,
    
    flammability = 2,
}

i.carpetFloor2 = lume.copy(i.carpetFloor)
i.carpetFloor2.source = "tiles/carpet_2.png"

i.key = {
    name = "key",
    source = "items/key.png",
    light = {3,"gold"},
}

i.cagedKey = {
    name = "key",
    source = "items/key.png",
    light = {3,"gold"},
    
    noPickup = true,
    isObstacle = true,
    
    postMake = function(self)
        self.cage1 = self:add_image("items/cage_back.png")
        self.source2 = self:add_image("items/key.png")
        self.cage2 = self:add_image("items/cage_front.png")
        
        self.unlock = i.cagedKey.unlock
        self.lock = i.cagedKey.lock
    end,
    
    unlock = function(self)
        self.cage1.source = "none.png"
        self.cage2.source = "none.png"
        self:spawnDebris()
        
        self.noPickup = false
        self.isObstacle = false
        self.isThrowObstacle = false
    end,
    
    lock = function(self)
        if self.noClose then
            return
        end
        
        if self.isCarried then
            self.noClose = true
            return
        end
        
        if self.tile and self.tile.unit then
            self:addLog(string.format("The cage broke on %s!", self.tile.unit.title))
            self.tile.unit:getAttacked({name="cage",tile="a closing cage"},2)
            self.noClose = true
            return
        end
        
        self.cage1.source = "items/cage_back.png"
        self.cage2.source = "items/cage_front.png"
        self:spawnDebris()
        
        self.noPickup = truw
        self.isObstacle = true
        self.isThrowObstacle = true
    end
    
}

i.anvil = {
    name = "anvil",
    source = "items/anvil.png",
    
    --isObstacle = true,
    noPickup = true,
    
    isAnvil = true,
    
    onRecieve = function(self, item)
        if true then--self.room.instances[item.id] then
            self.room:destroy_instance(item)
        end
        
        self.mixes = self.mixes or {}
        self.mixes[#self.mixes+1] = item
        warn(#self.mixes..",!!!!")
        if #self.mixes > 1 then
            local mixed = false
            
            -- iterations kinda redundant, since only 2 max at a time
            for i = 1, #self.mixes do
                local m = self.mixes[i]
                if m then
                for ii = 1, #self.mixes do
                    local m2 = self.mixes[ii]
                    if m2 ~= m then log(m.mixes) log(m2._id)
                        local its = m.mixes[m2._id] or m2.mixes[m._id]
                        if its then
                            mixed = true
                            if type(its) ~= "table" then
                                its = {its,"axe"}
                            end
                            --self.room:addLog(inspect(its,1).."?")
                            cwarn("sp","yellow")
                            spawnNewItems(its,m,m2,self)--;else error(m._id)
                            break
                        end
                    end
                end
                end
                
                if mixed then
                    break
                end
            end
            
            if not mixed then
                local m = self.mixes[1]
                local m2 = self.mixes[2] cwarn("spawning","cyan")
                spawnNewItems({m._id},m,m2,self)
                spawnNewItems({m2._id},m,m2,self)
            end
            
            self.mixes = {}
            
        end
                        
    end
}

i.skull = {
    name = "skull",
    source = "items/skull.png",
    -- isObstacle = true,
    noPickup = true,
    
    isLiving = false,
}


i.matchbox = {
    name = "matchbox",
    source = "items/matchbox",
    item = "matchstick",
    isUtility = true,
    amount = 15,
}

i.carrot = {
    name = "carrot",
    space = 1,
    useText = "eat",
    nutrition = 5,
    source = "items/carrot.png",
    isFood = true
}

i.meat = {
    name = "meat",
    space = 2,
    useText = "eat",
    nutrition = 3,
    source = "items/meat.png",
    isFood = true,
    
    freshness = 15,0,--7,
    rotten = {
        name = "rotten meat",
        source = "items/rotten_meat.png",
        -- color,
        gas = {
            colors = gasColors.stinky,
            range = 1,
            flammability = 1,
            every = 3,
            intensity = 1
        },
        burn = true,
        updateTurn = function(self)
            if not self.user then
                if self.spawned then
                    self.spawned = (self.spawned>0 and self.spawned-1) or nil
                    self.totalSpawn = 0
                elseif self.isCarried then --warn("[MEAT_GAS]")
                    self.count = (self.count or 0) + 1
                    if self.count > 4 then
                        self.count = 0
                        self.totalSpawn = (self.totalSpawn or 0)+1
                        
                        if self.totalSpawn >= math.random(3,4) then
                            -- start spawning worms again in 10 to 14 turns
                            self.spawned = math.random(10,14)
                        end
                        
                        if not self.tile.unit then
                            self.room:spawnCreatureGroup("worm", self.tile, nil_team, {1,1,2}, true)
                        end
                    end
                end
            end
        end
        
    },
    
    cookedValue = 2,
    cooked = {
        name = "cooked meat",
        source = "items/cooked_meat.png",
        nutrition = 4,
        freshness = 50,
        rot = true
    },
    
    burnValue = 4,
    burnt = {
        name = "Burnt Meat",
        source = "items/burnt_meat.png",
        nutrition = -1,
        isBurnt = true
    }
    
}

i.stone = {
    name = "stone",
    source = "items/stone.png",
    space = 1,
    attack = 1,
    price = 2,
    
    onThrow = function(self, tile)
        if tile.unit then
            tile.unit:getAttacked(self)
            self.room:removeItem(self)
            return
        end
        
        return itemLand(self, tile)
    end,
    
    mix = {
        stone = "rock"
    },
}

i.rock = {
    name = "rock",
    source = "items/rock.png",
    space = 1,
    attack = 4,
    
    onThrow = function(self, tile)
        if tile.unit then
            tile.unit:getAttacked(self)
            self.room:removeItem(self)
            return
        end
        
        return itemLand(self, tile)
    end,
    
    mix = {
        hammer = {"stone","stone","hammer"}
    },
}


i.campfire = {
    name = "campfire",
    anim = "items/campfire",
    
    background = true,
    
    isObstacle = false,
    noPickup = true,
    
    postMake = function(self)
    end,
    
    isFireproof = true,
    
    updateTurn = function(self)
        local u = self.tile and self.tile.unit
        local i = self.tile and self.tile.item
        
        if u then
            --u:applyBuff("burn", self.grade+2)
        end
        
        if i and not i.isFireproof and not i.isCooked then
            i:applyBuff("burn", self.grade+2)
        end
    end,
    
    light = {4,"orange"}
}


local function turnOff(self)
    self.sprite:switch("off")
    if self.lightData then
        self.room:removeLight(self.lightData)
    end
    self._otlight = self.light or self._otlight
    self.light = nil
    self:addLog("The torch turned off!")
    self.off = true
end

local function turnOn(self)
    self.sprite:switch("idle")
    if self.lightData then
        self.room:removeLight(self.lightData)
    end
    self.light = self._otlight or self.light
    if self.light then
        self.lightData = self.room:addLight((self.user and self.user.tile) or self.tile or self.room:getTileP(self.x, self.y), self.light[1], self.light[3])
    end
    self.off = false
    self:addLog("The torch turned on!")
end

i.torch = {
    name = "torch",
    anims = {
        idle = {
            delay = .15,
            source = "items/torch_on_old",
            loop = true,
            useImage = true
        },
        off = {
            delay = .15,
            source = "items/torch_off",
            loop = true,
            useImage = true
        }
    },
    
    background = false,
    onUse = function(self)
        if not self.off then
            self:turnOff()
        else
            self:turnOn()
        end
    end,
    
    isObstacle = false,
    noPickup = false,
    
    -- burning_matchstick =
    
    updateTurn = function(self, u)
        --if not self.throwing then return end
        
        local u = self.user
        local tile = u and u.tile or self.tile--self.room:getTileP(self.x, self.y)
        self.tile = tile
        if false then-- u and u.tile and self.oldTile ~= u.tile and self.light and not self.lightData then
            self.oldTile = tile
            if self.lightData then
                self.room:removeLight(self.lightData)
            end
            self.lightData = self.room:addLight(tile, self.light[1], self.light[2])
            self.tile = u.tile
        end
        
        
        if self.light and (not self.light.isLight) and self.user then--and self.world:hasItem(self) then
            local room = self.room
            local ttile = self.user and self.user.tile or room:getTileP(self.x, self.y)
        
            if self.oldLightTile ~= ttile then
                self.oldLightTile = ttile
            
                if self.lightData then
                    room:removeLight(self.lightData)
                    self.lightData = nil
                end
                
                if self.light then
                    local l = self.light
                    self.lightData = room:addLight(ttile, l[1], l[2])
                end
            end
        elseif self.lightData then
            local room = self.room
            room:removeLight(self.lightData)
            self.lightData = nil
            self.oldLightTile = nil
        end
    end,
    
    postMake = function(self)
        self.turnOff = turnOff
        self.turnOn = turnOn
        self.angle = math.random(-40,40)
    end,
    
    onCarry = function(self, user)
        self.oldLightTile = nil
        self:updateTurn(user)
    end,
    
    isFireproof = true,
 
    light = {4,"orange"}
}

i.ice = {
    name = "ice",
    -- source = "ice.png",
    anims = {
        idle = {
            source = "items/ice",
            delay = .15,
            mode = "once",
        }
    },
    
    growthTime = 9,
    life = 6,
    h = th*2,

    
    isObstacle = true,
   -- flammability = 4,
    noPickup = true,
    
    postMake = function(self)
        self:play_sound("freeze")
    end,
    
    updateTurn = function(self)
        self.fadeAway = true
        self.isIce = true
        self.defense = 2.2
        
        if self.ddone then
            return
        end
        
        if lume.distance(self.x, self.y, self.room.player.x, self.room.player.y)/tw>50 then
            --!return
        end
        
        log("[ICE] life: "..self.life..", color: "..inspect(self.color).." | "..tostring(self))
       -- self.itemColor=getColor("purple")
       
        if self.life<=0 then
            warn(self.reallyDied)
        end
        
        if self.tile then --assert(self.tile.ice == self)
            local t = self.tile:getTemperature()
            local u = self.tile.unit and not self.tile.unit.isDormant and self.tile.unit.stats.strength/2 or 0
            if t > 8 then
                self:getAttacked(t-8+u, nil, true)
            else--if self.life < 7 and t < 0 then
                local val = (1*(t<0 and (self.lifeTime and .5 or .5) or 1)+u)/(self.tile.water and 1 or 1)
                
                
                if (self.life - val) <= 0 then
                    log("[ICE] life/: "..self.life.."/"..val.." | "..tostring(self))
                    local done = function()
                        self:getAttacked(val,nil,true)
                        if not self.dead then
                            self:die()
                        end
                    end
                    self.ddone = true
                    self.room:tween(.7, self, {image_alpha=0}, "out-quad", done)
                else
                    self:getAttacked(val,nil,true)-- self.life = self.life + 1
                end
            end
            
            self.image_alpha = lume.max(self.life/6,.4)
            if self.life <= 0 then
                local un = self.tile.unit
                if un then
                    un:addLog(string.format("%s broke through the ice!", un.title))
                else
                    self:addLog("Some ice melted away!")
                end
            end
        
        end
        warn("[IcE] self.life: "..tostring(self).." : "..self.life)
    end,
    
    onDestroyed = function(self)
        warn("[ICE] died "..tostring(self))
        self.diedd = true
        self.tile.ice = nil
        self.tile:increaseTemperature(3)

        
        if self.water then
            self.tile:addWater(self.water)
        end
        
        
        self.room:removeItem(self)
    end
}
ff=function() pl:addItem("wand_of_freezing") end
i.explosive_barrel = {
    source = "items/explosive_barrel.png",
    name = "Explosive Barrel",
    space = 3,
    flammability = 13,
    w = th, h = th,
    isObstacle = true,
    life = 5,
    
    onDestroyed = function(self)
        Fire:new({
            source = self.tile,
            explode = true,
            range = 2.5
        })
    end,
    
    light ={2,"white"}
}

i.bomb = {
    source = "items/bomb.png",
    name = "Bomb",
    price = PRICES.normal,
    isUtility = true,
    
    space = 3,
    flammability = 13,
    w = th, h=th,
    
    onUse = function(self, user)
    
        user:removeItem(self, true)
        local item = user:forceAddItem("burning_bomb")
        item.light = user.room:addLight(user.tile, mRange, self.color or "red")
        item.tile = user.tile
        item.color = self.color or "orange"
        --item.life = item.life + 1
        --item:updateTurn(user)
        self.used = true
    end,
    
    onDamaged = function(self, attacker) --assert(not self.used)
        if attacker.isFire or attacker.isExplosion then
            self.room:destroy_instance(self)
            self.tile.item = nil
            local item = self.room:spawnItem("burning_bomb",self.tile)
            item.light = self.room:addLight(self.tile, mRange,self.color or "red")
            item.tile = self.tile
            item.color = self.color or "red"
            --item.life = item.life + 1
        
        elseif attacker.isExplosion then
            
            Fire:new({
                source = self.tile,
                explode = true,
                range = 5,
            })
            self.room:destroy_instance(self)
            self.tile.item = nil
        end
    end,
    
    updateTurn = function(self)
        if self.isCarried and self.user.onFire and not self.user.inventory.burning_bomb then
            local user = self.user
            user:removeItem(self, true)
            local item = user:forceAddItem("burning_bomb")
            item.light = user.room:addLight(user.tile, mRange, self.color or "red")
            item.tile = user.tile
            item.color = self.color or "orange"
            --item.life = item.life + 1
            --item:updateTurn(user)
            self.used = true
            item:addLog(string.format("%s bomb &colors.red #activated ~!", self.user.titlePossessive))
        end
    end
        
}

i.burning_bomb = {
    source = "items/bomb_on.png",
    name = "&colors.red @Activated ~ Bomb",
    color = "red",
    info = "This bomb will go off in 4 turns",
    isUtility = true,
    
    space = 3,
    flammability = 13,
    w = th, h=th,
    life = 4,
    utility = -3,
    
    onUse = nil,
    burning = true,
    type = "bomb",
    
    onDamaged = function(self, attacker)
        if true then--attacker.isExplosion and not self.exploded then
            --if
            self.llife=1--:onThrown(self.tile)
            self.noDie = true
            --self.exploded = true
        end
    end,
        
    _onThrow = function(self, tile)
        
        Fire:new({
            source = tile,
            explode = true,
            range = bombRange,
        })
        self.room:destroy_instance(self)
        self.exploded = true
        if self.light then self.room:removeLight(self.light) end
    end,
    
    update = function(self)
        if not self.throwing then return end
        
        local tile = self.room:getTileP(self.x, self.y)
        if tile and tile ~= self.oldTile then
            self.oldTile = tile
            if self.light then
                self.room:removeLight(self.light)
            end
            self.light = self.room:addLight(tile, mRange, self.color)
        end
    end,
    
    postMake = function(self)
        self:shake(10,1000,20)
        self:play_sound("clock")
    end,
    
    updateTurn = function(self, user)
        self.color = "red"
        self.llife = (self.llife or self.life) - 1
        self.info = string.format("This bomb will go off in %s turn%s", self.life, self.life>1 and "s" or "")
        
        if self.llife <= 0 then
            self.room:addLog("The bomb...")-- burnt out")
            if user then
                user:removeItem(self, true)
            else
                self.room:removeItem(self)
            end
            
            self.room:removeLight(self.light)
            if not self.exploded then  
                Fire:new({
                    source = self.tile or user and user.tile,
                    explode = true,
                    range = bombRange,
                })
                self:addLog("A bomb &colors.orange @exploded ~!")
                self.exploded = true
            end
            return
        end
        
        if not user then
            if self.light then
                self.room:removeLight(self.light)
            end
            self.light = self.room:addLight(user and user.tile or self.tile, mRange,self.color)
        
            return
        end
        
        if self.tile ~= user.tile then
            user.room:removeLight(self.light)
            self.light = user.room:addLight(user.tile, mRange, "red")
            self.tile = user.tile
        end
    end
}

local grenadeColor = "nil"--"lightgreen"

i.grenade = {
    source = "warcrawl/items/grenade.png",
    name = "Grenade",
    isUtility = true,
    price = PRICES.normal,
    
    space = 3,
    flammability = 13,
    w = th, h=th,
    
    onUse = function(self, user)
    
        user:removeItem(self, true)
        local item = user:forceAddItem("burning_grenade")
        item.light = user.room:addLight(user.tile, mRange, grenadeColor or self.color or "red")
        item.tile = user.tile
        item.color = self.color or "orange"
        --item.life = item.life + 1
        --item:updateTurn(user)
        self.used = true
    end,
    
    onDamaged = function(self, attacker) --assert(not self.used)
        if attacker.isFire or attacker.isExplosion then
            self.room:destroy_instance(self)
            self.tile.item = nil
            local item = self.room:spawnItem("burning_grenade",self.tile)
            item.light = self.room:addLight(self.tile, mRange,grenadeColor or self.color or "red")
            item.tile = self.tile
            item.color = self.color or "red"
            --item.life = item.life + 1
        
        elseif attacker.isExplosion then
            
            Fire:new({
                source = self.tile,
                explode = true,
                range = 5,
            })
            self.room:destroy_instance(self)
            self.tile.item = nil
        end
    end,
    
    updateTurn = function(self)
        if false then -- self.isCarried and self.user.onFire and not self.user.inventory.burning_grenade then
            local user = self.user
            user:removeItem(self, true)
            local item = user:forceAddItem("burning_grenade")
            item.light = user.room:addLight(user.tile, mRange, grenadeColor or self.color or "red")
            item.tile = user.tile
            item.color = self.color or "orange"
            --item.life = item.life + 1
            --item:updateTurn(user)
            self.used = true
            item:addLog(string.format("%s grenade &colors.red #activated ~!", self.user.titlePossessive))
        end
    end
        
}

i.burning_grenade = {
    source = "warcrawl/items/grenade.png",
    name = "&colors.red @Activated ~ Grenade",
    isUtility = true,
    color = "red",
    info = "This grenade will go off in 4 turns",
    
    space = 3,
    flammability = 13,
    w = th, h=th,
    life = 4,
    utility = -3,
    
    onUse = nil,
    burning = true,
    type = "bomb",
    
    onDamaged = function(self, attacker)
        if true then--attacker.isExplosion and not self.exploded then
            --if
            self.llife=1--:onThrown(self.tile)
            self.noDie = true
            --self.exploded = true
        end
    end,
        
    onThrgow = function(self, tile)
        -- explode on impact? (disabled)
        if tile.unit and false then
            self.room:removeLight(self.light)
            if not self.exploded then  
                Fire:new({
                    source = self.tile or user and user.tile,
                    explode = true,
                    range = bombRange,
                })
                self:addLog("A grenade &colors.orange @exploded ~!")
                self.exploded = true
                self.life = .1
                self:getAttacked(2,self) -- self.chik = true
                self.room:destroyItem(self)
                if self.light then self.room:removeLight(self.light) end
            end
            -- self.__step = error
            if self.tile.item == self then self.tile.item = nil end
        end
        
    end,
    
    update = function(self)
        if not self.throwing then return end
        
        local tile = self.room:getTileP(self.x, self.y)
        if tile and tile ~= self.oldTile then
            self.oldTile = tile
            if self.light then
                self.room:removeLight(self.light)
            end
            self.light = self.room:addLight(tile, mRange, grenadeColor or self.color)
        end
    end,
    
    postMake = function(self)
        self:shake(10,1000,20)
    end,
    
    updateTurn = function(self, user)
        self.color = "red"
        self.llife = (self.llife or self.life) - 1
        self.info = string.format("This grenade will go off in %s turn%s", self.life, self.life>1 and "s" or "")
        assert(not self.chik)
        if self.llife <= 0 and not self.exploded then
        
            if user then
                user:removeItem(self, true)
            else
                self.room:removeItem(self)
            end
            
            self.room:removeLight(self.light)
            if not self.exploded then  
                Fire:new({
                    source = self.tile or user and user.tile,
                    explode = true,
                    range = bombRange,
                })
                self:addLog("A grenade &colors.orange @exploded ~!")
                self.exploded = true
                self.life = .1
                self:getAttacked(2,self) -- self.chik = true
                self.room:destroyItem(self)
                if self.light then self.room:removeLight(self.light) end
            end
            -- self.__step = error
            if self.tile.item == self then self.tile.item = nil end
            
            return
        end
        
        if not user then
            if self.light then
                self.room:removeLight(self.light)
            end
            self.light = self.room:addLight(user and user.tile or self.tile, mRange, grenadeColor or self.color)
        
            return
        end
        
        if self.tile ~= user.tile then
            user.room:removeLight(self.light)
            self.light = user.room:addLight(user.tile, mRange, grenadeColor or "red")
            self.tile = user.tile
        end
    end
}

local function puffPostMake(self)
    self:center()
    self:play_sound("grass")
    self.isPuff = true
end

local fireLightRange = 5
local thi = th*.6
i.firepuff = {
    anim = "items/puff_outlined",
    frameDelay = .15,
    name = "firepuff",
    color = "orange",
    info = "A puff of fire",
    noPickup = true,
    
    throwVa = 0,
    face_direction = true,
    
    space = 3,
    flammability = 3,
    w = thi, h=thi,
    life = 4,
    
    onUse = nil,
    burning = true,
    
    onDestroyed = function(self)
        SMOKE_DEBRIS = true
        local nn = 2
        self.debrisW = tw*.7
        self.w, self.h = self.w*nn, self.h*nn
        self:spawnDebris(self.color)
        self.w, self.h = self.w/nn, self.h/nn
        self:addLog("smoke?")
        SMOKE_DEBRIS = nil
    end,
    
    onDamaged = function(self, attacker)
        if attacker.isExplosion and not self.exploded then
            --if
            --self.llife=1--:onThrown(self.tile)
            --self.noDie = true
            --self.exploded = true
        end
    end,
        
    onThrow = function(self, tile)
        
        Fire:new({
            source = tile,
            intensity = tile:getFlammability()+(self.attack or 1)/2
        })
        
        if tile.unit then
            tile.unit:applyBuff("burn", self.attack or 1, (self.attack or 1)/2)
        end
        
        if tile.item then
            tile.item:applyBuff("burn", self.attack or 1, (self.attack or 1)/2)
        end
        
        self:onDestroyed()
        local l = function()
            self.room:destroy_instance(self)
        end
        
        self.room:tween(.2, self, {image_alpha=0}, "out-quad", l)
        self.exploded = true
        if self.light then self.room:removeLight(self.light) end
    end,
    
    update = function(self)
        if not self.throwing then return end
        
        local tile = self.room:getTileP(self.x, self.y)
        if tile and tile ~= self.oldTile and not tile.solid then
            self.oldTile = tile
            if self.light then
                self.room:removeLight(self.light)
            end
            self.light = self.room:addLight(tile, fireLightRange, self.color)
            
            local l = tile:getFlammability()
            if l>0 then
                Fire:new({source = tile, intensity = self.flammability})
            end
        end
    end,
    
    postMake = puffPostMake or function(self)
        --self:shake(10,1000,20)
    end,
    
    updateTurn = function(self, user)
  
    end
}

i.icepuff = {
    source = "snowflake.png",--items/puff_outlined",
    frameDelay = .15,
    name = "snowpuff",
    color = "skyblue",
    info = "A puff of ice",
    noPickup = true,
    
    throwVa = 0,
    face_direction = true,
    
    space = 3,
    flammability = 3,
    w = thi*3, h=thi*3,
    life = 4,
    
    onUse = nil,
    
    onDestroyed = function(self)
        DEBRIS_SOURCE = "snowflake.png"
        local nn = 2
        local t = self.tile
        self.debrisW = tw*.7
        self.w, self.h = self.w*nn, self.h*nn
        self:spawnDebris(self.color)
        self.w, self.h = self.w/nn, self.h/nn
        DEBRIS_SOURCE = nil
        
        
        for g = 1, self.grade+1 do
            for x = 1, 8+1 do
                local d = dirs8[x] or {0,0}
                local g = g
                if g == (self.grade+1) then
                    g = math.random()>.5 and g or nil
                end
                
                if g then
                    local t = self.room:getTile(d[1]+t._x*g, d[2]+t._y*g)
                    if t and not t:isSolid() then
                        --t:addWater(nil, false)
                        if t then
                            tt = t:getTemperature()
                            --tt = tt>0 and lume.min(self.grade*11,tt) or 2
                            t:decreaseTemperature((2*(self.grade-1)))
                            t.iceLifeTime = self.grade
                            if tt <= 10 then
                                t:spawnIce(self.grade and (self.grade*3+4) or error())
                            else
                                t:addLog("You hear the sizzle of steam.")
                            end
                        end
                    end
                end
            end
        end
    end,
    
    onDamaged = function(self, attacker)
        if attacker.isExplosion and not self.exploded then
            --if
            --self.llife=1--:onThrown(self.tile)
            --self.noDie = true
            --self.exploded = true
        end
    end,
        
    onThrow = function(self, tile)
        
        
        self:onDestroyed()
        local l = function()
            self.room:destroy_instance(self)
        end
        
        self.room:tween(.2, self, {image_alpha=0}, "out-quad", l)
    end,
    
    update = function(self)
        if not self.throwing then return end
        
        if math.random() > .8 then
            DEBRIS_SOURCE = "snowflake.png"
            self:spawnDebris()
            DEBRIS_SOURCE = nil
        end
        
        local tile = self.room:getTileP(self.x, self.y)
        if tile and tile ~= self.oldTile and not tile.solid then
            --[[self.oldTile = tile
            if self.light then
                self.room:removeLight(self.light)
            end
            self.light = self.room:addLight(tile, fireLightRange, self.color)
            
            local l = tile:getFlammability()
            if l>0 then
                Fire:new({source = tile, intensity = self.flammability})
            end]]
            
            tile:increaseTemperature(-7)
        end
    end,
    
    postMake = puffPostMake or function(self)
        --self:shake(10,1000,20)
    end,
    
    updateTurn = function(self, user)
  
    end
}

i.puff = {
    anim = "items/puff_outlined",
    frameDelay = .15,
    name = "puff",
    info = "A puff of smoke",
    noPickup = true,
    
    throwVa = 0,
    face_direction = true,
    
    space = 3,
    flammability = 3,
    w = thi, h=thi,
    life = 4,
    
    onUse = nil,
    
    onDestroyed = function(self)
        SMOKE_DEBRIS = true
        local nn = 2
        self.debrisW = tw*.7
        self.w, self.h = self.w*nn, self.h*nn
        self:spawnDebris(self.color)
        self.w, self.h = self.w/nn, self.h/nn
        self:addLog("smoke?")
        SMOKE_DEBRIS = nil
    end,
    
    onDamaged = function(self, attacker)
        if attacker.isExplosion and not self.exploded then
            --if
            --self.llife=1--:onThrown(self.tile)
            --self.noDie = true
            --self.exploded = true
        end
    end,
        
    onThrow = function(self, tile)
        
        if tile.unit and self.buff then
            tile.unit:applyBuff(self.buff, self.buffTime, self.intensity)
        end
        
        if tile.item and self.itemBuff then
            tile.item:applyBuff(self.itemBuff, self.buffTime, self.intensity)
        end
        
        self.tile = tile
        
        self:onDestroyed()
        local l = function()
            self.room:destroy_instance(self)
        end
        
        self.room:tween(.2, self, {image_alpha=0}, "out-quad", l)
        self.exploded = true
        if self.light then self.room:removeLight(self.light) end
    end,
    
    update = function(self)
        if not self.throwing then return end
        
        local tile = self.room:getTileP(self.x, self.y)
        if tile and tile ~= self.oldTile and not tile.solid and self.light then
            self.oldTile = tile
            if self.light then
                self.room:removeLight(self.light)
            end
            self.light = self.room:addLight(tile, fireLightRange, self.color)
        end
    end,
    
    postMake = puffPostMake or function(self)
        self:center()--:offset(-(tw-self.w)/2, 0)
        --self:shake(10,1000,20)
    end,
    
    updateTurn = function(self, user)
  
    end
}

i.gravestone = {
    source =( {"items/gravestone1.png","items/gravestone2.png"})[1],
    name = "gravestone",
    
    noPickup = true,
    isObstacle = true,
    unitBlocks = true,
    
    life = 5,
    
    onDestroyed = function(self)
        self:spawnDebris()
        if math.random() > .5 then
            self.room:spawn("skeleton", self.tile)
        end
    end
}


i.cursed_gravestone = {
    source =( {"items/gravestone1.png","items/gravestone2.png"})[1],
    name = "gravestone",
    
    noPickup = true,
    isObstacle = true,
    unitBlocks = true,
    
    life = 5,
    
    onDestroyed = function(self)
        self:spawnDebris()
        if math.random() > .5 or true then
            self.room:spawn("skeleton", self.tile)
        end
    end
}
i.burning_matchstick = {
    source = "items/matchstick.png",
    name = "Burning Matchstick",
    isUtility = true,
    
    space = 1,
    flammability = 0,
    life = 3,
    w = tw/2, h = th/2,
    burning = true,
    
    onUse = function(self, user)
    end,
    
    onThrow = function(self, tile)
        Fire:new({source=tile})
        self.room:destroy_instance(self)--user:removeItem(self, true)
        if self.light then
            self.room:removeLight(self.light)
        end
    end,
    
    update = function(self)
        if not self.throwing then return end
        
        local tile = self.room:getTileP(self.x, self.y)
        if tile and tile ~= self.oldTile then
            self.oldTile = tile
            if self.light then
                self.room:removeLight(self.light)
            end
            self.light = self.room:addLight(tile, mRange, self.color)
        end
    end,
    
    updateTurn = function(self, user)
        
        self.life = self.life - 1
        
        if self.life <= 0 then
            local room = self.room
            room:addLog("The matchstick burnt out")
            
            if user then
                user:removeItem(self, true)
            else
                room:removeItem(self)
            end
            room:removeLight(self.light)
            return
        end
        
        --if not user then return end
        
        if user and self.tile ~= user.tile then
            user.room:removeLight(self.light)
            self.light = user.room:addLight(user.tile, mRange, self.color)
            self.tile = user.tile
        end
        local l = self.tile:isFlammable(true)
        if l then
                Fire:new({
                    source = self.tile,
                    intensity = 1,
                })
            
        end
        
    end
}