local i = items
local standardRechargeTimes = {
        100,
        70,
        50,
        30,
        20,
        10,
        4,
        2
}

local RECHARGE = {
    VERY_HOT = {"VERY HOT","red"},
    HOT = {"HOT", "lightred"},
    WARM = {"WARM","darkorange"},
    WARMISH = {"WARMISH", "orange"},
    LUKE_WARM = {"LUKE WARM", "yellow"},
    COLDISH = {"COLDISH", "royalblue"},
    COLD = {"COLD", "blue"},
    ICE_COLD = {"ICE COLD", "skyblue"}
}
    
local function getRecharge(charm)
    local v = charm:getRechargePercentage()
    
    local recharge = 
        v >= 100 and RECHARGE.VERY_HOT or
        v >= 90 and RECHARGE.HOT or
        v >= 75 and RECHARGE.WARM or
        v >= 60 and RECHARGE.WARMISH or
        v >= 45 and RECHARGE.LUKE_WARM or
        v >= 30 and RECHARGE.COLDISH or
        v >= 15 and RECHARGE.COLD or
        RECHARGE.ICE_COLD
    
    return recharge[1], recharge[2], recharge[3]
end

local getRechargeTime = function(self)
    if type(self._rechargeTime) == "table" then
        self.rechargeTime = self._rechargeTime[self.grade] or self._rechargeTime[#self._rechargeTime]
    else
        self.rechargeTime = self:_rechargeTime(self.grade)
    end
    
    return self.rechargeTime
end

local function getRechargePercentage(self)
    return ((self.rechargeTime-self.timeToRecharge)/self.rechargeTime)*100
end

local charmPostMake = function(self)
    self.timeToRecharge = self.data.timeToRecharge or 0
    self._rechargeTime = self.data.rechargeTime or standardRechargeTimes
    self.getRechargeTime = getRechargeTime
    self.getRechargePercentage = getRechargePercentage
    
    self.lightColor = self.data.lightColor or "gold"
    
    self:getRechargeTime()
end

local charmUpdateTurn = function(self)
    self.timeToRecharge = self.timeToRecharge - 1
    
    self.rechargeName, self.rcolor = getRecharge(self)
    if self.rechargeName ~= self.oldRechargeName then
        if self.user and self.user.isPlayer then
            self.user.room:reloadInventoryUI()
            self.user:addLog(string.format("Your %s is now &colors.%s %s ~ ...",self.data.name, self.rcolor, self.rechargeName))
        end
    end
    
    self.oldRechargeName = self.rechargeName
    
    if self.timeToRecharge <= 0 then
        self.light = {5,self.data.lightColor or "gold"}
        self.timeToRecharge = 0
    elseif self.lightData then
        self.room:removeLight(self.lightData)
        self.lightData = nil
    end
end

local charmEnchanted = function(self, lvl)
  
    --self.ogName = self.ogName or self.name
    --self.name = string.format("&colors.gold %s%s %s (%s%s)", self.grade>0 and "+" or "-",self.grade,self.ogName,self:getRechargePercentage(),"%")
end


local charmGetName = function(self)
    self.ogName = self.ogName or self.name
    local v = self:getRechargePercentage()
    local recharge, color = getRecharge(self)
        
    return string.format("&colors.gold %s%s %s ~ [ &colors.%s %s ~ ]", self.grade>0 and "+" or "-",self.grade,self.ogName,color,recharge)
end

local charmOnUse = function(self, user)
    if self:getRechargePercentage() >= 100 then
        user:addLog(string.format("%s used %s", user.title, self.title))
        
        self.timeToRecharge = self:getRechargeTime()
        
        self.room:addEnchantLight(user.tile, 3, self.lightColor)
        
        for g = 1, self.grade do
            local f = self.data.actions[g]
            if f then
                f(self, user, self.grade)
            end
        end
    else
        user:addLog(string.format("%s fizzled", self.title))
    end
end
    
--[[
Utility ([int] or function(user))
    3   - instant saving grace
    2   - pretty good advantage
    1   - better than nothing
    0   - meh
    -1  - hmph, devious
    -2  - wow, that's bad
    -3  - pure evil. Very faily
    
    
]]

local function new_charm(data)
    data.postMake = data.postMake or charmPostMake
    data.updateTurn = data.updateTurn or charmUpdateTurn
    data.getName = data.getName or charmGetName
    data.onEnchant = data.onEnchant or charmEnchanted
    data.rechargeTime = data.rechargeTime or standardRechargeTimes
    
    data.isCharm = true
    
    data.onUse = data.onUse or charmOnUse
    
    return data
end

i.charm_of_fire = new_charm({
    source = "items/charms/charm_of_fire.png",
    name = "charm of fire",
    
    postMake = charmPostMake,
    updateTurn = charmUpdateTurn,
    getName = charmGetName,

    onEnchant = charmEnchanted,
    
    rechargeTime = standardRechargeTimes,
    isCharm = true,
    utility = 2,
    
    actions = {
        [1] = function(self, user, grade)
            local t = user.tile
            local inc = 2
            for x = 1, 8 do
                local d = dirs8[x]
                local n = user.room:getTile(t._x+d[1]*inc, t._y+d[2]*inc)
                if n then
                    Fire:new({
                        source = n,
                        range = 1,
                        intensity = grade
                    })
                end
                
            end
        end,
        [2] = function(self, user, grade)
            local t = user.tile
            local inc = 2
            for x, t in ipairs(user.seenTiles) do
                -- local d = dirs8[x]
                -- local n = user.room:getTile(t._x+d[1]*inc, t._y+d[2]*inc)
                if t.unit and t.unit ~= user then
                   Fire:new({
                        source = t,
                        range = 1,
                        intensity = grade
                    })
                end
                
            end
        end,
        [3] = function(self, user, grade)
            user:makeImmuneTo("burn", 4*grade*grade)
            local val = user.immuneTo.burn
            
            user:addLog(string.format("%s is now immune to burning %s",
                user.title,
                type(val) == "number" and 
                    string.format("for %s turns!", val) or "!"
                )
            )
        end,
        [4] = function(self, user, grade)
            attacks.summon(user, "wisp", math.floor(self.grade/2), user.tile, user.team)
        end
        
        
        
    },

})