local i = items

local weights = {
    common = 60,
    rare = 25,
    ultrarare = 12,
    mythical = 3
}

scrolls = {}
local scrollWeights = {}

function revealScroll(u, p, fake)
    
    if u.isPlayer and not data.knownScrolls[p._id] then
        if not fake then
            u:addLog(string.format("The %s must be a %s!", p.name, p.data.oldName))
        end
        data.knownScrolls[p._id] = true
        if u.room.inventoryIsOpen then
            u.room:reloadInventoryUI()
        end
    end
end

local scroll = {
    source = "items/scroll.png",
    name = "Scroll",
    space = 1,
    symbol = "beta",
    
    life = 3,
    w = tw*.8, h = th*.8,
    buff = {"frightful",41,10},
    postMake = function(self)
        self.symbol = self:add_image(string.format("items/symbols/%s.png",self.data.symbol))
    end,
    
    onEnchant = function(self)
        if self.user then
            self.user:reloadItem(self)
        end
    end,
        
    useText = "read",
    useDamage = -3,
    
    getName = function(self)
        
        if data.knownScrolls[self._id] then
            self.ogName = self.data.oldName
            self.name = string.format("%s%s%s ~ %s", self.enchants>0 and "&colors.cyan " or "", self.enchants>0 and "+" or self.enchants==0 and "" or "-",self.enchants ~=0 and self.enchants or "",self.ogName)
            self.title = string.format("the %s", self.name)
        end
        
        
        return string.format("%s%s%s ~ %s", self.enchants>0 and "&colors.cyan " or "", self.enchants>0 and "+" or self.enchants==0 and "" or "-",self.enchants ~=0 and self.enchants or "",self.ogName)
        
    end,
    
    onUse = function(self, user)
        user:play_sound("paper")
        
        if self.buff then
            user:applyBuff(self.buff, self.buffTime, self.intensity)
        end
        
        if self.data.func then
            self.data.func(self, user)
        end
        
        if user.isSeen then
            revealScroll(user.room.player, self)
        end
        
        self:getAttacked(user, 3)--.life = self.life - 1
    end
}

local function teleportUser(self, user)
    self.room:teleport(user, nil, true)
end

local function cloneUser(self, user)
    for x = 1, (self.grade+1) do
        -- clone: tile, fullHP, advanced cloning (keep equipment)
        user:clone(nil, true, true)
    end
    user.room:addLight(user.tile, 3, "lime", 2.55)
    
    user.room.nm:addNoise("alert",  7, user.tile)
end

local function summonElectricBall(self, user)
    local e = self.room:spawn("electric_ball", user.tile, user.team)
end

local function _enchantItem(item, room)
    item:enchant(room.choosingItem.chooser.grade)
    room:addEnchantLight(room.choosingItem.tile, 3, "lightgreen")
end

local function enchantItem(self, user)
    if user.isPlayer then
    
        self.room:reloadInventoryUI(user)
        self.room:openInventory()
    
        self.room.choosingItem = {
            chooser = self,
            canCancel = true,
            tile = user.tile,
            cancelText = "Waste a scroll of enchantment?",
            text = "enchant",
            action = _enchantItem
        }
        
        self:getAttacked(user, 3)
        
        -- user:addLog("It's a scroll of enchanting!")
        user:addLog("Enchant want?")
        
    else
        self.done = true error(inspect(user,2))
        -- todo: enchant highest/item enchanting
    end
end

function vegetate(self, user)
    attacks.vegetation(user.tile, self.grade)
end

function getScroll(utility, rarity, name, symbol, buff, intensity, func, light, postMake)
    local p = lume.copy(scroll)
    p.name = name
    --p.color = color
    p.symbol = symbol or p.symbol
    p.intensity = intensity or 10
    p.buff = buff
    p.func = func
    p.light = light --or (light == nil and {2, color})
    p.isScroll = true
    
    if postMake then
        postMake(p)
    end
    
    
    p.utility = utility
    
    scrollWeights[p.name] = getValue(weights[rarity or "common"])
    scrolls[p.name] = p
    
    return p
end

local syllables = {
    "il","lo","i","la","xo","xand","cro","ja","ji","jik","fig","nu",
    "he","hep","hed","hu","hum","hra","hg","hk","hok","hop","hen",
    "gre","gha","gan","got","gco","gvho","ganim","ghij","gob","gof",
    "fer","ger","mer","neh","imo","jai","jojo",
    "zok","zek","zed","ze","za","zim","zig",
    "yah","yatt","yam","yox","yoj",
    "pox","pix","pax","po","pod","pojhn","pol",
    "tre","tohn","ta","tek","tom","tox","tehn"
}

local takenNames = {}

local scrollSymbols = {
    "twinkle",
    "null",
    "mitosis",
    "wifi",
    "hammer",
    "flame",
    "skull",
    "star",
    "theta",
    "beta",
    "lambda",
    "omega"
    
}

local wordLen = {2,3,3,3,3,4,4}
function getRandomScrollName()
    local n
    while not n or takenNames[n] do
        n = "scroll labeled %s %s"
        local n1, n2
        for x = 1, 2 do
            -- local done = {}
            local nn = ""
            local len = getValue(wordLen)
            for xx = 1, len do
                nn = string.format("%s%s",nn,getValue(syllables))
            end
            n1 = x == 1 and nn or n1
            n2 = x == 2 and nn or n2
        end
        
        n = string.format(n, n1, n2)
    end
    
    takenNames[n] = true
    return n
end

function shuffleScrolls()
    takenNames = {}
    local symbols = lume.copy(scrollSymbols)
    local symbLen = #symbols
    
    data.knownScrolls = {}
    
    for x, i in pairs(scrolls) do
        i.oldName = i.oldName or i.name
        i.name = getRandomScrollName()
        
        i.oldSymbol = i.oldSymbol or i.symbol
        
        assert(#symbols > 0, string.format("%s is not enough symbols!",symbLen))
        
        i.symbol = lume.eliminate(symbols)
    end
    
    revealScroll(items.scroll_of_collection)
end

local function shuffleCreatures(self, user)
    local tile = user.tile
    local room = tile.roomData
    
    for x, i in ipairs(user.seenTiles or room.allTiles) do
        if i.unit then
            local cr = i.unit
            
            local ot = cr.tile or self:getTileP(cr.x, cr.y)
            local ti = room:getRandomSpaceTile()
            local tile
            local max = 10
    
            if not tile then
                while (ti == ot) or ((ot and ot.roomData and ot.roomData == ti.roomData) and max>0) do
                    ti = room:getRandomSpaceTile()
                    max = max - 1
                end
            end
            
            if ti then
                ti.room:teleport(cr, ti)
            end
        end
    end
    
    user.room:addEnchantLight(tile, 3, "yellow")
    user:addLog("@Everyone in the room got shuffled!")
end


local function causeDiscord(self, user)
    local tile = user.tile
    local room = tile.roomData
    local val  = 5*self.grade
    local range = 5*self.grade
    
    for x, i in ipairs(user:getTilesInRange(range)) do--.seenTiles or room.allTiles) do
        if i.unit then
            local cr = i.unit
            
            cr.discordant = cr.discordant + val
        end
    end
    
    user.room:addEnchantLight(tile, 3, "maroon")
    user:addLog("A &colors.maroon #disorienting ~light comes out affecting nearby creatures!")
end

local function causeFear(self, user)
    local tile = user.tile
    local room = tile.roomData
    
    user:applyBuff("frightful",8*self.grade,4)
    user.room:addEnchantLight(tile, 3, "maroon")
    user:addLog(string.format("A &colors.maroon #disorienting ~light comes out affecting %s appearance!",user.titlePossessive))
end


local collectColors = {"gold","brown","pink","skyblue","royalblue","purple","white"}

local function collectItems(self, user)
    for i, t in ipairs(user.tile.roomData.allTiles) do
        if t.item and not t.item.noPickUp then
            local item = t.item
            if user:addItem(t.item,nil,nil,nil,nil,true) then
                spawnTrail(t.x+tw/2,t.y+th/2,user.tile.x+tw/2,user.tile.y+tw/2,item.itemColor or item.color or self.ogColor or getValue(collectColors),math.random(100,130)/90)
            else
            end
        end
    end
    user:addLog("You collected the items in the room!")
    user.room:closeInventory()
end

local function shatter(self, user)
    local t = user and user.tile or self.tile
    local e = self.grade*2+1
    
    t.room:addEnchantLight(t, 3, "green", 3)
    
    for x = -1*e, e do
        for y = -1*e, e do
            local t2 = t.room:getTile(t._x+x, t._y+y)
            local dis = t2 and lume.distance(t.x+t.w, t.y+t.h, t2.x+t2.w, t2.y+t2.h)/tw
            if t2 and dis <= e and not t2:isLastWall() and t2.solid then
                local k = lume.max(math.abs(x), math.abs(y))
                local e1 = e-1
                local a = math.abs(x)+math.abs(y)
                
                local function br()
                    local func = function()
                        t2:getAttacked(t2.life/(k>=(e-2) and 1 or k<e1 and 1 or 1), self)
                    end
                
                    t.room.timer:tween(.3, t2, {life=0}, "out-quad", func)
                end
                
                local time = lume.min(2,(a)*.4)
                t2:shake(10,time+.4,15)
                t.room:after(time, br)
            end
        end
    end
end

local summonTable = {
    [1] = {rat=1,goblin=1,kobold=1,goblin_conjurer=1},
    [2] = {rat=1,goblin=2,pig=1,boar=1,goblin_conjurer=1},
    [3] = {goblin=3,boar=3,bloat=2,skeleton=1},
    [4] = {angel=4,skeleton=5}
}

local classTable = {
    kobold = {"archer","bomber","scout","scout","warrior",false,false},
    goblin = {"bezerker","archer","scout","warrior",false,false},
    goblin_conjurer = {"conjurer",false,false}
}

local function summon(self, user)
    local cr = lume.weightedchoice(summonTable[self.grade] or summonTable[#summonTable])
    local class = getValue(classTable[cr])
    local cr = user.room:spawn(cr, user.tile)
    
    
    self.room:addEnchantLight(user.tile, 8, "lime")
    
    if not cr then
        return user:addLog("The fabric of space trembles but nothing happens...")
    end
    
    if class then
        cr:addClass(class, self.grade)
    end
    
    cr.team = tostring(cr)
    cr.discordant = 1000
    user:addLog(string.format("%s was summoned! It looks angry...", cr.unspecific))
    
    if user.isSeen then
        revealScroll(self)
    end
end


function getRandomScroll()
    return scrolls[lume.weightedchoice(scrollWeights)]
end


--[[
Utility ([int] or function(user))
    3   - instant saving grace
    2   - pretty good advantage
    1   - better than nothing
    0   - meh
    -1  - hmph, devious
    -2  - wow, that's bad
    -3  - pure evil. Very faily
    
    
]]

-- heheh, crum
local C, R, U, M = "common", "rare", "ultrarare", "mythical"

i.scroll_of_teleportation = getScroll(2,R,"scroll of teleportation", "beta", nil, nil, teleportUser)
i.scroll_of_duplication = getScroll(3,R,"scroll of duplication", "mitosis", nil, nil, cloneUser)
i.scroll_of_enchanting = getScroll(0,U,"scroll of enchanting", "beta", nil, nil, enchantItem)
i.scroll_of_electricity = getScroll(3,U,"scroll of electric ball", "beta", nil, nil, summonElectricBall)
i.scroll_of_shuffle = getScroll(2,R,"scroll of shuffle", "beta", nil, nil, shuffleCreatures)
i.scroll_of_discord = getScroll(0,R,"scroll of discord", "beta", nil, nil, causeDiscord)
i.scroll_of_fear = getScroll(-2,R,"scroll of fear", "skull", nil, nil, causeFear)
i.scroll_of_vegetation = getScroll(-2,R,"scroll of vegetation", "skull", nil, nil, vegetate)
i.scroll_of_collection = getScroll(3,U,"scroll of collection", "mitosis", nil, nil, collectItems)
i.scroll_of_summoning = getScroll(3,R,"scroll of summoning", "mitosis", nil, nil, summon)
i.scroll_of_shattering = getScroll(1,C,"scroll of shattering", "mitosis", nil, nil, shatter)