local multiply = multiplyc

local wand = {
    source = "guns/shotgun.png",
    name = "wand",
    pcolor = "white",
    space = 2,
    
    charges = 4,
    onEnchant = function(self, n)
        self.charges = self.charges + n*self.chargeIncrement
    end,
    
    isShooter = true,
    useText = "zap",
    
    
    postMake = function(self)
        local sourceID = math.random(1, 8)
        
        local img1 = string.format("items/wands/wand_%s_in.png", sourceID)
        self.source = img1
        self.image1 = self:add_image(img1)
        self.image1.color = lume.copy(getColor(self.data.pcolor))
        self.ogColor = getColor(self.data.pcolor)
        
        self.image2 = self:add_image(string.format("items/wands/wand_%s.png",sourceID))
        
        self.charges = self.data.charges
        self.chargeIncrement = self.data.chargeIncrement
        self.life = self.data.charges
    end,
    

    
    onDestroyed = function(self)
        self:addLog(string.format("%s broke!",self.title))
    end,

            
    
    update = function(self, dt)
        if self.color then
            local r = getColor(self.color)
            local og = getColor(self.ogColor)
            local c = self.image1.color
            
           -- c[1], c[2], c[3] = 1,1,1--og[1]+r[1], og[2]+r[2])/2
            c = multiply(c, og or colors.white)--, r)
        else
            
            self.name_tag = ","
            local og = self.ogColor
            local c = self.image1.color
            c[1], c[2], c[3] = og[1], og[2], og[3]
        end
        
        self.image1.alpha = self.image_alpha
        self.image1.light_alpha = self.light_alpha
    end,
    
    getName = function(self)
        self.ogName = self.ogName or self.name
        return string.format("%s%s%s ~ %s &colors.cyan [%s]", self.enchants>0 and "&colors.cyan " or "", self.enchants>0 and "+" or self.enchants==0 and "" or "-",self.enchants ~=0 and self.enchants or "",self.ogName,self.life)
    end,
    
    onUse = function(self, user)
        if self.buff then
            local b = buffs[self.buff] or self.buff
            user:applyBuff(self.buff, self.buffTime or b.getTime(self.grade), self.intensity or b.getValue(self.grade))
        end
        
        self:addLog(string.format("%s used %s", user.title or user.name, self.title))
        
        if true then--self:hasAmmo(user) then
            user.room.selectingTarget = true
            user.room.throwing = self
            user.room.onTarget = self.onTarget
        end
        return true
    end,
    
    onTarget = function(item, user, target)
        item.data.onDidTarget(item, user, target)
        
        item:getAttacked(user, 1)
        --attacks.shootWebs(user, target, 3)
        
        -- knockback from gun
        --user.room:moveObject(user, getDir(user.x-target.x), getDir(user.y-target.y), .2)
    end,
    
}

local i = data.items

wands = {}
data.wands = wands
local wands = wands
local wandsByRarity = {
    common = {},
    rare = {},
    ultrarare = {},
    mythical = {}
}

function getWand(utility, name, color, onTarget, wandID, rarity, charges, chargeIncrement, postMake)
    --[[
        rarity (%):
            1 - common (50%)
            2 - rare (30%)
            3 - ultrarare (15%)
            4 - almost unseen (5%)
        
        chargeType (level 1 charge, increment):
            1 - lots and lots of charge (10, 10)
            2 - not too much charge (4, 3)
            3 - basic charge (3, 1)
            4 - stingy (1, 1)
    ]]
    
    local p = lume.copy(wand)
    p.name = name
    --p.color = color
    p.pcolor = color
    p.onDidTarget = onTarget or null
    
    p.charges = charges or 3
    p.chargeIncrement = chargeIncrement or 1
    p.rarity = rarity or "rare"
    
    p.isWand = true
    p.utility = utility
    
    table.insert(wandsByRarity[p.rarity], p)

    p.light = {2, color}
    
    if postMake then
        postMake(p)
    end
    
    wands[wandID] = p
    
    return p
end


local webs = function(item, user, target)
    return item.grade > 2 and attacks.shootCompactWebs(user, target, item.grade+1, math.floor(item.grade/2)) or attacks.shootWebs(user, target, item.grade+1)
end

local lightning = function(item, user, target)
    attacks.lightning(user, target, item.grade*2)
end

local teleport = function(item, user, target)
    attacks.teleportationBlast(user, target)
end

local causeDiscord = function(item, user, target)
    attacks.blast(user, target, "discord", item.grade)
end

local flames = function(item, user, target)
    attacks.flame(user, target, 2)
end

local ice = function(item, user, target)
    attacks.iceShot(user, target, item.grade)
end

local fL = function(item, user, target)
    attacks.fireLightning(user, target)
end

local polymorph = function(item, user, target)
    attacks.polymorphBlast(user, target, (item.grade*item.grade) + 1)
end

local invisible = function(item, user, target)
    attacks.blast(user, target, "invisible", item.grade, "white")
end

local immolate = function(item, user, target)
    attacks.fireLightning(user, target)
    attacks.shootWebs(user, target, 3)
end

local poisonLevels = {2,3,4,5,7,8}

local poison = function(item, user, target)
    attacks.poisonBlast(user, target, poisonLevels[item.grade] or item.enchants+1)
end

local createGas = function(item, user, target)
    attacks.gasBlast(user, target, item.grade)
end

local hasten = function(item, user, target)
    attacks.blast(user, target, "haste", item.grade, "orange")
end

local explode = function(item, user, target)
    attacks.shootExplosion(user, target, item.grade)
end

local blink = function(item, user, target)
    attacks.blink(user, target)
end

local beckon = function(item, user, target)
    attacks.beckon(user, target)
end

local conjure = function(item, user, target)
    attacks.shootSummon(user, target, item.grade, "spectral_blade")
end

local causeFear = function(item, user, target)
    attacks.blast(user, target, "fear", item.grade, "grey")
end

local negate = function(item, user, target)
    attacks.fireLightning(user, target)
    attacks.shootWebs(user, target, 3)
end

local causeDeath = function(item, user, target)
    if target.applyBuff then
        target:applyBuff("quietus",1,1) -- attacks.fireLightning(user, target)
    else
        user:addLog("The blast was wasted...")
    end
    
    --attacks.shootWebs(user, target, 3)
end

local heal = function(item, user, target)
    attacks.healBlast(user, target, item.grade*5)
end

local tunnel = function(item, user, target)
    attacks.tunnel(user, target, item.grade*3+1)
end

local obstruct = function(item, user, target)
    attacks.fireLightning(user, target)
    attacks.shootWebs(user, target, 3)
end

local mindSwitch = function(item, user, target)
    attacks.mindSwitchBlast(user, target, item.grade*3+2)
end

local switchPlace = function(item, user, target)
    attacks.switchPlaceBlast(user, target)
end

local enslave = function(item, user, target)
    attacks.enslavementBlast(user, target, item.grade*3+2)
end

local shakeShakeShake = function(p)

end

--[[
        rarity (%):
            1 - common (50%)
            2 - rare (30%)
            3 - ultrarare (15%)
            4 - mythical (5%)
        
        chargeType (level 1 charge, increment):
            1 - lots and lots of charge (10, 10)
            2 - not too much charge (4, 3)
            3 - basic charge (3, 2)
            4 - lower charge (3, 1)
            5 - little charge (2, 1)
            6 - stingy (1, 1)
]]

function getRandomWand(rarity, v) local m
    if not rarity then
         m = math.random()* (v or 100)
        if m <= 50 then
            rarity = "common"
        elseif m <= 80 then
            rarity = "rare"
        elseif m <= 96 then
            rarity = "ultrarare"
        else
            rarity = "mythical"
        end
    end
    
    return getValue(wandsByRarity[rarity])
end



--[[
Utility ([int] or function(user))
    4   - emergency
    3   - instant saving grace
    2   - pretty good advantage
    1   - better than nothing
    0   - meh
    -1  - hmph, devious
    -2  - wow, that's bad
    -3  - pure evil. Very faily
    -4  - not useable on player
    
    
]]

-- 20 wands --(21)
-- utility, name, color, effect, wandID, rarity, charges, chargeIncrement, postMake
i.wand_of_webbing = getWand(1, "wand of webbing", "white", webs, "webbing", "common", 4, 3)
i.wand_of_lightning = getWand(2, "wand of lightning", "royalblue", lightning, "lightning", "common", 4, 3)
i.wand_of_teleportation = getWand(4, "wand of teleportation", "yellow", teleport, "teleportation", "rare", 3, 2)
i.wand_of_discord = getWand(-4, "wand of discord", "red", causeDiscord, "discord", "rare", 4, 3)
i.wand_of_flames = getWand(2, "wand of flames", "orange", flames, "flames", "common", 4, 3)
i.wand_of_freezing = getWand(1, "wand of freezing", "skyblue", ice, "freezing", "common", 4, 3)
i.wand_of_fire_lightning = getWand(2, "wand of pyro-electricity", "orangered", fL, "fire_lightning", "ultrarare", 4, 3)
i.wand_of_polymorphing = getWand(3, "wand of polymorphing", "green", polymorph, "polymorphing", "ultrarare", 3, 1)
i.wand_of_invisibility = getWand(-1, "wand of invisibility", "darkgray", invisible, "invisibility", "common", 4, 3)
i.wand_of_immolation = getWand(1, "wand of immolation", "hotpink", immolate, "immolation", "ultrarare", 3, 2)
i.wand_of_poisoning = getWand(3, "wand of poisoning", "purple", poison, "poisoning", "common", 4, 3)
i.wand_of_gas = getWand(1, "wand of gas", "olive", createGas, "gas", "ultrarare", 4, 1)
i.wand_of_hasting = getWand(-2, "wand of hasting", "plum", hasten, "hasting", "rare", 4, 1)
i.wand_of_explosion = getWand(1, "wand of explosion", "tan", explode, "explosion", "ultrarare", 3, 2)
i.wand_of_beckoning = getWand(0, "wand of beckoning", "skyblue", beckon, "beckoning", "common", 3, 2)
i.wand_of_blinking = getWand(1, "wand of blinking", "gray", blink, "blinking", "common", 3, 2)
i.wand_of_conjuring = getWand(2, "wand of conjuring", "lightpink", conjure, "conjuring", "common", 4, 3)
i.wand_of_fear = getWand(-4, "wand of fear", "black", causeFear, "fear", "rare", 4, 1, shakeShakeShake)
i.wand_of_negation = getWand(1, "wand of negation", "blue", negate, "negation", "rare", 4, 1)
i.wand_of_death = getWand(3, "wand of death", "deeppink", causeDeath, "death", "mythical", 3, 1)
i.wand_of_healing = getWand(-3, "wand of healing", "lime", heal, "healing", "ultrarare", 4, 2)
i.wand_of_tunneling = getWand(1, "wand of tunneling", "dirtbrown", tunnel, "tunneling", "common", 3, 2)

i.wand_of_switch_place = getWand(1, "wand of switch place", "cyan", switchPlace, "switching place", "common", 3, 2)
i.wand_of_mind_switch = getWand(1, "wand of mind-switching", "lightorange", mindSwitch, "mind-switching", "rare", 3, 2)
i.wand_of_enslavement = getWand(1, "wand of enslavement", "dirtbrown", enslave, "enslaving", "common", 3, 2)



-- i.wand_of_obstruction = getWand("wand of obstruction", "teal", obstruct, "obstruction", "common", 3, 2)