local traits = {}

local t = traits



t.quickhands = {
    events = {
        
    },
    
    color = "orange",
    info = "Throwing an item doesn't take a turn",
    once = function(self, user)
        user.quickhands = true
    end,
}

t.quickstep = {
    color = "orange",
    name = "Quickstep",
    canRepeat = true,
    info = "Become a little bit faster",
    once = function(self, user)
        user.stats.speed = user.stats.speed + .25
    end
}

t.bulk = {
    color = "green",
    info = "More max HP",
    canRepeat = true,
    once = function(self, user)
        user.totalHealth = user.totalHealth + 3--math.floor(user.totalHealth * .3)
        user.maxHealth = user.totalHealth
    end
}

t.vampirism = {
    color = "lightpink",
    canRepeat = true,
    info = "&colors.red Kills ~ have a chance to heal",
    events = {
        Kill = function(data, user, victim)
            if math.random() > .5 then
                user:heal(math.random(1,2))
                user:cry("&colors.red Vampirism")
            end
        end
    }
}

t.bloodlust = {
    color = "orange",
    canRepeat = true,
    info = "Increase speed temporarily after a kill",
    events = {
        Kill = function(data, user, victim)
            if true then
                user:applyBuff("haste",3,.5)
                user:cry("&colors.hotpink Bloodlust")
            end
        end
    }
}

t.triggerhappy = {
    color = "pink",
    name = "Trigger Happy",
    info = "Reload all guns after a kill",
    events = {
        Kill = function(data, user, victim)
            if true then
                user:heal(math.random(1,2))
                user:cry("&colors.pink Trigger Happy")
            end
        end
    }
}

t.ghost = {
    info = "Become invisible for a while after a kill",
    canRepeat = true,
    color = "white",
    events = {
        Kill = function(data, user, victim)
            if true then
                user:applyBuff("invisibile",7,1)
                user:cry("&colors.white Ghost")
            end
        end
    }
}

t.stress = {
    info = "Deal more damage when health is low",
    canRepeat = true,
    color = "red",
    once = function(self, user)
        local f = user.filterAttack
        user.filterAttack = function(self, val, victim)
            if f then
                val = f(self, val, victim)
            end
            
            val = val + (self.maxHealth - self.stats.health)*.5
            
            return val
        end
    end
}

t.hothead = {
    info = "Chance to set enemy on fire on attack",
    canRepeat = true,
    color = "darkorange",
    events = {
        Attack = function(data, user, victim)
            if math.random() > .5 then
                victim:applyBuff("burn",5,3)
                user:cry("&colors.darkorange Hothead")
            end
        end
    }
}

t.firepunch = {
    info = "Kills heal you when burning",
    canRepeat = true,
    color = "darkorange",
    events = {
        Kill = function(data, user, victim)
            if true and user.isBurning then
                user:heal(math.random(1,2))
                user:cry("&colors.darkorange Firepunch")
            end
        end
    }
}

t.allmight = {
    info = "Increase stats for first few turns of new map",
    canRepeat = true,
    color = "cyan",
    events = {
        
    }
}

t.strongarms = {
    info = "Melee kills heal",
    name = "Strong arms",
    canRepeat = true,
    color = "red",
    events = {
        Kill = function(data, user, victim, attacker)
            if attacker == user then
                user:heal(math.random(1,3))
                user:cry("&colors.red Strong Arms")
            end
        end
    }
}

t.dragonorb = {
    info = "Shortly increase one stat randomly on a kill",
    canRepeat = true,
    color = "white",
    name = "Dragon Orb",
    events = {
        Kill = function(data, user, victim)
            if true then
                local r = math.random()
                
                if r > .8 then
                    user:applyBuff("haste",5,1)
                elseif r > .5 then
                    user:applyBuff("might",5,2)
                elseif r > .3 or true then
                    user:applyBuff("protect",3,1)
                end
                
                user:cry("&colors.orange Dragon Orb")
            end
        end
    }
}

t.hotspot = {
    info = "Chance to burst into flames when shot",
    canRepeat = true,
    color = "darkorange",
    events = {
        Damaged = function(data, user, attacker)
            if math.random() > .6 then
                user:addLog(string.format("%s burst into flames!",user.title))
                Fire:new({
                    tile = user.tile,
                    intensity = user.hotspot + 1,
                    range = 2 + user.hotspot,
                    forceSpread = true
                })
                
                user:cry("&colors.darkorange Hotspot!")
            end
        end
    },
    
    once = function(self, user)
        user.hotspot = (user.hotspot or 0)+1
    end
}

t.fireforce = {
    info = "Killed enemies burst into flames",
    canRepeat = true,
    name = "Fire Force",
    color = "darkorange",
    events = {
        Kill = function(data, user, victim)
            if math.random() > .6 and victim.tile then
                user:addLog(string.format("%s burst into flames!",victim.title or victim.name or "something"))
                Fire:new({
                    tile = victim.tile,
                    intensity = user.fireforce + 1,
                    range = 2 + user.fireforce,
                    forceSpread = true
                })
                
                user:cry("&colors.darkorange Fire force!")
            end
        end
    },
    
    once = function(self, user)
        user.fireforce = (user.fireforce or 0)+1
    end
}

t.flamingFeet = {
    info = "Become a little faster when burning",
    canRepeat = true,
    color = "orange",
    name = "Flaming Feet",
    once = function(self, user)
        user.flamingFeet = (user.flamingFeet or 0)+1
    end
}

t.hephestusheat = {
    info = "Deal more damage if burning",
    canRepeat = true,
    color = "red",
    name = "Hephestus' heat",
    
    once = function(self, user)
        local f = user.filterAttack
        user.filterAttack = function(self, val, victim)
            if f then
                val = f(self, val, victim)
            end
            
            if self.isBurning then
                val = val + math.random(1,3)
                self:cry("&colors.red Hephestus' Heat")
            end
            
            return val
        end
    end
}

t.thunderstep = {
    info = "Deal damage to nearby units after teleporting",
    canRepeat = true,
    color = "darkorange",
    events = {
        Teleport = function(data, user, victim)
            if true and user.isBurning then
                user:heal(math.random(1,2))
                user:cry("&colors.darkorange Firepunch")
            end
        end
    }
}

t.tacticalretreat = {
    info = "Teleport a short distance on kill",
    name = "Tactical Retreat",
    canRepeat = false,
    color = "darkorange",
    events = {
        Kill = function(data, user, victim)
            local t
            local count = 100
            while not t do
                count = count  - 1
                t = user.room:getRandomSpaceTile()
                local dis = lume.distance(t._x, t._y, user.tile._x, user.tile._y)
                if (dis > 15 or dis<2) and count > 0 then
                    t = nil
                end
            end
            
            user.room:teleport(user, t)
        end
    }
}

t.sturdy = {
    info = "Can't be knocked back",
    canRepeat = false,
    color = "darkorange",
    once = function(self, user)
        user.isSturdy = true
    end
}

t.joykill = {
    info = "Increase a random stat @permanently ~ on kill",
    canRepeat = true,
    color = "darkorange",
    events = {
        Kill = function(data, user, victim)
        
            if true then
                local r = math.random()
                
                if r > .9 then
                    user.stats.speed = user.stats.speed + .2
                    user:cry("&colors.orange :) speed+")
                elseif r > .5 then
                    user.stats.strength = user.stats.strength + .25
                    user:cry("&colors.cyan :) strength+")
                elseif r > .3 then
                    user.stats.defense = user.stats.defense + .25
                    user:cry("&colors.lightbrown :) defense+")
                else
                    user.totalHealth = user.totalHealth + .5--math.floor(user.totalHealth * .3)
                    user.maxHealth = user.totalHealth
                    user:cry("&colors.lime :) Max Hp+")
                end
                
                -- user:cry("&colors.orange Dragon Orb")
            end
            
        end
    }
}

t.teleheal = {
    info = "Heal a little after teleporting",
    canRepeat = true,
    color = "lime",
    events = {
        Teleport = function(data, user, victim)
            user:heal(math.random(1,2))
            user:cry("&colors.lime Teleheal")
        end
    }
}

t.bombsquad = {
    info = "Get a grenade on kill",
    canRepeat = true,
    color = "darkorange",
    events = {
        Kill = function(data, user, victim)
            if true then
                user:forceAddItem("grenade")
                user:cry("&colors.darkorange Bombsquad")
            end
        end
    }
}


return traits