local Fire = class:extend("Fire")
local nnt = 2
local fireLight = colors.orange--{1/nnt,.7/nnt,.7/nnt}
_G.fireLight = fireLight

function Fire:__init__(kwargs, source)
    self.uuid = lume.uuid()
    
    self.map = kwargs.map or toybox.room
    
    self.lifeBonus = kwargs.lifeBonus or -1
    self.updateFunc = kwargs.update or kwargs.updateFunc or null
    self.onFinish = kwargs.onFinish or null
    
    self.intensity = kwargs.intensity or 3
    self.rate = kwargs.rate or 1
    self.color = kwargs.color or "green"
    
    -- range: how far fire can spread each turn, or if explode how big explosion is
    self.range = kwargs.range or 2
    
    self.corrosive = kwargs.corrosive
    self.onlyCorrosive = kwargs.onlyCorrosive
    
    self.affectedTiles = {}
    self.affectedAmount = 0
    
    self.gasSource = kwargs.start or kwargs.source or kwargs.gasSource or kwargs.tile or source
    self.intensity = self.intensity+self.gasSource:getFlammability()
    self.explosionDamage = kwargs.explosionDamage or lume.max(self.explode and 10 or 10, self.intensity)
    self.explode = kwargs.explode
    
    self.forceSpread = kwargs.forceSpread or false
    
    self:addTile(self.gasSource, {self.intensity}, self.forceSpread and .1, self.forceSpread and self.gasSource)
    
    self.forceSpread = nil
    
    self.startTimer = kwargs.startTimer or 1
    
    self.dead = false
    self.alive = true
    
    self.room = toybox.room

    self.depth = DEPTHS.EFFECTS
    self.gased = {}
    
    self.id = lume.uuid()
    toybox.room:store_instance(self)
    self.room:must_update(self)
    self.room:addFire(self)
    self.room:must_draw(self)
    self.updateDt = 0
    
    self._currentFrame = 0
    self.currentFrame = 0
    
    self.timer = self.room.timer
end

function Fire:__draw()
    local done
    for i = 1, self.affectedAmount do
        local tile = self.affectedTiles[i]
        if tile then
            self:drawFire(tile, tile.gases[self.uuid])
        end
    end
end

gasit = function(i,c,r,rr) Fire:new({lifeBonus=rr or -3,rate=r, map=self, start=self.player.tile, intensity=i, color=c}) end
function Fire:destroy()
    assert(not self.dead)
    self.room:removeFire(self)
    self.onFinish(self)
    self.dead = true
    self.alive = false
end

function Fire:__step(dt)
    if self.startTimer>0 then
        self.startTimer = self.startTimer-dt
        --return
    end
    
    self._currentFrame = self._currentFrame + dt*(1/.15)
    self.currentFrame = math.floor(self._currentFrame)
    
    self.updateDt = self.updateDt - dt
    if self.updateDt <= 0 then
        self.updateDt = .3
    else
        return
    end
end

function Fire:updateTurn()
    
    local done
    for i = 1, self.affectedAmount do
        local tile = self.affectedTiles[i]
        if tile and not tile.isCreature then
            self.updateFunc(self, tile)
            self:updateFire(dt,tile)
            done = true
        end
    end
    if not done then
        self:destroy()
    end
end

function Fire:drawFire(tile,gas)
    
    if tile.drawID ~= tile.room.drawID then
        return
    end
    
    local source = "setpieces/flame"
    local maxFrames = 4
    if (gas[1]/self.intensity)<.5 or true then
        maxFrames = 5
        source = "setpieces/fire"
    end
    
    local currentFrame = self.currentFrame%maxFrames
    local cr = currentFrame + (gas[4]==1 and 0 or gas[4])
    
    while cr > (maxFrames-1) do
        cr = 0+(cr-(maxFrames))
    end
    while cr < 0 do
        cr = 0
    end
    if true then
        gas.xx = (gas.ooxx or 0)+math.random(-20,20)
        gas.oldC = cr
    end

    local source = string.format("%s/%s.png",source, cr)
    local img = game:getAsset(source)
    local _w, _h = resizeImage(img, tw, th)
    local xx = gas.xx*0
    local yy = 5*0
    lg.draw(img, tile.x+xx, tile.y+math.random(-yy,yy), 0, _w, _h)
    
    local r,g,b,a = love.graphics.getColor()
    
    love.graphics.setColor(r,g,b,a)
end

local explosion = {
    attack = 10,
    name = "an explosion",
    isExplosion = true,
    class = {
        name = "Explosion"
    }
}

local dirs2 = {{-1,0},{1,0},{0,1},{0,-1},
{1,1},{1,-1},{-1,-1},{-1,1}
}
local dirs = {{-1,0},{1,0},{0,1},{0,-1}}
local explosionRange = 2

function Fire:updateFire(dt, tile)
    local val = tile.gases[self.uuid][1]-self.rate*1
    local gas = tile.gases[self.uuid]
    local alpha = gas[1]/self.intensity
    
    if self.lifeBonus~=0 and dt~=0 and (tile.unit or tile.item) then
      
        if self.lifeBonus<0 then
            if tile.unit and tile.unit.getAttacked then
                self.attack = alpha * self.lifeBonus*-1
                tile.unit:getAttacked(self)
                tile.unit:applyBuff("burn",math.floor(math.max(1,gas[1]))+3)
            end
            
            if tile.item and tile.item and not tile.item.isFireproof then
                self.attack = alpha * self.lifeBonus*-1 * (1-(tile.item.fireResistance or .5))
                tile.item:applyBuff("burn",math.floor(math.max(1,gas[1]))+2)
                tile.item:getAttacked(self)-- = tile.item.life - alpha*self.lifeBonus
                elseif tile.item and tile.item.isTree then er() 
            end
            
        elseif tile.unit and tile.unit.life then
            tile.unit:heal(self.lifeBonus*alpha)
            if tile.item and tile.item.life and not tile.item.isFireproof then
                tile.item.life = tile.item.life + alpha*self.lifeBonus
            end
        end
    end
    
    -- chance to force spread
    if val <= 0 and math.random()>.4 then
        gas.dead = true
    end
    
    if tile.grass and (tile.grass.flammable or (tile.grass.flammability or 0)>0) then
        tile.grass = nil
    end
    
    if tile.web then
        tile.web = nil
    end
    
    if not gas[2] or gas[2]==1 or gas.dead then log("***try spread")
        if val>0 and gas[2] ~= 1 and not self.explode and gas[1]<=2 then
            gas[2] = 1
        elseif tile.fires<=1 then
            self:spread(tile, gas)
            gas[2] = true
            log("===try done")
        end
    end
    
    if val >= 100 or self.explode then
        --warm("exploding at "..val)
        gas.exploded = toybox.room.currentTurn
        Explode(tile.x+tw/2,tile.y+th/2)
        gas[1] = 0
        explosion.attack = self.explosionDamage
        if tile.unit and tile.unit.getAttacked then
            tile.unit:getAttacked(explosion)
            log("attacked "..tostring(tile.unit))
        end
        
        if tile.item and tile.item and not tile.item.isFireproof then
            tile.item:getAttacked(explosion)
        end
        
        if tile.solid and self.explode then
            tile:getAttacked(explosion.attack/(tile.isInside and 5 or 1), explosion)
            tile.isInside = false
        end
        
        -- to intensive
        if doRangeFalse and nil then
        local done = false
        for xx = 1, 4 do
            local d = dirs[xx]
            local t = tile.room:getTile(tile._x+d[1], tile._y+d[2])
            -- true means that tile is not surrounded by fire that's about to explode
            if t and not t.gases[self.uuid] then
                if t:getFlammability()<=0 then
                    done = true
                    break
                end
            end
        end
        
        for r = 1, done and explosionRange or 0 do
            for dd = 1,#dirs2 do
                local d = dirs2[dd]
                local t = tile.room:getTile(tile._x+d[1]*r, tile._y+d[2]*r)
                Explode(tile.x+tw/2,tile.y+th/2)
                if t.unit and t.unit.getAttacked then
                    t.unit:getAttacked(explosion)
                end
            end
        end
        
        end
            
        val = 0
    end
    
    if val<=0 then
        self:releaseTile(tile)
    else
        tile.gases[self.uuid][1] = val
    end
end


function Fire:spread(ntile, gas, og)
    local tooFar = lume.distance(ntile._x,ntile._y,self.gasSource._x,self.gasSource._y)>self.range*(ntile.solid and .7 or 1)
    
    
    
    if tooFar and not self.explode then
        return
    end
    
    if gas[1]<.5 then
        --return
    end
    
    local ogg = self.gasSource 
    for x = 1, #dirs do
        local dir = dirs[x]
        local x, y = ntile._x+dir[1], ntile._y+dir[2]
        local tile = self.map:getTile(x, y)
        
        local exp = self.explode and 10 or 0
        
        local burn = tile and (
            --tile.unit and tile.unit:isFlammable() or
            tile.item and (tile.item.flammable or ((tile.item.flammability or 0)>exp and tile.item.flammability))
            or (tile.grass and ((tile.grass.flammability or 0)>exp and tile.grass.flammability))
            or (tile.web and not self.explode)
            or self.explode and lume.distance(ogg._x,ogg._y,tile._x,tile._y)<=(self.range) and (log("expppp") or 1)
        )
        
        local fl = type(burn) == "number" and burn or 0
        local ffl = fl
        fl = 0
        for x, i in pairs(tile and tile.gases or {}) do
            local f = i.flammability or i[3].flammability
            if f and f > (self.explode and 0 or 0) and x ~= self.uuid and i[1] > .01 and (not i[3].explode) then
                fl = lume.max(fl, f)--fl + f
                burn = true
                i[1] = .01--i[1]/10
                i[3]:releaseTile(tile)
                i[2] = true
            end
            --if i.exploded==toybox.room.currentTurn then burn = false break end
        end
        fl = fl + ffl
        
        -- spread fast if gas (type(burn) ~= "number") or self.explode or almost dead
        if (
            tile and tile.fires<3 and
            (not tile.solid or self.explode) and burn and
                (gas.dead or self.explode or
                type(burn)~="number" or fl==0 or math.random(40)<(fl*10)
                )
            )
            or
        self.forceSpread then
            if ((not self.explode or self.explode and fl>=2) or (not tooFar)) and tile then
                gas[6] = lume.max(fl, 2)
                tile.room.player.cry(tile, fl.."?"..gas[1])
                self:addTile(tile,gas,fl,og or ntile)
            elseif lume.distance(tile._x,tile._y, self.gasSource._x, self.gasSource._y)<(self.range+1) then
                self.noSpread = 1
                Fire:new({
                    source = tile,
                    range = 2,
                    intensity = self.intensity
                })
            end
            
        end
    end
    self.spreadV = (self.spreadV or 0)+.2
end

function Fire:addTile(tile, ngas, added, og)
    if tile.solid and not self.explode then
        return
    end
    
    
    if not tile.gases[self.uuid] then log("spread! "..(tile.fires+1))
        tile:play_sound("catch_fire")
        self.affectedAmount = self.affectedAmount + 1
        self.affectedTiles[self.affectedAmount] = tile
        tile.gases[self.uuid] = {0,nil,self,math.random(0,3)}
    
        local gas = tile.gases[self.uuid]
        gas[1] = gas[1] + ngas[1]*.9 + (added or 0)
        tile.gasesAmount = tile.gasesAmount+1
        
        if tile.fires>100 then error("Too many fires...") end
        
        tile.fires = tile.fires + 1
        tile:increaseTemperature(gas[1]/2)
        
        -- don't spread too far too fast, works for fire, not explosions (look at range for that)
        if added and added > 0 and lume.distance(og._x,og._y,tile._x,tile._y)<=(ngas[6] or self.range) then
            log("aspr?") self:spread(tile, gas, og) log("asspp")
        end
        
        if og and og.solid then
            tile.isInside = true
        end
        
        if not self.explode then
            local nnt = 1
            gas[5] = tile.room:addLight(tile,3,fireLight)
        end
        
        
        if gas[1]>=10 or self.explode then
            log(self.explode and "=== explooo" or "=== high gas power")
            self:updateFire(0, tile)
        end
    end
end

function Fire:releaseTile(tile)
    local gas = tile.gases[self.uuid]
    if gas then
        self.affectedAmount = self.affectedAmount - 1
        lume.remove(self.affectedTiles, tile)
        --tile.gases[self.uuid] = nil
        tile.gasesAmount = tile.gasesAmount-1
        tile.fires = tile.fires - 1
        gas.dead = true
        if gas[5] then
            tile.room:removeLight(gas[5])
        end
    end
end

function Fire:draw()
end

return Fire