local Gas = class:extend("Gas")


function Gas:__init__(kwargs, source)
    if kwargs.isFire then
        return Fire:new(kwargs, source)
    end
    
    self.gasSource = kwargs.start or kwargs.source or kwargs.gasSource or kwargs.tile or source
    
    if not self.gasSource then return end -- improper position
    
    self.gasSource:play_sound("steam")
    
    self.uuid = lume.uuid()
    
    self.map = kwargs.map or toybox.room
    
    self.lifeBonus = kwargs.lifeBonus or -1
    self.updateFunc = kwargs.update or kwargs.updateFunc or null
    self.onFinish = kwargs.onFinish or null
    
    self.intensity = kwargs.intensity or 3
    self.flammability = kwargs.flammability or kwargs.flammable or .5
    self.rate = kwargs.rate or 1
    self.color = kwargs.color or "green"
    self.range = kwargs.range or 5
    self.corrosive = kwargs.corrosive
    self.onlyCorrosive = kwargs.onlyCorrosive
    
    self.buff = kwargs.buff
    self.buffIntensity = kwargs.buffIntensity or self.intensity
    self.buffTime = kwargs.buffTime
    
    self.affectedTiles = {}
    self.affectedAmount = 0
    
    self:addTile(self.gasSource, {self.intensity})
    self.startTimer = kwargs.startTimer or 1
    
    self.dead = false
    self.alive = true
    
    self.room = toybox.room

    self.depth = DEPTHS.EFFECTS
    self.gased = {}
    
    self.id = lume.uuid()
    
    self.name = kwargs.name or "gas"
    
    toybox.room:store_instance(self)
    self.room:must_update(self)
    self.room:addGas(self)
    self.updateDt = 0
    
    self.timer = self.room.timer
end

function Gas:__draw()
end

gasit = function(i,c,r,rr) Gas:new({lifeBonus=rr or -3,rate=r, map=self, start=self.player.tile, intensity=i, color=c}) end
function Gas:destroy()
    assert(not self.dead)
    self.room:removeGas(self)
    self.onFinish(self)
    self.dead = true
    self.alive = false
end

function Gas:__step(dt)
    if self.startTimer>0 then
        self.startTimer = self.startTimer-dt
        --return
    end
    
    self.updateDt = self.updateDt - dt
    if self.updateDt <= 0 then
        self.updateDt = .3
    else
        return
    end
    SMOKE_DEBRIS=3
    --DEBRIS_IN = .7
   -- DEBRIS_SOURCE = "setpieces/debris/0.png"
    for i = 1, self.affectedAmount do
        local tile = self.affectedTiles[i]
        if not self.gased[tile] and tile:canBeSeen() then
            local x = {tile.x, tile.x+tile.w}
            local y = {tile.y, tile.y+tile.h}
            local w,g,vx,vy
            vx, vy = {-5,5}
            STORE_DEBRIS = {}
            vy =vx
            local life = 10000
            local va = {100,300}
            local sc = {1,1,.52,1.2,1.4,.5}
            Debris.spawn(x,y,self.color,nil_source,getValue(4,5),w,h,vx,vy,life,sc,va)
            self.gased[tile] = STORE_DEBRIS
            STORE_DEBRIS = nil
        end
    end
    SMOKE_DEBRIS=nil
    DEBRIS_IN = false
    DEBRIS_SOURCE = nil
end

function Gas:updateTurn()
    
    local done
    for i = 1, self.affectedAmount do
        local tile = self.affectedTiles[i]
        if tile then
            self.updateFunc(self, tile)
            self:updateGas(dt,tile)
            done = true
        end
    end
    if not done then
        self:destroy()
    end
end

function Gas:drawGas(tile,gas)
    local r,g,b,a = love.graphics.getColor()
    local alpha =  (gas[1]/self.intensity)*2*(nnaa or .5)
    local color = getColor(self.color, alpha)
    love.graphics.setColor(color)
    love.graphics.rectangle("fill",tile._x,tile._y, tile.w, tile.h)
    love.graphics.setColor(r,g,b,a)
end

function Gas:updateGas(dt, tile)
    local val = tile.gases[self.uuid][1]-self.rate*1
    local gas = tile.gases[self.uuid]
    local alpha = gas[1]/self.intensity
    
    if self.lifeBonus~=0 and (
      (self.onlyCorrosive and tile.unit and not tile.unit.isLiving) or 
      self.corrosive or 
      (tile.unit and tile.unit.isLiving)
    ) then
      
        if self.lifeBonus<0 and tile.unit and tile.unit.getAttacked then
            self.attack = alpha * self.lifeBonus*-1
            tile.unit:getAttacked(self)
        elseif tile.unit and tile.unit.life then
            tile.unit.life = tile.unit.life + self.lifeBonus*alpha
        end
    end
    
    if tile.unit and self.buff then
        tile.unit:applyBuff(self.buff, self.buffTime, self.buffIntensity*alpha)
    end
    
    if not gas[2] then
        self:spread(tile, gas)
        gas[2] = true
    end
    
    if val<=0 then
        self:releaseTile(tile)
    else
        tile.gases[self.uuid][1] = val
        -- only activley condense if below normal temperature
        if tile:getTemperature() <= 5 then
            condenseGas(tile, gas)
        end
    end
end

local dirs = {{-1,0},{1,0},{0,1},{0,-1},
{1,1},{1,-1},{-1,-1},{-1,1}
}

function Gas:spread(ntile, gas)
    if lume.distance(ntile._x,ntile._y,self.gasSource._x,self.gasSource._y)>self.range then
        return
    end
    
    if gas[1]<.5 then
        return
    end
    
    for x = 1, #dirs do
        local dir = dirs[x]
        local x, y = ntile._x+dir[1], ntile._y+dir[2]
        local tile = self.map:getTile(x, y)
        if tile and not tile.solid then
            self:addTile(tile,gas)
        end
    end
end

function Gas:addTile(tile, ngas)
    if not tile.gases[self.uuid] and tile.fires<=0 and tile then
        self.affectedAmount = self.affectedAmount + 1
        self.affectedTiles[self.affectedAmount] = tile
        tile.gases[self.uuid] = {0,nil,self}
    
        local gas = tile.gases[self.uuid]
        gas[1] = gas[1] + ngas[1]*.9
        tile.gasesAmount = tile.gasesAmount+1
    end
end

function Gas:releaseTile(tile)
    local gas = tile.gases[self.uuid]
    if gas then
        self.affectedAmount = self.affectedAmount - 1
        lume.remove(self.affectedTiles, tile)
        --tile.gases[self.uuid] = 0
        gas.flammability = 0
        tile.gasesAmount = tile.gasesAmount-1
        local debris = self.gased[tile]
        self.gased[tile] = nil
        if debris then
            for x = 1, #debris do
                debris[x].ll = .3
                --debris[x]:destroy()
            end
        end
    end
end

function Gas:draw()
end

return Gas