local Item = Entity:extend("Item")

INV_DEPTH = {
    helmet = 100,
    armour = 90,
    weapon = 80,
    
    glasses = 79,
    
    ammo = 78,
    
    utility = 76,
    
    wand = 75,
    charm = 72,
    
    potion = 70,
    scroll = 60,
    food = 50,
    seed = 40,
    

}

itemLand = function(self, tile)
    self.room:spawnItem(self,tile)
end

itemBreak = function(self)
    self.user:removeItem(self, true)
end

local itemGetName = function(self)
    return self.name
end

local function enull()
end

local itemBreak = itemBreak
local onThrow = itemLand

local ostep = Entity.__step

function Item:create(k, fake)
    if fake then
        self.play_sound = enull
    end
    
    if type(k) ~= "table" then
        error(string.format("No item type <%s> ?",inspect(k)))
    end
    
    local pm = k.postMake
    k.postMake = nil
    
    Entity.create(self, k)
    Object.create(self, k)
    self.source = getValue(k.source) or self.source
    
    k.postMake = pm
    
    self.w = k.w or ((tw)/scale)
    self.h = k.h or ((th)/scale)
    self:set_box()
    
    self:center()
    local yy = self.offset_y
    self.offset_y = self.h>th and self.h == self.w and (yy - (self.h-th/2) + th) or 
        self.h<th and (self.offset_y+(th-self.h)/2) or self.h~=th and self.offset_y - (self.h-th)/2 or yy
    
    self.name = k.name or "stinky item"
    self.ogName = self.name
    self.title = k.title or string.format("the %s", self.name)
    
    self.article = data.article or "the"
    self.unspecificArticle = data.unspecificArticle or startsWithVowel(self.name) and "an" or "a"
    self.pronoun = data.pronoun or "it"
    self.pronoun2 = data.pronoun2 or "its"
    
    self.unspecific = data.unspecific or string.format("%s %s",self.unspecificArticle, self.name)
    self.titlePossessive = data.possessive or data.titlePossessive or string.format("%s's", self.title)
    
    self.getName = k.getName or itemGetName
    self.info = k.info or k.des
    if self.info then
        self.info = string.format("~ %s", self.info)
    end
    
    self._id = k._id or k.id
    
    -- whether to not adjust for smaller creatures (i.e. kobolds)
    self.small = k.small
    
    self.summoned = {}
    
    self.mixes = k.mixes or k.mix or {}
    self.mix   = self.mixes
    
    self.amount = k.amount or nil
    self.item   = k.item or nil -- item to actually collect
    
    self.__onUse = k.onUse or null
    self.__onThrown = k.onThrown or k.onThrow or onThrow
    self.onThrow  = self.onThrown
    
    self.onCooked = k.onCooked
    self.onBurnt = k.onBurnt
    self.onRotted = k.onRotted


    self.onDestroyed = k.onDeath or k.onDestroyed  or k.onDie or null
    self.onDamaged = k.onDamaged or k.onAttacked or null
    
    self._update = k.update
    self.onDrop = k.onDrop or null
    self.onCarry = k.onCarry or k.onPickup or null
    --self.onBreak = k.onBreak or itemBreak
    
    self.onEnchant = k.onEnchant or k.onEnchanted or null
    
    self.anvil = k.anvil
    self.isAnvil = k.isAnvil
    self.onRecieve = k.onRecieve or null
    
    self.onSteppedOn = k.onSteppedOn
    
    self.canBurn = k.canBurn or false
    
    self.flammability = k.flammability or 0
    
    self.boilingPoint = k.boilingPoint or nil
    self.boilTo = k.boilTo or nil
    self.toBoil = math.random(2,4)
    
    self.life = k.life
    self.space = k.space
    
    self.burnLife = k.burnLife
    self.life = self.burnLife or self.life
    
    self.burning = k.burning or false
    
    self.kayoteTime = k.kayoteTime or 0
    
    self.onlyFireDamages = k.onlyFireDamages or self.burnLife
    
    self.nutrition = k.nutrition or 0
    
    self.isCooked = k.isCooked
    self.isBurnt  = k.isBurnt
    self.isRotten = k.isRotten
    
    self.cookedItem = k.cookedItem
    self.rottedItem = k.rottedItem
    self.burntItem  = k.burntItem
    
    self.freshness = k.freshness or 10
    self.burntValue = k.burntValue or lume.max(self.nutrition+1, 3)
    self.cookedValue = k.cookedValue or lume.max(self.nutrition/2, 1)
    
    self.isGun = k.isGun
    self.shootAmount = k.shootAmount
    self.shootAngle = k.shootAngle
    self.bullet = k.bullet
    self.accuracy = k.accuracy or 5
    
    self.price = k.price or 25
    
    
    self.durability = getValue(k.durability or -1)/1.5
    self.showDurability = k.showDurability or k.durability
    self.maxDurability = self.durability
    
    self.wieldAngle = k.wieldAngle
    
    
    self.unitBlocks = k.unitBlocks or k.isObstacle or false -- true means unable to land on unit tile
    self.isObstacle = k.isObstacle or false
    
    self.noPickup = k.noPickup or (k.canPickup == false) or false
    
    self.isThrowObstacle = k.isThrowObstacle or self.isObstacle or false
    
    self.attack = k.attack or 0
    
    self.lungeAttack = k.lungeAttack or k.lunge
    self.attackRecovery = k.attackRecovery or k.useRecovery or 0
    self.attackKnockback = k.attackKnockback or 0
    self.pierce = k.pierce
    self.meleeRange = k.meleeRange

    self.__getAttack = k.getAttack
    if not self.__getAttack then
        self.getAttack = false
    end
    
    self.isWeapon  = k.isWeapon
    self.isScroll  = k.isScroll
    self.isPotion  = k.isPotion
    self.isSeed    = k.isSeed
    self.isCharm   = k.isCharm
    self.isWand    = k.isWand
    self.isArmour  = k.isArmour
    self.isHelmet  = k.isHelmet
    self.isGlasses = k.isGlasses
    self.isFood   = k.isFood
    self.isAmmo   = k.isAmmo
    self.isUtility = k.isUtility
    self.isWall   = k.isWall
    self.isFloor  = k.isFloor
    
    assert(not k.isUtlity and not k.utlity)
    
    self.itemType =
    
        self.isWeapon and "weapon" or
        self.isScroll and "scroll" or
        self.isPotion and "potion" or
        self.isArmour and "armour" or
        self.isHelmet and "helmet" or
        self.isGlasses and "glasses" or
        self.isCharm and "charm" or
        self.isWand and "wand" or
        self.isSeed and "seed" or
        self.isFood and "food" or
        self.isAmmo and "ammo" or
        self.isUtility and "utility" or
        
        self.isFloor and "floor" or
        self.isWall and "wall" or
        
        k.isShooter and "weapon" or
        k.bullet and "weapon" or
        
        "item"
    
    
    self.type = k.type or self.itemType
    k.itemType = self.itemType
    
    if k.utlity then
        error(string.format("Typo (utlity for *utlity) for item type %s!",self.type))
    end
    
    self.utility = k.utility or 0
    self.healing = k.healing or 0
    
    self.noDrop = k.noDrop or false
    
    self.isFireproof = k.isFireproof or false
    self.fireResistance = k.fireResistance or .5
    
    self.range = lume.max(self.meleeRange or 0, k.range or 0)
    self.isShooter = k.isShooter or k.shooter
    self.ammos = k.ammos or k.ammo
    
    self.throwDistance = k.throwDistance or nil
    
    self.onTarget = k.onTarget or null
    
    self.itemColor = k.color
    
    self.___updateTurn = k.updateTurn or null
    
    self.intensity = k.intensity or nil
    self.buffTime = k.buffTime or nil
    self.buff = k.buff or nil
    self.selfBuff = k.selfBuff or nil -- buff to apply on user if weapon
    
    self.light = k.light
    self.gas = k.gas and lume.copy(k.gas)
    self.noGasWhenHeld = k.noGasWhenHeld
    
    if self.gas then
        self.maxGas = k.maxGas or getValue(self.gas.every or 4)
        self.toGas = self.maxGas
    end
    
    self.isCarried = nil
    self.flip = k.flip
    
    self.isHelmet = k.isHelmet
    self.isArmour = k.isArmour
    
    self.isAmmo = k.isAmmo
    
    self.data = k
    
    self.buffs = {}
    self._enchants = {}
    
    self.enchanted = 0
    self.enchants = 0
    self.grade = k.grade or 1
    
    self.useText = k.useText or nil
    self.useDamage = k.useDamage or nil
    
    self.stats = k.stats or self.stats
    
    if (self.isWeapon and not self.isAmmo) or self.isWand then
        self:makeDifferentFromStack()
    end

 
    self.onDrawCard_list = {}
    self.onPlayCard_list = {}
    self.onGetBuff_list = {}
    self.onPlayBuff_list = {}
    self.onStartTurn_list = {}
    self.onEndTurn_list = {}
    self.onSummon_list = {}
    self.onDeath_list = {}
    
    self.eventLists = {
        "DrawCard",
        "PlayCard",
        "GetBuff",
        "PlayBuff",
        "StartTurn",
        "EndTurn",
        "Summon",
        "Death",
        "Enchanted"
    } log("new "..self.name)
    
    self.events = {}
    
    for x = 1, #self.eventLists do
        local ev = self.eventLists[x]
        self.events[ev] = {}
        
        
        -- add event: on<Event Name>
        
        self[string.format("on%s", ev)] = function(self, func, val, times, data)
            local list = self.events[ev]
            local data = data or {}
            list[#list+1] = {func, val or err(), times, data}
            data.data = {func, val, times}
            data.times = times
            
            return ev, list[#list]
        end
        
        
        --play the event: play<Event Name>
        
        self[string.format("play%s", ev)] = function(self, ...)
            local done
            local list = self.events[ev]
            for x = 1, #list do
                local data = list[x]
                if data then
                    data[1](data[4], self, data[2], ...)
                    done = data[1]~=null or done
                    data[4].times = data[4].times-1--data[3]-1
                    --data[4].times = data[3]
                    if data[4].times == 0 then
                        table.remove(list, x)
                        if data[4].kill then
                            data[4].kill(data[4], self, data[2], ...)
                        end
                        
                        if data[4].addedBuff then
                            lume.remove(self.buffs, data[4])
                        end
                    end
                end
            end
            
            return done
        end
        
    end
    
    self.currentFrame = 0
    
    if k.postMake then
        k.postMake(self)
    end
    
    if fake and self.play_sound == enull then
        self.play_sound = nil
    end

end

function Item:transformPotion(to)
    if not self.isPotion then
        return
    end
    
    local old = self:getName()
    
    local p = items[to] or error(string.format("No potion by name %s", to))
    
    self.name = p.name

    self.pcolor = p.pcolor
    self.intensity = p.intensity
    
    self.boilingPoint = p.boilingPoint
    self.boilTo = p.boilTo
    
    self.nutrition = p.utility
    
    self.buff = p.buff
    self.doGas = p.doGas-- == true and nil or gas
    self.light = p.light
    self.utility = p.utility
    
    self.explode = p.explode
    
    self.onUse = p.onUse
    self.onThrow = p.onThrow
    
    self.image1.color = lume.copy(getColor(p.pcolor))
    self.ogColor = getColor(p.pcolor)
    
    self._oldP_OnThrow = p.onThrow
    
    self._id = p._id

    self.ogName = self.name
    self.title = string.format("the %s", self.name)
    
    self.article = self.article or "the"
    self.unspecificArticle = self.unspecificArticle or startsWithVowel(self.name) and "an" or "a"
    self.pronoun = self.pronoun or "it"
    self.pronoun2 = self.pronoun2 or "its"
    
    self.unspecific = string.format("%s %s",self.unspecificArticle, self.name)
    self.titlePossessive = self.possessive or string.format("%s's", self.title)
    
    self:addLog(string.format("%s -changed to %s!", old, self:getName()))
    
    self:stop_shake()
end

function Item:isRevealed()
    if self.isScroll then
        return data.knownScrolls[self._id]
    elseif self.isPotion then
        return data.knownPotions[self._id]
    elseif self.isWeapon then
    
    end
    
    return true
end

function Item:play_sound(...)
    return Creature.play_sound(self, ...)
end

function Item:getDurabilityPercentage()
    return (self.durability/self.maxDurability)*100
end

function Item:repair(val)
    self.maxDurability = self.maxDurability + 5
    self.durability = lume.min(self.durability+val, self.maxDurability)
end

function Item:getDurabilityText()
    local p = self:getDurabilityPercentage()
    
    if self.lastLegDurability then
        return "@very fragile"
    end
    
    if p <= 10 or (self.durability < 1) then
        return "@fragile"
    elseif p <= 40 then
        return "worn"
    else
        return
    end
end

function Item:getDurabilityColor()
    local p = self:getDurabilityPercentage()
    
    if self.lastLegDurability then
        return "red"
    end
    
    if p <= 10 then
        return "red"
    elseif p <= 60 then
        return "orange"
    elseif p <= 110 then
        return "lime"
    end
    
end

function Item:getDurabilityTitle()
    if self.maxDurability > 1 then
        local col = self:getDurabilityColor()
        local txt = self:getDurabilityText()
        if txt then
            return string.format("&colors.%s %s ~ ", col, txt)
        end
    end
    
    return ""
end

function Item:updateTurn(...) log("item "..tostring(self.name)..","..tostring(self._id))
    self:___updateTurn(...)
    if self.__updateTurn then error("__updateTurn typo ... not ___updateTurn") end
    
    if self.showDurability and self.user then
        self.durabilityText = self:getDurabilityText()
        if self.durabilityText ~= self.oldDurabilityText then
            if self.user.isPlayer then
                local txt  = string.format("Your %s is now %s",self.name,self:getDurabilityTitle())
                self.user:addLog(txt)
                self.user:cry(txt)
                self.room:alert(txt)
                if self.durabilityText == "fragile" or self.lastLeg then
                    self.room.camera:_flash(.2,colors.red)
                end
            end
        end
        
        self.oldDurabilityText = self.durabilityText
    end
    
    if self.tile and self.tile.solid then if self.isWall then self.tile.solid = nil else
        return self.room:destroyItem(self) end
    end
    
    if self.gas and (not self.noGasWhenHeld or not self.user) then
        self.toGas  = self.toGas - 1
        
        if self.toGas <= 0 then
            local g = self.gas
            Gas:new(g, self.isCarried and self.user and self.user.tile or self.tile)
            
            self.toGas = self.maxGas
        end
    end
    
    if self.tile and (not self.tile.item or self.tile.item.noPickup) and not self.isDeadDead and not self.isCarried then
        self.tile.item = self
    end
    
    if self.tile and self.tile.item ~= self and self.tile.lightp and not self.isCarried then
    
        self.light_alpha = self.tile.lightp
        self.color = self.color and addc(getColor(self.color),getColor(self.tile.colorp),.5) or self.tile.colorp
        self.isSeen = self.tile:canBeSeen()
        if not self.isSeen then
            self.light_alpha = 0
        end
    end
    
    if self.tile and self.tile:isCold() then
        self.freshness = self.freshness - (1/self.grade)
    end
    
    if self.freshness <= 0 and self.rottedItem then
        return self:rot()
    end
    
    if self.tile and self.boilingPoint then
        local temp = self.tile:getTemperature()
        local bp = self.boilingPoint
        if temp >= bp[1] and temp <= bp[2] then
            self.toBoil = self.toBoil - 1 self:addLog("&colors.orange "..temp..","..self.tile.temperature)
            
            if self.doBoil then
                self:doBoil()
                self.doBoil = nil
            end
            
            if self.toBoil <= 0 then
                if self.isPotion and self.boilTo then
                    self:transformPotion(self.boilTo)
                end
                self.toBoil = 3
                self.boilingPoint = nil
                self.tile:decreaseTemperature(2*self:getDir(bp[1]))
                self.sprite_2 = nil
            end
        else
            self.toBoil = lume.max(self.toBoil, 2)
        end
    end
    
end

function Item:onUse(target, ...)

    if self.useText == "read" and target.cantRead then
        target:addLog(string.format("%s can't read!", target.title))
        return
    end
    
    local r = self:__onUse(target, ...)
    if self.useText ~= "equip" then
        self:checkEnchants(target)
    end
    
    if self.useDamage then
        self:getAttacked(target, self.useDamage)
    end
    
    if self.user then
        self.user:wieldItem(self)
    end
    
    if self.isPotion and self.user then
        revealPotion(self.user, self)
    end
    return r
end

function Item:onThrown(target, ...)
    if self.user and not self.shot then
        self.user:addLog(string.format("%s threw %s",self.user.title, self.unspecific))
    end
    
    local r = self:__onThrown(target, ...)
    
    return r
end


function Item:getAttack(user, victim)
    local r = self:__getAttack(user, victim)
    
    return r
end

function Item:revealName()
    if self.revealedName then
        return
    end
    
    local e = self.enchanted
    
    local ss = string.format("%s%s%s",self.grade>0 and "&colors.cyan " or self.grade<=0 and "&colors.red " or "", self.grade>1 and "+" or "",self.grade>1 and (self.grade-1) or self.grade==1 and "" or self.grade-1)
    local s
    local name = self.ogName
    
    for b, data in pairs(self._enchants) do
        local buff = buffs[b]
        warn("name for "..b)
        
        if buff.color then
            self.itemColor = buff.color
        end
        
        if not s then
            s = getValue(data.isCurse and buff.curseName or buff.name)
            if data.isCurse then
                if not buff.curseName then
                    s = string.format("%s-%s", buff.cursePrefix or buff.curse == "user" and "self" or "enemy", s)
                end
            end
            
            name = string.format("%s ~ -of %s", name, s)
        else
            local p = getValue(buff.prefix or buff.name)
            
            if data.isCurse then
                p = string.format("%s-%s", buff.cursePrefix or buff.cursePrefix or buff.curse == "user" and "self" or "enemy", p)
            end
            
            name = string.format("%s - %s", p, name)
        end
        
        warn("name "..name)
    end
    
    name = string.format("%s ~%s", ss, name)
    
    if not self.revealedName then
        self:makeDifferentFromStack()
        self:addLog(string.format("%s is revealed as a %s", self.title, name))
    end
    cwarn("name r","cyan")
    self.name = name
    self.revealedName = true
    
    self.color = self.itemColor
    
    self.room:reloadInventoryUI()
end

function Item:makeDifferentFromStack()
    if not self.spaceData then
        
        local user
        local equipped
        if self.user then
            log("[ITEM] Differentiating from inventory stack")
            user = self.user
            equipped = self.equipped
            self.user:removeItem(self, true)
        end
        
        self.spaceData = lume.copy(self.data)
        
        self.data = self.spaceData
        
        if user then
            user:addItem(self,nil,nil,true,nil,true)
            if equipped then
                user:equipItem(self)
            end
        end
        
        
        
    end
    
end

function Item:enhance(level)
    if not self.spaceData then
        self:makeDifferentFromStack()
    end
    
    self.enchanted = self.enchanted + level
    self.enchants = self.enchanted
    self.grade = self.grade + level
    
    self.name = self:getName()
    
    self:onEnchant(level)
    self:playEnchanted(level)
    
    if self.enchanted < 0 and #self._enchants > 1 then
        self._enchants = {}
        self:addLog(string.format("%s has lost its powers.",self.title))
    end
end

function Item:enchant(buff, level, curse)
    if type(buff) == "number" or not buff then
        return self:enhance((buff or level or 1)*(curse and -1 or 1))
    end
    
    local b = buffs[buff] or buff
    buff = b._id
    assert(buff, string.format("No buff %s, ding dong!", inspect(b,1)))
    
    local e = self._enchants[buff] or {}
    e.level = level or self.grade-- or 1
    e.isCurse = curse
    e.buff = buff
    
    if b.onEnchant then
        b.onEnchant(self, level)
    end
    
    self._enchants[buff] = e
    
    if not self.revealedName and not self.glowing and (math.random()<.3 or self.user and self.user.isPlayer) then
    
        self.name = string.format("&colors.cyan glowing ~%s", self.name)
        self.glowing = true
        self.itemColor = "cyan"
    end
    
    return self:enhance((level or 1)*(curse and -1 or 1)), buff

end

function Item:curse(buff, level)
    local r, curse = self:enchant(buff, math.max(1,math.abs(level)), true)
    self.curseBuff = curse
    self.cursed = true
end

-- todo??? enchant bullets if shot??
function Item:checkEnchants(target, user)
    local user = user or self.user
    
    for buff, data in pairs(self._enchants) do
        local b = buffs[buff] warn(b.activateChance(
            -- choose max between level of buff and item's grade and 1
            math.max(math.abs(data.level),math.max(math.abs(self.grade),1)
        )*(data.isCurse and 1.5 or 1), data.isCurse).." activate?")
        if b.activateChance(
            -- choose max between level of buff and item's grade and 1
            math.max(math.abs(data.level),math.max(math.abs(self.grade),1)
        )*(data.isCurse and 1.5 or 1), data.isCurse) >= math.random(data.isCurse and 55 or 100) then
            local targ
            b.bless = b.bless or b.buff
            if data.isCurse then
                if b.curse == "user" and self.user then
                    targ = user
                elseif b.curse == "item" then
                    targ = self
                else
                    targ = target
                end
            else
                targ = b.bless == "user" and user or b.bless == "item" and self or target
            end
            
            if targ and (targ==target or self.turnActivated~=self.room.currentTurn) then
                local level = data.level + (self.grade<1 and math.abs(self.grade-1) or (self.grade-1))
                targ:applyBuff(buff, b.getTime(data.level), b.getValue(level), level)
                if targ.tile and b.color then
                    self.room:addEnchantLight(targ.tile, 5, b.color)
                end
                self:revealName()
            end
        end
    end
    
    self.turnActivated = self.room.currentTurn
end

function Item:getPenaltyValue()
    return self.grade < 1 and math.abs(self.grade-1) or 1
end

function Item:cook()
    local t = self.tile
    self.room:removeItem(self)
    local s = self.room:spawnItem(self.cookedItem, t)
    s.parentItem = self.data
    s.isCooked = true
    if self.onCooked then
        self.onCooked(s)
    end
end

function Item:rot()
    if self.rrt then return error("already rotted!"..tostring(self.remm)..tostring(self.isCarried)..tostring(self.room.items[self])..tostring(self.rro)..tostring(self.rrro)) end
    assert(not self.isRotten)
    self.rrt = true
    local t = self.tile ccr = 1 if self.isCarried then self.rro=true end
    self.room:destroyItem(self)
    local s = self.room:spawnItem(self.rottedItem, t) ccr=nil
    if self.onRotted then
        self.onRotted(s)
    end
end

function Item:burn()
    local t = self.tile
    self.room:removeItem(self)
    local s = self.room:spawnItem(self.burntItem, t)
    if self.onBurnt then
        self.onBurnt(s)
    end
end

function Item:__step(dt, ...)
    ostep(self, dt, ...)
    
    if self.oPColor then
        self.color = self.oPColor
        self.oPColor = nil
    end
    
    self.color = self.itemColor or self.color
    self.og_offset_y = self.og_offset_y or self.offset_y
    self.offset_y = self.og_offset_y - (self.tile and self.tile.pedestal and (self.tile.pLock and not self.tile.pLock.unlocked and tw/3 or tw/2) or 0)
    
    if self.pLock and not self.pLock.unlocked and self.pLock.taken ~= self then
        self.oPColor = self.color
        self.color = "darkgray"
    end
    
    if self.light and (not self.light.isLight) and self.world:hasItem(self) then
        local room = self.room
        local ttile = self.user and self.user.tile or room:getTileP(self.x, self.y)
        
        if self.oldLightTile ~= ttile then
            self.oldLightTile = ttile
            
            if self.lightData then
                room:removeLight(self.lightData)
                self.lightData = nil
            end
            
            if self.light then
                local l = self.light
                self.lightData = room:addLight(ttile, l[1], l[2])
            end
        end
    elseif self.lightData then
        local room = self.room
        room:removeLight(self.lightData)
        self.lightData = nil
        self.oldLightTile = nil
    end
    
    
    if not self.tile then
        local ttile = room:getTileP(self.x, self.y)
        if not ttile.item and not self.isCarried then
            ttile.item = self
            self.tile = ttile
        end
    end
    
    if self.tile and not self.isCarried and (not self.tile.item or self.tile.item.noPickup) and not self.isDeadDead then
        self.tile.item = self
    end
    
    
    
    
    if self.isFloor and (self.oldTile ~= self.tile or not self.tile.floor) then
    
        if self.oldTile then
            self.oldTile.floor = nil
        end
        
        self.tile.floor = self
        self.oldTile = self.tile
    end
    
    if self.tile and self.tile.item == self and self.isCarried then
        self.tile.item = nil
    end
end

function Item:draw()
    if Entity.draw then Entity.draw(self) end
    self:drawBuffs()
    
    --if selfisFloor then
end

-- add an event (played with "play<Event>" at appropriate time)
function Item:onEvent(eventName, event, ...)
    return self[string.format("on%s",eventName)](self, event, ...)
end

function Item:applyBuff(buff, time, value, enchants)
    buff = data.buffs[buff] or buff
    if buff.name == "burn" and not self.canBurn then
        return
    end
    
    buff.enchants = enchants or buff.enchants or 1
    value = value or 1
    time = time or 1
    
    if buff.once then
        buff.once(buff, self, value, time)
    else
        for x = 1, #self.buffs do
            local b = self.buffs[x]
            if b._id == buff then
                b.times = b.times + (time or 1)
                b.data[2] = b.data[2]+value
                if time < 0 then
                    b.times = time
                end
                
                return
            end
        end
            
    end
    
    if buff then
        local data = lume.copy(buff)
        data.addedBuff = true
        data.id = buff
        self.buffs[#self.buffs+1] = data
        data.times = time
        local event, eventData = self:onEvent(
            buff.eventType or self == self.room.player and "EndTurn" or "StartTurn", 
            buff.action or null,
            value,
            time,
            data
        )
        data.event, data.eventData = event, eventData
    end
    
end

function Item:removeBuff(data, n)
    if #self.buffs == 0 then return end
    
    if type(data) == "string" then
        local r
        for x = 1, #self.buffs do
            local buff = self.buffs[x]
            if buff and (data == true or buff._id == data) then
                r = self:removeBuff(buff, x)
            end
        end
        
        return r
    end
    
    lume.remove(self.events[data.event], data.eventData)
    
    -- n should save time, lume.remove is slower but needed for unknown index
    local f  =(n and table or lume)
    f.remove(self.buffs, n or data)
    
    return data
end
    

function Item:drawBuffs()
    local dd = 6
    local p = 3
    for xx = 1,#self.buffs do
        local buff = self.buffs[xx]
        
        if buff.draw then
            buff:draw(self)
        end
    end
end

function Item:drawBurn(alpha)
    
    local source = "setpieces/flame"
    local maxFrames = 4
    if true then
        maxFrames = 5
        source = "setpieces/fire"
    end
    
    local currentFrame = math.floor(self.currentFrame%maxFrames)
    local cr = currentFrame
    
    while cr > (maxFrames-1) do
        cr = 0+(cr-(maxFrames))
    end
    while cr < 0 do
        cr = 0
    end


    local source = string.format("%s/%s.png",source, cr)
    local img = game:getAsset(source)
    local _w, _h = resizeImage(img, tw, th)
    local ww, hh = img:getDimensions()
    local xx = tw/2--gas.xx*0
    local yy = 5
    local tile = self
    local r,g,b,a = love.graphics.getColor()
    set_color(1,1,1,alpha or 1)
   -- draw_rect("line",tile.x+xx,tile.y+th/2,tw,th)
    lg.draw(img, tile.x+xx, tile.y+math.random(-yy,yy)+th/2, math.rad(0 or self.angle), _w, _h, ww/2, hh/2)
    
    
    love.graphics.setColor(r,g,b,a)
end

function Item:hasAmmo(user)
    local user = user or self.user
    for i, dat in pairs(self.ammos) do
        local amount, attack = dat
        
        if type(dat) == "table" then
            amount = dat[1]
            attack = dat[2]
        end
        
        if (#(user.allInventoryItemsByType[items[i]] or {})) >= amount then
            return items[i], amount, attack
        end
    end
end

function Item:takeDamage(...)
    return self:getAttacked(...)
end

function Item:getAttacked(attacker, att, noFlash)
    
    
    if type(attacker) == "number" then
        att = att or {fake=true}
        att.attack = attacker
        attacker = att
        
    end
    local val = (type(att) == "number" and att or attacker.attack)/((not attacker.fake and self.defense or 1)+(self.isIce and self.tile and self.tile.water and 2 or 0))
    local enemy = attacker.user or attacker.parent or attacker
    local seen = attacker.isSeen or enemy.isSeen or self.isSeen
    
    
    if self.fadeAway and not self.ddone and (self.life-val) <= 0 then
    
                if self.isIce and enemy then
                    Entity.spawnDebris(self, "cyan")
                end
                
                
                if (self.life - val) <= 0 then
                    local done = function()
                        self:getAttacked(val,nil,true)
                        if not self.dead then
                            self:die()
                        end
                    end
                    self.ddone = true
                    self.room:tween(.7, self, {image_alpha=0}, "out-quad", done)
                end
                return
                
    end
    
    
    self:onDamaged(attacker, val, enemy)
    
    if attacker.isFire then
        self.cookedValue = self.cookedValue - 1
        if self.cookedValue <= 0 and self.cookedItem then
            return self:cook()
        end
        
        self.burntValue = self.burntValue - 1
        if self.burntValue <= 0 and self.burntItem then
            return self:burn()
        end
    end
        
    local burning = attacker.isFire or attacker.isExplosion
    
    if not burning and self.onlyFireDamages then return end
    if not self.life then return end
    
    if true then
        self.life = self.life - val
    end
    
    local shook
    
    if not self.isCarried and (not noFlash) then
        self:shake(30,.3,35)
        self:flash(.15)
        shook = true
    end
    
    if self.life <= 0 and not self.noDie then
        self.reallyDied = true
        if not shook then
            self:die()
        else
            local des = function() self:die() end
            self.room:after(.2,des)
        end
    end
end

function Item:die()
    if true then
        if self.shopkeeper and not self.shopkeeper.died then
            self.shopkeeper:playMerchandiseDestroyed(self)
            self.shopkeeper = nil
        end
    
        if self.user and self.user.wielding.right == self then
            self.user.wielding.right = nil
        end
        
        
        self.getAttacked = null
        self:onDestroyed(attacker, val)
        self:playDeath(attacker, val)
        
        for x = 1,#self.summoned+3 do
            local mon = self.summoned[x]
            if mon and mon.poweredByMagic and not mon.died then
                mon:die()
            end
        end
    
        if self.master then
            lume.remove(self.master.summoned, self)
        end
    
        if self.lightData then
            self.room:removeLight(self.lightData)
        end
        
        if self.onBreak then
            self:onBreak(attacker, val)
        else
            if self.user and self.isCarried then
                self.user:removeItem(self, true)
            else
                if self.tile.item == self then
                    self.tile.item = nil
                end
                
                -- causes problems for items trying to wrap up update
                -- self.tile = nil
                
                self.isDeadDead = true
                self.room:destroy_instance(self)
            end
        end
        
        self.dead = true
        
        if self.quickslot then
            local fs = self.room.freeQuickSlots
            fs[#fs+1] = self.quickslot
            self.quickslot.item = nil
            self.quickslot = nil
        end
    end
end

function Item:update(dt)
    self.currentFrame = self.currentFrame + dt
    if Object.update then Object.update(self, dt) end
    
    if self._update then self:_update(dt) end
end

function Item:addLog(...)
    return self.room:addLog(...)
end

local function alphabeticPngs(a, b)
    return tonumber(a:sub(1,-5)) < tonumber(b:sub(1,-5))
end

Tree = Item:extend("Tree")

function Tree:create(k)
    k.h = k.h or th*2
    k.noPickup = k.noPickup == nil and true or k.noPickup
    k.canBurn  = k.canBurn  == nil and true or k.canBurn
    
    Item.create(self, k)
    
    self.isLiving = true
    
    self.folder = self.source
    
    --self.__updateTurn = self.updateTurn
    --self.updateTurn = nil
    
    self.sources = love.filesystem.getDirectoryItems(string.format(game.sourcie,self.source))
    
    table.sort(
        self.sources,
        alphabeticPngs
    )
    
    self.source = string.format("%s/%s",self.folder,self.sources[1])
    
    local sourceLen = #self.sources
    self.growthTime = k.growthTime or sourceLen
    self.maxLife = k.life or sourceLen
    
    self.life = self.maxLife/sourceLen
    
    self.toGrow = 0
    self.growth = 0
    self.imageFraction = sourceLen/self.growthTime
    
    self.color = k.color or self.color
    self.skinColor = self.color or self.skinColor
    
end

function Tree:grow(num)
    if self.fullyGrown then return end
    
    self.toGrow = lume.min(self.growthTime, self.toGrow + 1)
    
    self.source = string.format("%s/%s",self.folder,
        self.sources[math.floor(self.toGrow*self.imageFraction)]
        or error(self.folder..","..self.toGrow.."-"..self.growthTime)
    )
    
    self.growth = (self.toGrow/self.growthTime)
    self.life = self.maxLife*self.growth
    
    self.fullyGrown = self.life == self.maxLife
end
    
function Tree:updateTurn(...)
    self:grow(1)
    return Item.updateTurn(self, ...)
end

return Item