local Level = toybox.Room("Level")
--16,16,25,25

throwColor = colors.skyblue
drinkColor = colors.lime

Card = Entity:extend("Card")

function Card:getEnemy()
        local enemy-- lume.keys(self.creatures)
        local dis
        local pl = self.room.player
        for x, cr in pairs(self.room.enemies) do
            local dist = lume.distance(cr.x,cr.y,pl.x,pl.y)
            if not enemy or dist<dis then
                enemy = cr
                dis = dist
            end
        end
        
        if not enemy then
            return
        end
        
        return enemy
    
end

function Card:effect()
    if true then
        local enemy-- lume.keys(self.creatures)
        local dis
        local pl = self.room.player
        for x, cr in pairs(self.room.enemies) do
            local dist = lume.distance(cr.x,cr.y,pl.x,pl.y)
            if not enemy or dist<dis then
                enemy = cr
                dis = dist
            end
        end
        
        if not enemy then
            return
        end
        
        local ww = 40
        local obj = Entity:new({x=self.x,y=self.y,w=ww,h=ww,source=self.attackSource})
        obj.depth = DEPTHS.EFFECTS
        obj.solid = false
        obj.alwaysTrail = 1
        obj.trailfW = 5
        Trail(obj)
        --pl.name_tag = (lume.distance(obj.x,obj.y,enemy.x,enemy.y))
        local function shake()
            enemy:shake(50,.5,35)
            self.room.camera:shake(25,.3,30)
        end
        self.room:tween(.5,obj,{y=obj.y-300,x=obj.x+40},"in-quad",function()
            self.room:tween(.5,obj,{x=enemy.x,y=enemy.y+enemy.h},"in-quad",function()
                --self.room:after(.3,shake)
                
                ----
                enemy:getAttacked(self)
                self:checkBuffs(self.user, enemy)
                ----
                
                obj.depth = enemy.depth+.05
                self.room:tween(.5,obj,{x=enemy.x+enemy.w,y=enemy.y+enemy.h*0},"in-quad",function()
                    obj:destroy()
                end)
            end)
        end)
    --elseif self.effect
    end
end

function Card:create(k)
    k = k or {}
    self.effectTime = .1
    Entity.create(self, k)
    self:set_box(self.x, self.y,self.w,self.h)
    self.parent = k.parent
    
    self:center()
    
    self.o = self:add_image("overlay.png")
    self.energyImage = self:add_image("energy.png")
    
    ---self.o.offset_y = self.offset_y
    --self.o.offset_x = self.offset_x
    
    --self.debug=1
    self.depth = DEPTHS.EFFECTS
    self.solid = false
    self.scale = 1
    
    local data = k.data or k
    
    self.data = data
    self.angle = 15
    self.energy = data.energy or 1
    
    self.attack = getValue(data.attack)
    
    self.info = string.format(data.info or "yo I do a bunch of stuff man",self.attack)
    self.baseInfo = self.info
    
    self.type = data.type
    
    if self.type == "potion" then
        self.info = string.format("%s &%s %s", self.info, inspect(throwColor, {spaces=false}), "(throw)")
    end
    
    
    self.icon = self:add_image(self.type=="potion" and "cards/potion_in.png" or
        data.source and 
        (string.format("cards/%s",getValue(data.source)))
        or "heart.png"
    )
    self.icon.color = data.color or self.icon.color
    
    if self.type == "potion" then
        self.cover = self:add_image("cards/potion.png")
    end
    
    self.text = Sentence:new({x=self.x+14,y=self.y-191,w=168})
    self.text:newText(self.info)
    
    self.title = Sentence:new({x=self.x,y=self.y,w=180})
    self.title:newText(self:getName())
    self.title.centered = K22
    
    self.parentCard = self.isClone
    self.children = {}
    
    self.user = self.room.player
    
    self:fixEnchants(data.enchants or 1)
    self.effect = data.effect
    if data.postMake then
        data.postMake(self, self.room, self.user)
    end
    
end

function Card:fixEnchants(n)
    self.enchants = n
end

function Card:getName()
    local n = self.data.name or "???"
    self.name = n
    
    self.oldName = n
    
    return n
end

-- if type is weapon
function Card:applyBuff(name)
    if not self.suffix then
        self.suffix = data.buffs.weapons[name]
        self.name = string.format("%s of %s", self.name, self.suffix.name)
        self.icon.color = self.suffix.color
    
    
    elseif not self.prefix then
        self.suffix = data.buffs.weapons[name]
        self.name = string.format("%s %s", self.prefix.prefix or self.prefix.name, self.name)
        self.icon.color = self.prefix.color
    
    else
        warn("RUNES FULL")
    end
end

function Card:checkBuffs(user, target)
    if self.prefix then
        local apply = 100--(self.accuracy
        if math.random(100) <= apply then
            self.prefix.apply(self, user, target)
        end
    end
    
    if self.suffix then
        local apply = 100--(self.accuracy
        if math.random(100) <= apply then
            self.suffix.apply(self, user, target)
        end
    end
end

function Card:addClone()
    local clone = Card:new({
        isClone = self,
        data = self.data,
    })
    
    self.children[#self.children+1] = clone
    
    return clone
end

function Card:fixTexts(dt)
    if true then
        
        self.text.w = 168*self.scale*1
        self.text.x = self.x+self.scale*14--+self.text.w/4
        self.text.y = self.y+self.scaleY*191
        self.text.scale = self.scale
        self.text.font = font8
        self.text:update(dt)
        
        self.title.w = 140*self.scale
        self.title.x = self.x+self.scale*50
        self.title.y = self.y+10*self.scaleY
        self.title.scale = self.scale
        self.title.font = font8
        self.title:update(dt)
    end
end

function Card:step(dt)
    local p = self.parent
    if self.active then
        self.depth = DEPTHS.EFFECTS
    end
    
    
    if not self.active or not self.world:hasItem(self) then
        self:fixTexts(dt)
        return
    end
    
    if self.active then
        self.depth = DEPTHS.EFFECTS
    end
    
    if p and p.card == self then
        self.selected = false
        self.depth = p.depth or self.depth
        
        if self.room.oldB == p then
            self.depth = self.depth+.1
            self.selected = true
        end
        local px, py = p.o and p.o.x or p.x, p.o and p.o.y or p.y
        if self.hasOb or self.room.oldB == p then
            self.hasOb = true
            px, py = p.x, p.y
        end
        self.x, self.y = px, py
        self.w, self.h = p.w, p.h
        self.bw = p.bw
        self.bh = p.bh
        self.angle = p.angle
    end
    
    if true then--self.w, self.h = self.bw or 100, self.bh or 400
        self.bw = self.bw or 1
        self.bh = self.bh or 1
        self.scale = self.w/self.bw
        self.scaleY = self.h/self.bh
        self:set_box(self.x,self.y,lume.max(1,self.w),lume.max(1,self.h))
        self:center()
    end
    
    self:fixTexts(dt)
end

function Card:draw_before()
    local p = self.parent or self.room.cardPoses[1]

    if p then
        local rx, ry = self.x+self.w/2, self.y+self.h/2
        lg.push()
        local ang = math.rad(self.angle)
        lg.translate(rx,ry)
        lg.rotate(ang)
        lg.translate(-rx, -ry)
        p.visible = true
        local ox, oy = p.x, p.y
        local ow, oh = p.w, p.h
        p.w, p.h = self.w, self.h
        p.x, p.y = self.x+self.shake_x, self.y+self.shake_y
        gooi.drawComponent(p)
        p.x, p.y = ox, oy
        p.w, p.h = ow, oh
        
        p.visible = false
        lg.pop()
    end
    --self.name_tag = self.parent and self.angle.."," or "??????"
end

local grey = {0,0,0,.5}
function Card:draw()
    self.scale = self.w/self.room.cardPoses[1].bw
    self.scaleY = self.h/self.room.cardPoses[1].bh
    
    local of = love.graphics.getFont()
    local nf = self.font or font13
    love.graphics.setFont(nf)
    local w,h = resizeImage("overlay.png",self.w, self.h)
    local o = game:getAsset("overlay.png")
    local ww,hh = o:getDimensions()
    
   -- draw_image("overlay.png",self.x,self.y,0,w,h)--, ww/2, hh/2)
   
    local rx, ry = self.x+self.w/2, self.y+self.h/2
    lg.push()
    local ang = math.rad(self.angle)
    lg.translate(rx,ry)
    lg.rotate(ang)
    lg.translate(-rx, -ry)
    
    local r,g,b,a = set_color(grey)
    local x = self.x+9*self.scale
    local y = self.y+185*self.scaleY
    draw_rect("fill",x,y,176*self.scale,109*self.scaleY)
    
    --set_color(0,0,0)
    if self.selected then
        draw_rect("fill",x,self.y,176*self.scale,(self.title.h or 63)*1)
    end
    
    set_color(r,g,b,a)
    local xx = self.title.x-self.x-10
    --draw_rect("line",self.title.x-xx,self.title.y+(0),self.title.w+xx,self.title.h)
    
    
    love.graphics.print(tostring(self.energy),self.x+20*self.scale,self.y+16*self.scaleY,0,self.scale,self.scaleY)
    
    self.text:draw()
    if self.selected then
        self.title:draw()
    end
    
    
    
    --draw_rect("line",self.title.x,self.title.y,self.title.w,self.title.w)
    
    
    lg.pop()
    love.graphics.setFont(nf)
    
    set_color(getColor"lightgreen")
    self:draw_image_data(self.energyImage)
    set_color(r,g,b,a)
    
    
end

function getBasicGun()
    local basicGun = {parts = {}}
    for x, i in pairs(gunparts) do
        basicGun.parts[x] = i.basic
    end
    basicGun.parts.trigger = triggers.hammer
    return basicGun
end

function Level:setup(k)
    self:activate_gooi()
    love.graphics.setDefaultFilter("nearest","nearest")
    gooi.defaultText = "..."
    game.squash = false
    local g = "level"
    
    self.field = gooi.newPanel({
        group = g,
        x = 0,
        y = 0,
        w = W(),
        h = H(),
        padding = 20,
        layout = "grid 1x6",
    })
        :setColspan(1,1,1)
        :setColspan(1,2,3)
        :setColspan(1,5,2)
        
    
    local pw = W()*.9
    self.players = gooi.newPanel({group=g,layout="grid 1x10",h=144,w=pw,
    x=W()/2-pw/2,y = H()/2-144/2,
    paddioing=0})
    self.players.opaque = truex
    self.players.bgColor = {0,0,0,0}
    
    self.opponent = gooi.newPanel({})
    self.field:add(self.opponent)
    self.opponent.opaque = true
    
    self.info = gooi.newPanel({layout="grid 10x1"})
    self.field:add(self.info)
    self.info.opaque = true
    self.info:setRowspan(1,1,7)
    self.info:setRowspan(8,1,3)
    
    self.log = gooi.newLabel({text=">Hmmmm"}):left()
    self.info:add(self.log)
    self.log.yOffset = 1
    
    self.options = gooi.newPanel({layout="grid 3x1"})
    self.info:add(self.options)
    self.optionButtons = {}
    for x = 1,3 do
        local b = gooi.newButton({text="---"})
        self.options:add(b)
        self.optionButtons[x] = b
    end
        gooi.removeComponent(self.field)

    
    self.positions = {}
    for x = 1,10 do
        local b = gooi.newButton({text=""})
        b.opaque = truem
        b.borderColor = {1,1,1,.3}
        b.n = x
        self.players:add(b)
        if x > 0 then
            self.positions[x] = b
        end
    end
    table.remove(self.positions,1)
    table.remove(self.positions,#self.positions)
    
    gooi.currentGroup = g
    
    self.enemies = {}
    self.friends = {}
    self.creatures = {}
    
    self.logs = {}
    
  --[[  self:addLog("@hmmm ~ yoo")
    self:addLog("okay")
    self:addLog("#Hit on by a donkey ~....")
    self:addLog("@Yes...")
    self:addLog("@A donkey   ~ ...")
    self:addLog("no way",true)
    ]]--
    
    local cardScale = .8
    
    local hh = H()*.4
    local ww = W()*.8
    local hh2 = 100
    local ww2 = hh2*5--ww*.6
    
    self.cardsUI = gooi.newPanel({
        layout = "grid 1x5",
        group = g,
        w = ww*cardScale,
        h = hh*cardScale,
        x = W()/2-ww*cardScale/2,
        y = H()-hh*cardScale-10
    })
    
    self.relicsUI = gooi.newPanel({
        layout = "grid 1x5",
        group = g,
        w = ww2, h = hh2,
        x = W()/2-ww2/2,
        y = 5--H()-hh2-5,
    })
    
    local cr = 2.5
    local cw, ch = (ww/5)/cr, hh/cr
    self.drawPile = gooi.newButton({
        group = g,
        w = cw, h = ch,
        x = 10, y = H()-hh-10+(hh/2-ch/2)
    })
    
    self.discardPile = gooi.newButton({
        group = g,
        w= cw, h = ch,
        x = W()-self.drawPile.x-cw,--self.cardsUI.x+ww+(self.cardsUI.x-(cw+10)),
        y = self.drawPile.y
    })
    
    local function endTurn()
        if self.notPlayer then  if self.endingTurn then warn() end return end
        if self.drawingCards == 0 and self.discardingCards == 0 then
            self.notPlayer = true
            if not self.player:playEndTurn() then
                self:nextTurn()-- name_tag
            end
        else
            self.endingTurn = true
            self.notPlayer = true
        end
    end
    
    
    self.endTurn = gooi.newButton({
        group = g,
        w= cw, h = ch-50,
        x = self.discardPile.x,--self.cardsUI.x+ww+(self.cardsUI.x-(cw+10)),
        y = self.drawPile.y+self.discardPile.h+5,
        text = "end"
    }):onRelease(endTurn)
    
    local pad = 5
    
    self.scrollPile = gooi.newButton({
        group = g,
        w = cw, h = ch,
        x = 10, y = H()-hh-10+(hh/2-ch/2-ch/2-pad/2)
    })
    
    self.drawPile.y = self.drawPile.y+ch/2+pad/2
    self.drawPile.x = self.drawPile.x - 50
    
    self.drawPile.visible = false
    self.discardPile.visible = false
    
    self.energy = gooi.newButton({
        group = g,
        text = " 5/5",
        w = 150,
        h = 50,
        x = 10, y = 10,
    }):setIcon("heart.png")
    local eb = self.energy
    local eh = 25
    eb.showBorder = false
    eb.font = font18
    eb.iw = resizeImage("heart.png",eh,eh)
    eb.ih = resizeImage("heart.png",eh,eh)
    
    self.cardPoses = {}
    self.cards = {}
    self.relics = {}
    
    local function selectCard(b, noUse) --er()
        if self.drawingCards >= 1 then return end
        if self.discardingCards >=1 then return end
        
        if self.oldB then
            local ob = self.oldB
            self.timer:cancel(self.bt)
            self.bt = self:tween(.1,ob,ob.o,"out-quad")
        end
        if b.card then
            b.card:shake(30,.2,30)
        end
        if self.oldB == b then
            self.oldB = nil
            self.lastB = b
            
            if b.card and b.card.parent == b and self.player.energy >= b.card.energy then
                
                self:playCard(b.card)
            elseif b.card and b.card.parent == b then
                self.player:cry("Not enough energy")
            end
            
            return
        end
        
        --self.timer:cancel(self.bt)
        self.bt = self:tween(.2,b,b.b,"in-bounce")
        self.oldB = b
    end
    
    for x = 1, 5 do
        local b = gooi.newButton({
            group = g,
            text = ""
        })
        self.cardsUI:add(b)
        self.cardPoses[x] = b
        b.n = x
        if x then
            b.bw = b.w/cardScale
            b.bh = b.h/cardScale
            b.by = b.y-(b.h-b.h*(cardScale))
            b.bx = b.x-(b.w-b.w*cardScale)
            b.angle = math.random(-15,30)
            
            b.b = {x=b.bx, y=b.by, w=b.bw, h=b.bh, angle=0}
            
            b.ox, b.oy, b.ow, b.oh = b.x, b.y, b.w, b.h
            b.o = {x=b.ox, y=b.oy, w=b.ow, h=b.oh, angle = b.angle}
            b.cardScale = cardScale
            b.visible = false
            b.alwaysPress = true
            b.depth = DEPTHS.CARDS+x/1000
        end
        
        b:onRelease(selectCard)
    end

    self.tmpCardPoses = lume.copy(self.cardPoses)
    
    for x = 1,-5 do 
        self.timer:after((x-1)*.1 or (x*math.random(10,50)/100),function()
        local c = Card:new({data=cards.weapons.sword})
        self:drawCard(c)
        end)
    end
    local ws = 11
    local sc = 1
    local lw, lh = W()*.7+(116.57+2.5)*3-30, H()*.9
    local lw, lh = lw*sc, lh*sc
    self.cardDisplay = gooi.newPanel({
        layout = "grid 11x11",
        group = g,
        w = lw, h = lh,
        x = W()/2-lw/2+30, y = H()/2-lh/2,
        padding = 5
    })
    
    local c = self.cardDisplay
    local s =""
    c:setColspan(1,1,11)
    self.displayLabel = gooi.newLabel({
        text = "Your items",
    })
    self.displayLabel.opaque = true
    c:add(self.displayLabel)
    
    for y = 2, 10, 2 do
        --s = s..","..y
        for x = 1, ws do
            c:setRowspan(y,x,2)
        end
    end
    
    self.displays = {}
    
    for xx = 1, 5*ws do
        local b = gooi.newButton({text="",group=g})
        c:add(b)
        self.displays[xx] = b
    end
    
    
    local cw = 60
    self.cancelDisplay = gooi.newButton({
        text = "x",
        x = 3, y = self.displayLabel.y,
        group = g,
        w = cw, h = cw
    }):onRelease(function()
        self:removeCards()
    end)
    
    for x = 1,5 do
        local b = gooi.newButton({
            group = g
        })
        self.relicsUI:add(b)
        self.relics[x] = b
    end
    
    self.deck = {}
    self.drawPileData = {}
    self.discardPileData = {}
    
    self.drawingCards = 0
    self.discardingCards = 0

    for x = 1,2-1 do
    self.timer:after((x-1)/5,function() self:spawnEnemyCreature(x==1 and creatures.cursed_potate or creatures.unicorn,10-(1-x)) end)
    end
    
    self:removeCards()
    
    self:startMatch()
    
    self:showCards()
end

function Level:showCards()
    if self.notPlayer then return end
    self.notPlayer = true
    
    self.showingCards = true
    local count = 1
    local maxW = 0
    for x = 1, #self.displays do
        local i = self.deck[x] --, i in pairs(self.deck) do
        local button = self.displays[x]
        if i then
            count = count + 1
            button.card = i
            button.visible = true
            count = x
            maxW = lume.max(maxW, button.x+button.w)
        else
            button.visible = false
        end
    end
    self.maxCardsDisplayed = count
    self.displayLabel.w = maxW-self.displayLabel.x
    gooi.addComponent(self.cardDisplay)
    gooi.addComponent(self.cancelDisplay)
end

function Level:removeCards()
    self.showingCards = false
    gooi.removeComponent(self.cardDisplay)
    gooi.removeComponent(self.cancelDisplay)
    self.notPlayer = false
end

function Level:drawCardDisplay()
    local r,g,b,a = set_color(0,0,0,.7)
    draw_rect("fill",-10,-10,W()+20,H()+20)
    set_color(r,g,b,a)
    
    gooi.drawComponent(self.cardDisplay,true)
    gooi.drawComponent(self.cancelDisplay)
    
    for x = 1, self.maxCardsDisplayed do
        local b = self.displays[x]
        local p = self.cardPoses[1]
        b.bx, b.by = p.bx, p.by
        b.bw, b.bh = p.bw, p.bh
        
        
        local c = b.card
        local op = b.card.parent
        local ox, oy, ow, oh, oa, os, oac = c.x, c.y, c.w, c.h, c.angle, c.scale, c.active
        c.active = true
        c.w,c.h = b.w, b.h
        c.x, c.y = b.x, b.y
        c.bw,c.bh = p.bw, p.bh
        c.angle = 0
        b.card.parent = b
        b.angle = 0
        c:set_box()
        c:__step(0)
        c:__draw(b.x,b.y)
        c.parent = op
        c.angle = oa or error()
        c.x, c.y, c.w, c.h = ox, oy, ow, oh
        c:set_box()
        c:__step(0)
        c.active = oac
    end
end

function Level:getPlayer()
    self:spawnPlayerCreature(Creature:new({
        source="boi.png",
        name="pony",
        stats = {
            health = 30
        
        }}),1)
    local d = {"fearpotion","sword","enchanting",{"sword",4},"sword","greatsword","axe","sword","sword","dagger"}
    
    self:loadDeck(d)

end

function Level:loadDeck(data)
    for _, card in ipairs(data) do
        local dat = card
        local enchants = 1
        if type(card) == "table" then
            dat = card[1]
            enchants = card[2]
        end
        
        local added = false
        
        for type, list in pairs(cards) do
            if list[dat] then
                list[dat].type = string.sub(type, 1,-2)
                self:addCardToDeck(list[dat])
                added = true
                break
            end
        end
        
        if not added then
            error(string.format("No card %s",dat))
        end
        
        if enchants ~= 1 then
        end
    end
end

function Level:sortDrawPile()
    for x, i in ipairs(self.drawPileData) do
        i.depth = #self.drawPileData and x
    end
end

function Level:startMatch()
    self:getPlayer()
    
    self.drawPileData = lume.shuffle(self.deck)
    self.discardPileData = {}

    self:startNewTurn()
    self:sortDrawPile()
    
    self.drawPileCopy = table.sort(lume.copy(self.drawPileData),depth)
end

function Level:startNewTurn()
    if self.oldB then
        local ob = self.oldB
        self.timer:cancel(self.bt)
        self:tween(.01,ob,ob.o,"out-quad")
    end
    local t
    for x = 1, 5 do
        local func = function()
            if t then
                self:after(t,function() self:drawCard() end)
            else
                t = self:drawCard()
            end
            
        end
        self:after(x*.1,func)
    end
end

function Level:reshuffle()
    self.drawPileData = lume.shuffle(self.discardPileData)
    local totalTime = 0
    for x, i in ipairs(self.drawPileData) do
        local time = math.random(6,8)/5
        totalTime = lume.max(totalTime,time)
        
        self:tween(time, i, {x=self.drawPile.x-50,yh=self.drawPile.y,angle=i.angle+180*2})
        self:tween(time, i, {y=self.drawPile.y-550},"in-parabola")
    end
    self.drawPileCopy = table.sort(lume.copy(self.drawPileData),depth)
    
    self.discardPileData = {}
    
    return totalTime
end

function Level:drawCard(card)
    if not card then
        if #self.drawPileData == 0 then
            local t = self:reshuffle()
            self.notPlayer = true
            self:after(t,function()
                self.notPlayer = false
                self:drawCard()
            end)
            return t
        end 
        card = table.remove(self.drawPileData, 1)
        self:sortDrawPile()
        self.drawPileCopy = table.sort(lume.copy(self.drawPileData),depth)
        
        --4 cards or less
        if not card then
            return --error(inspect(self.drawPileData,1))
        end
    end
    --self:must_draw(card)
    self.drawing = card
    local button = table.remove(self.tmpCardPoses, #self.tmpCardPoses)
    
    local d = self.drawPile
    
    --card:move_to(d.x, d.y)
    card.scale = 1/10
    card.bw = button.bw
    card.bh = button.bh
    card.hl = card.bh/10
    card.lw = card.bw/10
    card.active = true
    --self.world:add(card,card.x,card.y,card.w,card.h)
    card:set_box()
    self.drawingCards = self.drawingCards + 1
    if not card.angleSetOnce then
        card.angle = math.random(-1000,1000)
        card.angleSetOnce = true
    end
    if self.lastB == button or self.oldB == button then
        self.timer:cancel(self.bt)
        local o = button.o
        button.x, button.y, button.w, button.h, button.angle = o.x, o.y, o.w, o.h, o.angle
    end
    
    local xxy,yyx = button.x, button.y
    card.after_draw = button.n==85 and function(self) draw_rect("fill",self.x,self.y,10,10) end
    or null
    local time = lume.max(2/5,button.n/5)
    --card.parent = button
    self:tween(time, card, {
        x = button.x,
        y = button.y,
        angle = button.angle,
        scale = button.cardScale,
        w = button.w,
        h = button.h
    },"in-quad",function()
        card:move_to(button.x, button.y)
        card.scale = button.cardScale
        --self.timer:after(1,function()
            card.parent = button
        --end)
        card.hasOb = false
        if self.drawing == card then self.drawing = nil end
        --self.timer:after(.5,function()
            self.drawingCards = self.drawingCards - 1
        --end)
    end)
        
    button.card = card
    self.cards[#self.cards+1] = card
    card.n = #self.cards
end

function Level:addCardToDeck(card)
     if not card.isCard then
        card = Card:new({data=card})
    elseif card.in_deck then
        card = card:addClone()
    end
    
    
    -- the deck keeps growing larger and larger
    self.deck[#self.deck+1] = card
    --until you have a big deck
    
    assert(not card.in_deck)
    
    card.in_deck = (card.in_deck or 0)+1
    card.active = false
    card:move_to(self.drawPile.x, self.drawPile.y)
    card.angle = 
    math.random(50,150)*.01*360*math.random(1,4)+math.random(-15,15)
    card.w, card.h = self.cardPoses[1].w, self.cardPoses[1].h
    card.scale = self.cardPoses[1].cardScale
    card.bw = self.cardPoses[1].bw
    
    card:set_box()
    
    local op = card.parent
    card.active = true
    card.parent = self.cardPoses[1]
    card:__step(0)
    card.active = false
    card.parent = op
    
    if self.world:hasItem(card) then
        self.world:remove(card)
    end
    
    
    if card.type == "potion" and not card.isDrink then
        local ch = card:addClone()
        ch.info = string.format("%s &%s %s", ch.baseInfo, inspect(drinkColor, {spaces=false}), "(drink)")
        ch.text:newText(ch.info)
        ch.isDrink = true
        self:addCardToDeck(ch)
    end
    
end

function Level:shiftCards(n, noDraw, ending)
    self.tmpCardPoses = lume.copy(self.cardPoses)
    local cards = self.cards
    self.cards = {}
    noDraw=1
    for x = 1,n do
        --self.cardPoses[x].angle = math.random(-15,30)
    end
    
    self.drawingCards = self.drawingCards + (noDraw and 0 or 1)
    
    if self.oldB then
        local ob = self.oldB
        self.timer:cancel(self.bt)
        self:tween(.01,ob,ob.o,"out-quad")
    end
    local oldB = self.oldB
    local lastC
    
    for x = 1, #cards do
      local card = not ending and cards[x]
      
      if card then
        self.cards[#self.cards+1] = card
        lastC = card
        card.n = #self.cards
        local selected
        local b = self.cardPoses[#self.cardPoses-(card.n-1)]
        local function fix()
            card.parent = b
            b.card = card
            if selected and b~=self.oldB then
                b.events.r(b, true)
            end
            if card == cards[#cards] then
                self.drawingCards = self.drawingCards - (noDraw and 0 or 1)
                lastC = nil
            end
            b.bgColor =colors.black
        end
        --b.bgColor = colors.red
        lume.remove(self.tmpCardPoses, b)
        card.static = true
        
        --card:move_to(b.x,b.y)
        
        if card.parent and card.parent.card == card then
            if card.parent == oldB then
                selected = true
            end
            card.parent.card = nil
        end
        
        if card.shiftt then
            self.timer:cancel(card.shiftt)
        end
        card.shiftt = self:tween(.7,card,{x=b.x, y=b.y, angle=b.angle}, "in-quad", fix)
      end
    end
    
    return .7
end

local function endFunc(c)
    error(inspect(c,2))
end

function Level:_discardCard(card, noDraw, ending2)
    --card:move_to(self.drawPile.x, self.drawPile.y)
    card.depth = -100+#self.discardPileData
    if self.world:hasItem(card) then
        self.world:remove(card)--:destroy()
    else
        warn()
    end
    lume.remove(self.cards, card)
    local tt = ending2 and .1 or self:shiftCards(card.n,noDraw)
    self.notPlayer = true
    --table.insert(self.tmpCardPoses,card.parent)
    self.discardPileData[#self.discardPileData+1] = card
    self.discardPileCopy = lume.copy(self.discardPileData)
    table.sort(self.discardPileCopy, depth)
    card.active = false
    card.x = card.x-card.w*0
    local b = self.cardPoses[1]
    
    
            card.active = true
            card.parent = b
            card:__step(0)
            card:set_box()
            --card:center()
            card.parent = nil
            card.active = false
            
        self:after(tt-.1, function()
            self.notPlayer = false
        end)
        
    if not noDraw then
        self:after(tt, function()
            --self.notPlayer = false
            self:drawCard()
            self.discardingCards = self.discardingCards - 1
        end)
    end
end

function Level:discardCard(card, noDraw, ending2)
    local d = self.discardPile
    local dx = d.x
    local dy = card.y+15 or d.y
    
    local b = card.parent
    if b then
        b.card = nil
    end
    --card.parent = nil
    self.discarding = card
    local function endFunc()
        --if b then b.bgColor=getValue(color) end
        self:_discardCard(card, noDraw, ending2)
        if noDraw then
            self.discardingCards = self.discardingCards - 1
        end
        card.parent = nil
        if self.discarding == card then
            self.discarding = nil
        end
    end
    
    local b = b or self.cardPoses[1]
    local time = .8
    self.discardingCards = self.discardingCards + 1
    --card.after_draw = function(self) draw_rect("line",self.x,self.y,10,10) end
    --700
    self:tween(time,card,{x=dx,w=b.ow,h=b.oh,schale=b.cardScale,angle=card.angle+
    math.random(50,150)*.01*360*math.random(1,4)},
    "in-quad",endFunc)
    self:tween(time-.2,card,{y=card.y-200},"in-parabola",function()
        self:tween(.2,card,{y=dy},"in-quad")
    end)
end

function Level:playCard(card, pl)

    local pl = pl or self.player
    --card.va = 350
    self:discardCard(card)
    
    local function effect()
        card:effect()
    end
    
    pl.energy = pl.energy-card.energy
    
    self:after(card.effectTime or .2, effect)
end

function Level:draw()
    self.energy:setText(string.format("%s/%s",math.floor(self.player.energy),self.player.totalEnergy))
    local v = self.viewing
    if v then
        local w, h = 50, 50
        local x, y = v.x+(v.w/2-w/2), v.y-h-10
        draw_rect("fill",x,y,w,h)
    end
    
    self.discardPile:setText(tostring(#self.discardPileData))
    self.drawPile:setText(tostring(#self.drawPileData))
    
    --self.endTurn:setText((self.discardingCards+self.drawingCards)~=0 and "..." or "end")
    
    local v = (self.discardingCards+self.drawingCards)==0
    --self.endTurn.enabled = v
    
    if self.endingTurn and v then
        self.notPlayer = false
        self:nextTurn()--self.endTurn.events.r(self.endTurn)
        self.endingTurn = false
    end
    
    for x, i in ipairs(self.discardPileCopy or {}) do
        i:__draw()
    end
    
    for x, i in ipairs(self.drawPileCopy or {}) do
        i:__draw()
    end
    
    if self.drawing then
        self.drawing:__draw()
    end
    
    if self.discarding then
        self.discarding:__draw()
    end
    
    self.discardPile.visible = false
    gooi.drawComponent(self.endTurn)
    
    --self.player.name_tag = self.notPlayer and "No" or "Yes"--..","..self.cardPoses[5].x
    
    if self.showingCards then
        self:drawCardDisplay()
    end
end

function Level:nextTurn(noBuff) --cwarn(7,"blue")
    self.currentTurn = (self.currentTurn or 1)
    local crs = table.sort(lume.keys(self.creatures), order)
    self.currentTurn = self.currentTurn + 1
    if self.currentTurn > #crs then
        self.currentTurn = 1
    end
    
    local cr = crs[self.currentTurn]
    if cr ~= self.player then
        local time = .2
        if not self.notPlayer or #self.cards>0 then
            for x = 1, #self.cards do
                self.timer:after(x*.1,function()
                    if self.cards[x] then
                        self:discardCard(self.cards[x], true, true or x~=1)--#self.cards)
                    end
                end)
            end
            
            self.tmpCardPoses = lume.copy(self.cardPoses)
            local cards = self.cards
            self.cardis = {}
            time = .8
        end
        self:after(time, function()
            self.notPlayer = true
            if not self.noBuff and cr:playStartTurn() then
                self.currentTurn = self.currentTurn-1
                self.noBuff = true
            else
                self:doTurn(cr)
                self.noBuff = false
            end
        end)
    else
        self.notPlayer = false
        self:tween(.4,self.player, {energy=self.player.totalEnergy},"in-quad")
        self:startNewTurn()
    end
end

function Level:doTurn(cr)
    cr.energy = cr.totalEnergy
    if cr.getAttack then
        cr:getAttack()
    else
        self.timer:after(.5, function()
            self:nextTurn()
        end)
    end
end

function Level:viewCreature(cr)
    self:addLog(cr.info or "???",true)
    self.viewing = cr
end

function depth(a,b)
    return a.depth>b.depth
end

function order(a,b)
    return (a.position and b.position and a.position.n<b.position.n) or depth(a,b)
end

function Level:mousepressed(x,y)
    self.viewing = nil
    local crs = lume.keys(self.creatures)
    table.sort(crs,depth)
    for _, cr in ipairs(crs) do
        if cr:box_collides(x,y,10,10) then
            self:viewCreature(cr)
            break
        end
    end
end

function Level:addLog(text,r)
    self.logs[#self.logs+1] = text
    if r then
        self:reloadLogs()
    end
end

function Level:reloadLogs()
    local texts = ""
    for x = 1, #self.logs do
        local text = self.logs[x]
        texts = string.format("%s~> %s\n",texts,text)
    end
    
    self.log:setText(texts)
end

function Level:spawnPlayerCreature(creature, tile)

    local tn = type(tile)=="number" and tile
    if type(tile) == "number" then
        tile = self.positions[tile]
    elseif not tile then
        tile = getValue(self.positions)
    end
    
    creature:move_to(tile.x, tile.y-(creature.h-tile.h))
    creature.deibug = 1
    creature:speak("&{1,.5,1} Hey n @all how ~you been diing fr the past qy days\nfine")
    
    self.creatures[creature] = creature
    self.friends[creature] = creature
    creature.team = "player"
    
    creature.position = tile
    lume.remove(self.positions, creature.position)
    creature.depth = tn or 1
    
    self.player = self.player or creature
    self.draw_before = function()
        --set_color(1,1,1)
        draw_image("back.png",W()/2-512,-10)--rect("line",W()/2-512,0,1024,H()*.7)
        gooi.draw(gooi.currentGroup)
    end
    
    return creature
end

function Level:spawnEnemyCreature(data, team, tile)

    local w = data.w or 154/scale
    local h = data.h or 205/scale
    
    local width = data.w/(154/scale)
    
    local tn = tile or #self.positions
    if type(tile) == "number" or not tile then
        tile = self.positions[(tile or #self.positions)-lume.max(math.floor(width-1),0)]
    elseif not tile then
        tile = getValue(self.positions)
    end
    
    local field = tile
    
    local x = field.x--+field.w/2-w/2
    local y = field.y - (h-field.h)--+field.h/2-h/2
    
    data.w, data.h, data.x, data.y = w,h,x,y
    
    local cr = Creature:new(data)
    cr.flipX = -1*(data.flipX or 1)
    cr.txFlipX = -1
    cr.dejbug=1
    
    cr.position = self.positions[tn or #self.positions]
    cr.posNum = tn or #self.positions
    lume.remove(self.positions, cr.position)
    cr.depth = -tn--lume.max(math.floor(width-1),0)
    
    self.creatures[cr] = cr
    self.enemies[cr] = cr
    cr.team = team or data.team or "cp"
    self.timer:after({2,3,1,1.5},function() cr:speak(getValue({"get a load of this guy.","What a weeb","moo"})) end)
    
    return cr
end

function Level:removeCreature(cr)
    cr:destroy()
    self.positions[cr.position.n] = cr.position
    
    self.creatures[cr] = nil
    self.enemies[cr] = nil
end

return Level