-- this file just adds a bunch of functions related to populating the dungeon to the "Map" class
-- so map.lua won't be too bloated

ORC_PORTION = 1
GOBLIN_PORTION = 1

-- [orc:goblin] = weight
local ratioWeight = {
    [{1,3}] = 2,
    [{3,1}] = 2,
    [{2,2}] = 1,
    [{2,1}] = 1,
    [{1,2}] = 1,
    [{0,1}] = 10,
    [{1,0}] = 3
}

function randomizeOrcGoblinRatio()
    local ratio = lume.weightedchoice(ratioWeight)
    ORC_PORTION = ratio[1]
    GOBLIN_PORTION = ratio[2]
end

local prices = {
    potion = 50,
    armour = 0,
    weapon = 0,
    seed = 30,
    scroll = 50,
    charm = 100,
    wand = 100,
    food = 10
}

local priceIncrements = {
    potion = 15,
    armour = 0,
    weapon = 0,
    seed = 0,
    scroll = 10,
    charm = 30,
    wand = 20,
    food = 5,
}

function getItemPrice(item, increment, addon)
    local type = item.itemType or item.type
    
    local price = prices[type]
    local inc = priceIncrements[type] or 0
    
    local val = item.price or price
    
    if item:isRevealed() then
        val = val + inc*(item.grade-1)
        
        val = val + inc*(item.utility)
    end
    
    val = val * increment
    val = val + addon
    
    return lume.max(val, 1)
end
    
shopItemPools = {
    general = {
            wand = {
                weight = 2,
                item = getRandomWand,
                enchants = {0,0,0,1,1,1,1,2},
            },
            
            weapon = {
                weight = 10,
                durability = {30,60},
                enchants = {-1,-1,0,0,0,2,1,1},
                buffs = {false, false, false, "random"},
                item = {"axe", "dagger", "sword"}
            },
            
            potions = {
                weight = 40,
                items = getRandomPotion--"bomb"
            },
            
            
            scrolls = {
                weight = 20,
                items = getRandomScroll--"bomb"
            },
                
            misc = {
                weight = 20,
                item = {"bomb"}
            },
            
            seed = {
                weight = 5,
                items = getRandomSeed,
            }
        },
    
    chemist = {
            potions = {
                weight = 40,
                items = getRandomPotion--"bomb"
            },
            
            misc = {
                weight = 30,
                item = {"bomb"}
            },
            
            seed = {
                weight = 15,
                items = getRandomSeed,
            }
    },
    
    library = {
            
            
            scrolls = {
                weight = 20,
                items = getRandomScroll--"bomb"
            },

        }
        
}

function tp() self:teleport(pl, kk:getRandomSpaceTile()) end


roomTypes = {}

roomTypes.foodRoom = {
    spawnItem = {
        matchstick = {3, amount},
        carrot = {1, {1,1,1,2,3}},
        stone = {4,{3,5}},
        stick = {2,{1,1,2}},
        anvil = {1}
    }
}

roomTypes.monsterRoomEasy = {
    spawnCreature = 3, -- intensity,
    spawn = { -- extra
        --{"pig",1}
    }
}

roomTypes.monsterRoomHard = {
    spawnCreature = 6, -- intensity,
    spawn = {
        --{"pig",1}
    }
}


function getItemByWeight(t)
  local sum = 0
  for _, v in pairs(t) do
    -- assert(v >= 0, "weight value less than zero")
    sum = sum + v[1]
  end
  assert(sum ~= 0, "all weights are zero")
  local rnd = lume.random(sum)
  for k, v in pairs(t) do
    if rnd < v[1] then return k end
    rnd = rnd - v[1]
  end
end

local function getItemTableByWeight(t)
  local sum = 0
  for _, v in pairs(t) do
    assert(v.weight, string.format("item %s needs weight!", k))
    -- assert(v >= 0, "weight value less than zero")
    sum = sum + v.weight
  end
  assert(sum ~= 0, "all weights are zero")
  local rnd = lume.random(sum)
  for k, v in pairs(t) do
    if rnd < v.weight then return k end
    rnd = rnd - v.weight
  end
end

local getItemByWeight = getItemByWeight

local iBuffs = {
    "blasphemy", "knockback",
    "burn",
}

local function getRandomBuff()
    return getValue(iBuffs)
end

function Map:spawnItemFromPool(pool, tile)
    -- if itemNames[pool] and not self.dungeon.itemPool[pool] then
    
    -- end
    
    local pool = self.dungeon and self.dungeon.itemPool[pool] or shopItemPools[pool] or pool
    
    local itemName
    local itemData
    while not itemData do
        itemName = getItemTableByWeight(pool)
        itemData = pool[itemName]
    
        itemData.ogMax = itemData.ogMax or itemData.max
        if itemData.max then
            itemData.max = itemData.max - 1
            if itemData.max < 0 then
                itemData = nil
            end
        end
    end
    
    return self:spawnItemFromData(itemData, itemName, tile)
end

function Map:spawnItemFromData(itemData, itemName, tile)

    local item = type(itemData.item) == "function" and itemData.item(self) or type(itemData.items) == "function" and itemData.items(self) or itemData.itemsByWeight and lume.weightedchoice(itemData.itemsByWeight) or getValue(itemData.item or itemData.items or itemName)
    
    local enchants = getValue(itemData.enchants or 0)
    local durabilityPercentage = getValue(itemData.durabilityPercentage or 100)
    local buff = getValue(itemData.buff or itemData.buffs)
    local buff2 = getValue(itemData.buff2)
    local curse = getValue(itemData.curse or itemData.curses)
    local curse2 = getValue(itemData.curse2)
    
    log("[DUNGEON MAKER ITEM SPAWNING] Spawning "..tostring(item))
    
    local itemObject = Item:new(items[item] or item)
    
    
    if enchants ~= 0 then
        itemObject:enhance(enchants)
    end

    if buff and enchants > 0 then
        if buff == "random" then
            buff = getRandomBuff()
        end
        
        itemObject:enchant("burn",1)
        
        if buff2 then
            if buff2 == "random" then
                buff2 = getRandomBuff()
            end
            itemObject:enchant(buff2)
        end
    end
    
    
    if curse and enchants ~= 0 then
        if curse == "random" then
            curse = getRandomBuff()
        end
        itemObject:enchant(buff, nil, true)

        if curse2 then
            if curse2 == "random" then
                curse2 = getRandomBuff()
            end
            itemObject:enchant(buff2, nil, true)
        end
    end
    
    return self:spawnItem(itemObject, tile)
end
    
local function updateTable(old, new)
    for x, i in pairs(new) do
        old[x] = i
    end
    
    return old
end
allMaps= {}
function Map:getDMap(goal, goals, ogTile)
    local m = {}
    
    if ogTi5le then local nn = 10
        for x = -nn,nn do
            for y = -nn, nn do
                local t = self:getTile(ogTile._x+x, ogTile._y+y)
                if t and not t.solid then
                    m[t] = goals and goals[t] or t == goal and 0 or 99
                end
            end
        end
    else
    
    for x = 1,#self.allTiles do
        local t = self.allTiles[x]
        if not t.solid then
            m[t] = goals and goals[t] or t == goal and 0 or 99
       -- elseif t.door then error()
        end
    end
    
    end
    
    

    local changed
    repeat
    
        changed = false
        for tile, v in pairs(m) do
            local lowest, ne
            for x = 1, 4 do
                local d = dirs4[x]
                local neigh = self:getTile(tile._x+d[1], tile._y+d[2])
                local val = neigh and m[neigh]
                if val and (not lowest or val < lowest) then
                    lowest = val
                    ne = neigh
                end
            end
            
            if lowest and (v-lowest)>1 then
                m[tile] = lowest + 1
                changed = true
            end
        end
        
    until not changed
    
    --[[local tt = ""
    for y = self.minTileY, self.maxTileY do
        tt = tt.."\n"
        for x = self.minTileX, self.maxTileX do
            local t = self:getTile(x, y)
            if t and m[t] then if m[t] >=99 then self.player.w=tw*5 end
                tt = tt.."["..(m[t]<10 and "0" or "")..m[t].."]"
            else
                tt = tt .. "    "
            end
        end
    end]]
    allMaps[goal] = m
    return m
end

function Map:build(dungeonData)

    -- self:after(1,function() warn(self:getDMap(self.player.tile,nil and {[self:getRandomSpaceTile()]=0, [self:getRandomSpaceTile()]=-1})) end)
    self.dungeon = lume.copy(dungeonData)
    self.dungeon.creatureData = lume.deepcopy(dungeonData.creatureData)
    
    randomizeOrcGoblinRatio()
    local cd = self.dungeon.creatureData
    if cd.goblin and cd.orc then
        cd.goblin[1] = cd.goblin[1]*GOBLIN_PORTION
        cd.orc[1] = cd.orc[1]*ORC_PORTION
    end
    
    self.dungeon.spawn = self.dungeon.spawn or self.dungeon.creatures
    self.dungeon.creatures = self.dungeon.spawn
    
    self.roomAmount = getValue({dungeonData.minRooms or 8, dungeonData.maxRooms or 20})
    local rooms = {}
    
    local specialRooms = {}
    local spLvl = dungeonData.specialRoomLevel or 1
    for x, i in pairs(_G.rooms) do
        if ((i.minRoomLevel or i.roomLevel or 1) <= spLvl) and ((i.maxRoomLevel or 5) >= spLvl) then
           -- specialRooms[x] = i.chanceToSpawn or i.weight or 0
        end
    end
    
    local rt = self.rooms[#self.rooms].roomTypes
    local st = self.rooms[1].roomTypes
    self.rooms[1].startingRoom = 1
    self.rooms[#self.rooms].endingRoom = true
    st.startingRoom = true
    rt.endingRoom = true
    rt.lastRoom = true
    
    
    local r = self.rooms[#self.rooms]
    -- self:after(2,function() self:teleport(self.player,r:getRandomSpaceTile()) end)
    local t = r:getRandomSpaceTile()
    self:makeStaircase(t)
    t.nextFloor = true
    self.nextFloorTile = t
    
    if self.hasUpstairs then
        local r = self.rooms[1]
        local t = r:getRandomSpaceTile()
        self:makeStaircase(t)
        t.previousFloor = true
        self.previousFloorTile = t
        self.previousTile = t
    end
        
    
    while self.roomAmount > 0 do
        for _roomName, dat in ipairs(dungeonData.roomTypes) do
            local roomName = dat.name
            local spName
            local special
            
            if (dat.specialRoom and dat.specialRoom~=true) or dat.specialRooms then
                local dspecialRooms = dat.specialRooms
                if type(dat.specialRoom) ~= "boolean" and not dspecialRooms then
                    dspecialRooms = dat.specialRoom
                end
                
                spName = dspecialRooms and
                 (
                    (dspecialRooms[1] or type(dspecialRooms) == "string") and
                        getValue(dat.specialRooms) or
                    lume.weightedchoice(dspecialRooms)
                )
                 or nil --lume.weightedchoice(specialRooms)
                 
                special = _G.rooms[spName]
                if not spName then error(inspect(dat.specialRoom)) end
                assert(special, spName)
                
                if special then
                    special.name = special.name or spName
                end
                
                log("[DUNGEON] SPECIAL ROOM "..(dspecialRooms and "(dRooms) " or "(?) ")..spName)
            end
            
            local room = rooms[roomName]
            
            if not room then
                room = lume.copy(roomTypes[dat.roomType or roomName] or _G.rooms[roomName] or dat)
                room.name = room.name or roomName
                updateTable(room, dat)
            end
            
            if not room.name then
                error("No room name(?) at "..tostring(roomName))
            end
            
            rooms[roomName] = room
            room.chanceToSpawn = room.chanceToSpawn or 50
            room.amount = math.random(room.min or 1, room.max or 20)
            
            room.built = 0
            room.builtIn = {}
            
            
            for am = 1, (room.min or 0) do
                self:buildRoom(room, nil, special)
            end
            
            local totalRooms = #self.rooms
            
            for am = 1, #self.rooms do
                if room.max and room.max < 1 then
                    break
                end
                
                if room.amount <= 0 then
                    log("[DUNGEON MAKER ABORTING] Max Rooms "..room.name)
                    break
                end
                
                if math.random(100) <= room.chanceToSpawn then
                    self:buildRoom(room, self.rooms[am], special)
                end
                
                local pp = ((room.built/totalRooms)*100)
                if room.maxPercent and pp >= room.maxPercent then
                    break
                end
                
                if am == totalRooms and room.minPercent and pp < room.minPercent then
                    self:buildRoom(room, nil, special)
                end
            end
        end
    end
    
    local maxGold = 150
    local goldValues = {50, 25}
    local goldAmount = {1,1,1,2,3,2}
    local rooms = lume.shuffle(self.rooms)
    for am = 1, #self.rooms do
        local r = rooms[am]
        local amount = getValue(goldAmount)
        local divide = math.random()>.2
        for xx = 1, amount do
            local val = math.floor(lume.min(maxGold, getValue(goldValues)/(divide and amount or 1)))
            maxGold = maxGold - val
        
            local gold = self:spawnItem("gold", r:getRandomSpaceTile())
            if gold then
                gold.value = val
            else
                maxGold = maxGold + val
            end
        
            
            if maxGold <= 0 then
                break
            end
        end
        
            
        if maxGold <= 0 then
            break
        end
    end
    
    
    if self.dungeon.traps then
        local room = self.dungeon
        
        local trap = lume.weightedchoice(room.traps.trap)
        for x = 1, math.random(room.traps.min, room.traps.max) do
            local pos
            local count = 0
            while not pos do
                if count > 50 then
                    error("Can't get a room for traps!")
                end
                
                pos = self:getRandomSpaceTile().roomData
                if pos.noTraps then
                    pos = nil
                end
            end
            
            Trap.loadTrap(traps[trap] or trap, pos)
        end
    end
    
    for x = 1, 2 do
    
        local t = x == 1 and self.nextFloorTile or self.previousFloorTile
        if t and t.pedestal and t.item then
            t.pedestal = nil
            t.pLock = nil
            t.pedestalLocked = nil
            log("====[DUNGEON FIXED STAIRS PEDESTAL]====")
            self:destroy_instance(t.item)
            t.item = nil
        end
    
    end
end

function Map:makeStaircase(tile, basic)
    tile.stairs = true
    
    for x = 1, basic and 0 or 8 do
        local d = dirs8[x]
        local xx, yy = d[1], d[2]
        for y = 1, 2 do
            local r = self:getTile(tile._x+xx*y, tile._y+yy*y)
            if r then
                r.solid = nil
                r.isCorridor = true
                
                if xx == -1 and yy == 0 then
                    r.isCorridor = false
                    r.solid = true
                    r.keepSolid = true
                end
            end
        end
    end
end

function roomd()
    toybox.room:addLog("doors are: "..#pl.tile.roomData.doors)
end

function Map:bcuild(dungeonData)
    local ddata = dungeonData
    if ddata.nl then end
end 

function Map:buildRoom(room, ppos, special, parentRoom)
    log(string.format("[DUNGEON MAKER] making room %s %s", room.name, special and special.name or ""))
    
    if self.roomAmount <= 0 then
        log("    Aborting: no rooms left")
        return
    end
    
    room.chanceToSpawn = room.chanceToSpawn or 50
    room.amount = room.amount or math.random(room.min or 1, room.max or 20)
            
    room.built = room.built or 0
    room.builtIn = room.builtIn or {}
            
    local pos
    local posCount = #self.rooms*2
    while not pos and posCount >= 0 do
        pos = ppos or getValue(self.rooms)
        posCount = posCount - 1
        
        if room.builtIn[pos] or (pos.startingRoom and self.first) then
            pos = nil
        end
        
        if pos and room.notRoom then
            for x, i in pairs(pos.roomTypes) do
                if room.notRoom[i.name] then
                    pos = nil
                    --break
                end
            end
        end
        
        
        if pos and (room.noSpawnInParent or room.noSpawnInParentRoom) and pos == parentRoom then
            pos = nil
        end
        
        if pos and room.checkRoom and not ppos then
            for x, i in pairs(room.checkRoom) do
                if pos and ((i and not pos.roomTypes[x]) or (not i and pos.roomTypes[x])) then
                    pos = nil
                    --break
                end
            end
        end
        
        if pos and pos.roomTypes.startingRoom and not (room.canBeInStarting or (room.checkRoom or {}).starting) and not ppos then
            pos = nil
        elseif pos and pos.startingRoom and not ppos then
            error(inspect(room,2))
        end
        
        if pos and room.maxDoors and #pos.doors>room.maxDoors and not pposs then
            pos = nil
        end
        
        if pos and room.minDoors and #pos.doors<room.minDoors and not ppeos then
            pos = nil
        end
        
        if pos == self.rooms[1] and (room.spawn or room.spawnCreature or room.level) and not ppos  then
            pos = nil
        end
        
        -- so a room specific layout that failed won't spawn outside of that room
        if ppos then
            if not pos and room.sp then
                -- error(room.name..inspect(room,2))
            end
            break
        end
    end
    
    if not pos then
        return
    end
    
    log("pos accepted with "..#pos.doors.." (min, max: "..(room.minDoors or "null")..","..(room.maxDoors or "null")..")"..(ppos and "prepos" or "no pre"))
    
    
    pos.parentRoom = parentRoom or pos.parentRoom
    
    if not room.name then
        error("room has no room name "..inspect(room,1))
    end
    
    pos.roomTypes[room.name] = true
    
    for x, i in pairs(room.tags or {}) do
        pos.roomTypes[x] = i
    end
    
    if room.spawnRooms then
        local failed
        for x, i in pairs(room.spawnRooms) do
            if math.random(100) <= (i.chance or 100) then
                if not i.data then
                    warn(string.format("[DUNGEON MAKER] Room %s has no data for spawning a mini room",i.name))
                end
                
                log("    [DUNGEON MAKER] Spawning child room "..((i.data or i.room).name or "?"))
                pos.builtRooms = self:buildRoom(i.data or i.room or i, nil, nil, pos)
                failed = not pos.builtRooms
            end
        end
        if failed then
            warn("[DUNGEON MAKER] Couldn't complete room "..room.name.." due to incomplete children rooms!")
            return
        end
    end
    
    if room.preMake then
        room.preMake(pos, self)
    end
    
    room.chanceToSpawn = (room.chanceToSpawn-(room.dwindleChance or 0))/(room.dwindleRatio or 1)
    room.amount = room.amount - 1
    
    self.roomAmount = self.roomAmount - (room.noCount and 0 or 1)
    room.built = room.built + 1
    room.builtIn[pos] = true
    
    if room.noTraps then
        pos.noTraps = room.noTraps
    end
  
    
   
    if room.mutate and not pos.roomTypes.noMutation then
        log("[DUNGEON MAKER] MUTATE")
        self.mutator[getValue(room.mutate)](self.mutator, pos)
    end
    
    -- spawn more complex items without a pool
    if room.itemData then
        local iData = room.itemData
        
        local amount = getValue(iData.amount or 1)

        local onPedestal = iData.onPedestal
        local pedestalLocked = iData.pedestalLocked
        local pLock
        
        if pedestalLocked then
            pLock = {unlocked=true}
        end
        
        for x = 1, amount do
            local tile = self:getSpaceTileFrom(pos)
            local item = self:spawnItemFromData(iData, nil, tile)
            
            if iData.postMake then
                iData.postMake(item)
            end
            
            if item and (type(onPedestal) == "number" and math.random(100) <= onPedestal or (onPedestal == true)) then
                tile.pedestal = true
                tile.pedestalLocked = pLock
                item.tilePedestal = tile
                
                if pLock then
                    pLock[tile] = item
                    item.pLock = pLock
                    tile.pLock = pLock
                end
                item.onPedestal = true
            end
        end
    end
    
    if room.spawnItem then
        local itemName = type(room.spawnItem) == "string" and room.spawnItem or getItemByWeight(room.spawnItem)
        local item = type(room.spawnItem) == "string" and room.spawnItem or room.spawnItem[itemName]
        
        local tile = pos:getRandomSpaceTile()
        if tile then
            local itemsVal = item.items
            if itemsVal then
                for x = 1, #itemsVal do
                    local itemVal = itemsVal[x]
                    local count = 1
                    
                    if type(itemVal) == "table" then
                        if type(itemVal[2]) ~= "number" then
                            itemVal = getValue(itemVal)
                        end
                    end
                    
                    if type(itemVal) == "table" and type(itemVal[2]) == "number" then
                        count = getValue(itemVal[2])
                        itemVal = itemVal[1]
                    end
                    
                    if type(itemVal) == "table" then
                        if type(itemVal[2]) ~= "number" then
                            itemVal = getValue(itemVal)
                        end
                    end
                    
                    for xx = 1, count do
                        local itemVal = itemVal
                        if type(itemVal) == "function" then
                            itemVal = itemVal(self)
                        end
                        self:spawnItem(itemVal, math.random()>.7 and self:getSpaceTileFrom(pos) or tile)
                    end
                end
            end
            
            for num = 1, getValue(itemsVal and 0 or item[2] or 1) do
                -- small chance to spawn further away
                self:spawnItem(itemName, math.random()>.7 and self:getSpaceTileFrom(pos) or tile)
            end
        end
    end
    
    if room.items then
        local iData = room.items
        local amount = getValue(iData.amount or 1)
        local pool = iData.pool or getValue(self.dungeon.itemPools)
        
        local onPedestal = iData.onPedestal
        local pedestalLocked = iData.pedestalLocked
        local pLock
        
        if pedestalLocked then
            pLock = {unlocked=true}
        end
        
        for x = 1, amount do
            local tile = self:getSpaceTileFrom(pos)
            local item = self:spawnItemFromPool(pool, tile)
            
            if iData.postMake then
                iData.postMake(item)
            end
            
            if item and (type(onPedestal) == "number" and math.random(100) <= onPedestal or (onPedestal == true)) then
                tile.pedestal = true
                tile.pedestalLocked = pLock
                item.tilePedestal = tile
                
                if pLock then
                    pLock[tile] = item
                    item.pLock = pLock
                    tile.pLock = pLock
                end
                item.onPedestal = true
            end
        end
    end
    
    
    if room.spawnCreature then
        local v = getValue(room.spawnCreature)
        local count = 20
        while v > 0 and count > 0 do
            local cr = getItemByWeight(self.dungeon.spawn)
            local sp = self.dungeon.spawn[cr]
            local creature = creatures[cr] or error("no such creature "..cr)
            local intensity = getValue(sp[3] or creature.intensity or 1)
            if v >= intensity then
                for num = 1, getValue(sp[2] or 1) do
                    local tile = pos:getRandomSpaceTile()
                    if v >= intensity and tile then
                        local team = sp[4]
                        local group = sp[5]
                        v = self:spawnCreature(creature, tile, getValue(team), v, intensity, group)
                    end
                end
            end
            
            count = count - 1
        end
        
    end
    
    if room.spawn then
        for xn, sp in pairs(room.spawn) do
            local sc = type(sp) ~= "table" and sp
            local count = sc and 1 or sp[2] or 1
            sc = sc or sp[1]
            
            if type(sc) == "number" then
                count = sc
                sc = xn
            end
            
            for spawns = 1, getValue(count) do
                local t = pos:getRandomSpaceTile()
                if t then
                    self:spawn(getValue(sc), t)
                end
            end
        end
    end
    
    if room.traps then
        local trap = lume.weightedchoice(room.traps.trap or room.traps.traps)
        for x = 1, math.random(room.traps.min or 1, room.traps.max or 1) do
            Trap.loadTrap(traps[trap] or room.traps[trap] or trap, pos)
        end
    end
    
    if room.shop then
        local s = room.shop
        local keeper = getValue(s.shopkeeper)
        local shopType = getItemTableByWeight(s.shopType)
        
        local pool = shopType.itemPool or "general"
        
        local amounts = {3,3,4,4,4,5}
        
        pos.shopkeeper = self:spawn(keeper, pos:getRandomSpaceTile())
        pos.shopkeeper.shop = pos
        pos.shopkeeper:addClass("shopkeeper")
        pos.shopkeeper.title = "The shopkeeper"
        
        for x = 1, getValue(shopType.amount or s.amount or amounts) do
        
            local count = 0
            local tile
            while not tile or tile.unit do
                tile = self:getSpaceTileFrom(pos)
                count = count + 1
                if count > 50 then
                    break
                end
            end
            
            local item = self:spawnItemFromPool(pool, tile)
            item.tile.pedestal = true
            log("pool %s",inspect(pool,1))
            item.price = getItemPrice(
                item,
                shopType.increment or s.increment or 1,
                shopType.addon or s.addon or 0
            )
            
            item.shopkeeper = pos.shopkeeper
            
            
        end
    end
    
    if room.level then
        self:spawnFromData(room.level, pos)
    end
    
    
    if room.postMake then
        room.postMake(pos, self)
    end
    
    if special then
        log("    [DUNGEON MAKER] MAKING SPECIAL ROOM]")
        special.sp = 1
        return self:buildRoom(special, pos)
    end
    
    return true
    
end    

function Map:getSpaceTileFrom(room)
    local t = room:getRandomSpaceTile()
    count = 50
    while t.isEntrance and count > 0 do
        t = room:getRandomSpaceTile()
        count = count - 1
    end
    
    return t
end

local function leadership(a, b)
    return a.leaderLevel > b.leaderLevel
end

function Map:spawnFromData(lvl,pos,cr,da)
    if type(lvl) == "string" then
        local l = lvl
        lvl = cr or nil
        cr = l
    end
    
    local intensity = lvl or self.dungeon.intensity[lvl] or 1
    local count = 100
    local acreatures = {}
    
    while intensity > 0 and count  > 0 do
        count = count - 1
        local index = da or getValue(cr) or getItemByWeight(self.dungeon.creatureData)
        local cr = da or self.dungeon.creatureData[index]
        if not cr then
            warn("No creature as "..index.." in this level!")
            return
        end
        
        for amount = 1, (cr.amount or 1) do
        
            local creatures = {}
            for x = 1, (getValue(cr.group) or 1) do
      
                local int = getValue(cr.intensity) or 1
                local level = (cr.level or 1)+int-1
        
                local name = x~=1 and getValue(cr.groupCreatures) or getValue(cr.name) or index
        
        
                local class
                -- todo check for other species ????
                if cr.class then
                    class = getItemByWeight(cr.class)
                    level = getValue(cr.class[class][2] or 1)+level-1
                    name = getValue(cr.class[class][3] or name)
                end
        
                local c = self:spawn(name, pos:getRandomSpaceTile())
                
                if c then
                    creatures[#creatures+1] = c
                    acreatures[#acreatures+1] = c
        
                    if cr.class then
                        c:addClass(class, level)
                    end
    
                    intensity = intensity - (amount == 1 and level or 0)
                end
            end
        
            local c = #creatures
            if c > 1 and cr.leader then
                table.sort(creatures, leadership)
                local leader = table.remove(creatures, 1)
                for x = 1, c - 1 do
                    creatures[x]:makeLeader(leader)
                end
            end
        end
    end
    
    return acreatures
end
    

local gSize = {1,2,2,3}

function Map:spawnCreatureGroup(creature, tile, team, group, noLeader)
    local c = self:spawn(creature, tile, team)
    local creatures = {c}
    
    local group = getValue(group or creature.group or 1)
    
    if group ~= 1 then
        local groupSize = type(group) == "number" and group or getValue(self.dungeon.groupSize or gSize)
        
        for num = 1, groupSize do
            local s = "none"
            local count = 100
            while not s and count > 0 do
                s = creature.spawn and lume.weightedchoice(creature.spawn) or creature._id
                count = count - 1
            end
            
            local sp = self.dungeon.spawn[s]
            if sp then
                local t = self:getFreeTileAround(tile)
                if t then
                    local c = self:spawn(s, t, team)
                    creatures[#creatures+1] = c
                end
            end
            
        end
    
    end
    
    local c = #creatures
    if c > 1 and not noLeader then
        table.sort(creatures, leadership)
        local leader = table.remove(creatures, 1)
        for x = 1, c - 1 do
            leader:addAlly(creatures[x])
        end
    end
end

function Map:spawnCreature(creature, tile, team, totalIntensity, intensity, group)
    local c = self:spawn(creature, tile, team)
    local creatures = {c}
    totalIntensity = totalIntensity - intensity
    
    if totalIntensity <= 0 then
        return totalIntensity
    end
    
    local group = getValue(group or creature.group or 1)
    
    if group ~= 1 then
        local groupSize = type(group) == "number" and group or getValue(self.dungeon.groupSize or gSize)
        
        for num = 1, groupSize do
            local s = "none"
            local count = 100
            while not self.dungeon.spawn[s] and count > 0 do
                s = creature.spawn and lume.weightedchoice(creature.spawn) or creature._id
                count = count - 1
            end
            
            local sp = self.dungeon.spawn[s]
            if sp then
                local t = self:getFreeTileAround(tile)
                if t then
                    local c = self:spawn(s, t, team)
                    creatures[#creatures+1] = c
                    totalIntensity = totalIntensity - (sp[3] or _G.creatures[s].intensity or 1)
                    if totalIntensity <= 0 then
                       -- break
                    end
                end
            end
            
        end
    
    end
    
    local c = #creatures
    if c > 1 then
        table.sort(creatures, leadership)
        local leader = table.remove(creatures, 1)
        for x = 1, c - 1 do
            leader:addAlly(creatures[x])
        end
    end
    
    
    return totalIntensity
end


function Map:loadMapData(data)
    self.spawnTiles = {}
    self.letterTiles = {}
    
    for y = 1, #data do
        local k = data[y]
        local l = #k
        for x = 1, l do
            local ti = string.sub(data[y],x,x)
            local text = ti
            
            local isSolid = ti == "#"
            local isCorridor = ti == "*"
            
            local t = self:getTile(x,y) or self.Tile:new({
                _x = x,
                _y = y,
                x = (x-1)*tw,--nn,
                y = (y-1)*th,
                solid = isSolid
            })
            
            -- t.space = v ~= 1
            t.isCorridor = isCorridor
            t.isEntrance = isCorridor
            -- t.isEntrance = v == 2
            
            --if v ~=0 and v~=1 then error() end
            
            t.edge = t._x == 1 or t._y == 1 or t._x == l or t._y == #data
        
            -- tv==1
            if t.solid and t.space and (self.caving and not t.realRoom or not self.caving) 
            and not t.isCorridor then
                -- t.solid = v==1--false
            end
            
            if v==0 and not t.keepSolid then
                t.solid = false
            end
            
            
            if t.edge then
                t.solid = true
            end
            
            if text == "+" then
                t:makeDoor()
            end
            
            if text == "D" then
                t:makeDoor(true)
            end
            
            if text == "x" then
                self.spawnTiles[#self.spawnTiles+1] = t
            end
            
            if text == ">" then
                self:makeStaircase(t, true)
                t.nextFloor = true
                self.nextFloorTile = t
            end
            
            if text == "_" then
                t.chasm = true
            end
            
            
            if text == "<" then
                log("made prv. floortile")
                self:makeStaircase(t, true)
                t.previousFloor = true
                self.previousFloorTile = t
                self.previousTile = t
            end
            
            self.letterTiles[text] = self.letterTiles[text] or {}
            table.insert(self.letterTiles[text], t)
            t.letter = text
            
            self:storeTile(t)
            t.roomData = self
        end
    end
    
    local tiles = lume.copy(self.allTiles)
    local IDs = {}
    
    while #tiles > 0 do
        local t = lume.eliminate(tiles)
        if not t.solid and not t.door and not IDs[t] then
            local roomTiles, doors = checkRoomFloodFill(t, nil, IDs)
            local ro = Room:new({
                parent = self,
                data = room,
                noLoadTiles = true,
                real = true, math.random()>.9,--true
            })
            self.rooms = self.rooms or {}
            self.rooms[#self.rooms+1] = ro
            
            for xx = 1, #roomTiles do
                ro:loadTile(roomTiles[xx], ro.kwargs)
            end
        end
    end
end