--[[
Manages the creation of noises/sound made by the player for
crawlers to track.
]]

local NoiseManager = class:extend("NoiseManager")

function NoiseManager:__init__(kwargs)
    self.map = kwargs.map or kwargs.room or toybox.room
    self.increment = kwargs.increment or .5
    self.noises = {}
    self.sources = {}
end

local map, nm, currentNoise, sx, sy, maxDis, noiseID, currentIncrement
local function addNoise(x,y)
    tile = map:getTile(x,y)
    if not tile then
        return
    end
    
    tile.noises = tile.noises or {}
    local dis = lume.distance(x,y,sx,sy)+1
    
    if not tile.noises[currentNoise] then
        table.insert(nm.noises[currentNoise],tile)
    end
    
    tile.noises[currentNoise] = tile[currentNoise] or {[4]=currentNoise}
    tile.noises[currentNoise][1] = (tile.noises[currentNoise][1] or 0)+maxDis/dis--noiseValue
    tile.noises[currentNoise][2] = currentIncrement
    tile.noises[currentNoise][3] = noiseID
    
    
    --debug--??
    if doDebug then
    tile.light = (tile.noises[currentNoise][1])/maxDis
    tile.color = getColor("red",tile.light)
    tile.maxnDis = maxDis
    end
end

function nes(n) self.noise:addNoise("pl",10,self.player,n) end
function NoiseManager:addNoise(n,val,x,y,increment)

    --map is not ready
    if not self.map.fov then
        return
    end
    
    if type(x) == "table" then
        increment = y
        if x.tile then
            y = x.tile._y
            x = x.tile._x
        else
            y, x = x._y, x._x
        end
    end
    
    -- rate at which noise goes down
    currentIncrement = increment or self.increment
    
    noiseID = lume.uuid()
    
    self.noises[n] = self.noises[n] or {}
    self.sources[noiseID] = self.map:getTile(x,y)
    map = self.map
    nm = self
    currentNoise = n
    sx, sy = x,y
    maxDis = lume.distance(x,y,x+val,y+val)
    self.map.fov:compute(x,y,val+1,addNoise)
    
    if val > 2 then
        map:shout(map:getTile(x,y), lume.max(val-1,1)*tw)
    end
    
    map, nm, currentNoise, maxDis, sx, sy, noiseID, currentIncrement = nil
end

function NoiseManager:giveInfo()
    local ss = ""
    for x, i in pairs(self.noises) do
        ss = ss..x..":\n    "
        for vx, t in ipairs(i) do
            ss = ss..tostring(t.light).." "..tostring(t.maxnDis).."\n    "
            ss = ss..inspect(t.noises[x]).."\n"
        end
    end
    return ss
end

function NoiseManager:getSource(noise)
    local noiseID = noise[3]
    local source = self.sources[noiseID]
    if not source then
        warn("Source of a noise is missing!")
    end
    return source
end

function NoiseManager:update(dt)
    for n, s in pairs(self.noises) do
        local ls = #s
        if ls<1 then
            self.noises[n] = nil
        else
            for l, tile in ipairs(s) do
                --local tile = s[l]
                
                if tile.noises[n] then

                    tile.noises[n][1] = tile.noises[n][1]-dt*tile.noises[n][2]
                    local l = tile.noises[n][1]
                    if doDebug then
                    tile.color = getColor("red",l/tile.maxnDis)
                    end
                end
                
                if not tile.noises[n] or tile.noises[n][1]<0 then
                    if tile.noises then
                        local noiseID = tile.noises[n][3]
                        if tile == self.sources[noiseID] then
                            self.sources[noiseID] = nil
                        end
                    end
                    tile.noises[n] = nil
                    lume.remove(s,tile)
                    tile.color = nil
                    tile.light = nil
                end
            end
        end
    end
end

return NoiseManager